/******************************************************************************* 
 *  Copyright 2007 Amazon Technologies, Inc.  
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  
 *  You may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at: http://aws.amazon.com/apache2.0
 *  This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
 *  CONDITIONS OF ANY KIND, either express or implied. See the License for the 
 *  specific language governing permissions and limitations under the License.
 * ***************************************************************************** 
 *    __  _    _  ___ 
 *   (  )( \/\/ )/ __)
 *   /__\ \    / \__ \
 *  (_)(_) \/\/  (___/
 * 
 *  Amazon LS CSharp Library
 *  API Version: 2007-06-05
 *  Generated: Wed Nov 14 17:05:49 PST 2007 
 * 
 */

using ImageNexus.BenScharbach.ICLTools.Amazon.LS.Amazon.LS.Model;

namespace ImageNexus.BenScharbach.ICLTools.Amazon.LS.Amazon.LS
{


    /// <summary>
    /// The LS web service allows DevPay clients to interact with the AWS Auth System.
    /// DevPay enabled products can retrieve new identity components for an end user
    /// and product subscription information.
    /// 
    /// </summary>
    public interface  AmazonLS {
    
                
        /// <summary>
        /// Activate Desktop Product 
        /// </summary>
        /// <param name="action">Activate Desktop Product  action</param>
        /// <returns>Activate Desktop Product  Response from the service</returns>
        /// <remarks>
        /// Allows a distributed devpay product to retrieve an HMAC key pair and user token that can be subsequently used for an end user against other AWS services.
        ///   
        /// </remarks>
        ActivateDesktopProductResponse ActivateDesktopProduct(ActivateDesktopProduct action);

                
        /// <summary>
        /// Activate Hosted Product 
        /// </summary>
        /// <param name="action">Activate Hosted Product  action</param>
        /// <returns>Activate Hosted Product  Response from the service</returns>
        /// <remarks>
        /// Allows a controlled devpay product to retrieve a user token. Note this is an authenticated request, and as such it must be signed.
        ///   
        /// </remarks>
        ActivateHostedProductResponse ActivateHostedProduct(ActivateHostedProduct action);

                
        /// <summary>
        /// Verify Product Subscription By Tokens 
        /// </summary>
        /// <param name="action">Verify Product Subscription By Tokens  action</param>
        /// <returns>Verify Product Subscription By Tokens  Response from the service</returns>
        /// <remarks>
        /// Allows a product to determine if the user represented by the UserToken still has an active subscription.  Note this is an authenticated request, and as such it must be signed.
        ///   
        /// </remarks>
        VerifyProductSubscriptionByTokensResponse VerifyProductSubscriptionByTokens(VerifyProductSubscriptionByTokens action);

                
        /// <summary>
        /// Verify Product Subscription By Pid 
        /// </summary>
        /// <param name="action">Verify Product Subscription By Pid  action</param>
        /// <returns>Verify Product Subscription By Pid  Response from the service</returns>
        /// <remarks>
        /// Allows a PersistentIdentifier owner to determine if the user represented by the PersistentIdentifier still has an active subscription.  Note this is an authenticated request, and as such it must be signed.
        ///   
        /// </remarks>
        VerifyProductSubscriptionByPidResponse VerifyProductSubscriptionByPid(VerifyProductSubscriptionByPid action);

                
        /// <summary>
        /// Get Active Subscriptions By Pid 
        /// </summary>
        /// <param name="action">Get Active Subscriptions By Pid  action</param>
        /// <returns>Get Active Subscriptions By Pid  Response from the service</returns>
        /// <remarks>
        /// Gets all products that an end user is currently subscribed to for products owned by the requesting party.  Note this is an authenticated request, and as such it must be signed.
        ///   
        /// </remarks>
        GetActiveSubscriptionsByPidResponse GetActiveSubscriptionsByPid(GetActiveSubscriptionsByPid action);

    }
}