/******************************************************************************* 
 *  Copyright 2007 Amazon Technologies, Inc.  
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  
 *  You may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at: http://aws.amazon.com/apache2.0
 *  This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
 *  CONDITIONS OF ANY KIND, either express or implied. See the License for the 
 *  specific language governing permissions and limitations under the License.
 * ***************************************************************************** 
 *    __  _    _  ___ 
 *   (  )( \/\/ )/ __)
 *   /__\ \    / \__ \
 *  (_)(_) \/\/  (___/
 * 
 *  Amazon LS CSharp Library
 *  API Version: 2007-06-05
 *  Generated: Wed Nov 14 17:05:49 PST 2007 
 * 
 */

using System;
using System.IO;
using System.Reflection;
using System.Xml.Serialization;
using ImageNexus.BenScharbach.ICLTools.Amazon.LS.Amazon.LS.Model;

namespace ImageNexus.BenScharbach.ICLTools.Amazon.LS.Amazon.LS.Mock
{

    /// <summary>
    /// AmazonLSMock is the implementation of AmazonLS based
    /// on the pre-populated set of XML files that serve local data. It simulates 
    /// responses from Amazon LS service.
    /// </summary>
    /// <remarks>
    /// Use this to test your application without making a call to 
    /// Amazon LS 
    /// 
    /// Note, current Mock Service implementation does not valiadate requests
    /// </remarks>
    public  class AmazonLSMock : AmazonLS {
    

        // Public API ------------------------------------------------------------//
    
        
        /// <summary>
        /// Activate Desktop Product 
        /// </summary>
        /// <param name="action">Activate Desktop Product  action</param>
        /// <returns>Activate Desktop Product  Response from the service</returns>
        /// <remarks>
        /// Allows a distributed devpay product to retrieve an HMAC key pair and user token that can be subsequently used for an end user against other AWS services.
        ///   
        /// </remarks>
        public ActivateDesktopProductResponse ActivateDesktopProduct(ActivateDesktopProduct action) 
        {
            return Invoke<ActivateDesktopProductResponse>("ActivateDesktopProductResponse.xml");
        }
        
        /// <summary>
        /// Activate Hosted Product 
        /// </summary>
        /// <param name="action">Activate Hosted Product  action</param>
        /// <returns>Activate Hosted Product  Response from the service</returns>
        /// <remarks>
        /// Allows a controlled devpay product to retrieve a user token. Note this is an authenticated request, and as such it must be signed.
        ///   
        /// </remarks>
        public ActivateHostedProductResponse ActivateHostedProduct(ActivateHostedProduct action) 
        {
            return Invoke<ActivateHostedProductResponse>("ActivateHostedProductResponse.xml");
        }
        
        /// <summary>
        /// Verify Product Subscription By Tokens 
        /// </summary>
        /// <param name="action">Verify Product Subscription By Tokens  action</param>
        /// <returns>Verify Product Subscription By Tokens  Response from the service</returns>
        /// <remarks>
        /// Allows a product to determine if the user represented by the UserToken still has an active subscription.  Note this is an authenticated request, and as such it must be signed.
        ///   
        /// </remarks>
        public VerifyProductSubscriptionByTokensResponse VerifyProductSubscriptionByTokens(VerifyProductSubscriptionByTokens action) 
        {
            return Invoke<VerifyProductSubscriptionByTokensResponse>("VerifyProductSubscriptionByTokensResponse.xml");
        }
        
        /// <summary>
        /// Verify Product Subscription By Pid 
        /// </summary>
        /// <param name="action">Verify Product Subscription By Pid  action</param>
        /// <returns>Verify Product Subscription By Pid  Response from the service</returns>
        /// <remarks>
        /// Allows a PersistentIdentifier owner to determine if the user represented by the PersistentIdentifier still has an active subscription.  Note this is an authenticated request, and as such it must be signed.
        ///   
        /// </remarks>
        public VerifyProductSubscriptionByPidResponse VerifyProductSubscriptionByPid(VerifyProductSubscriptionByPid action) 
        {
            return Invoke<VerifyProductSubscriptionByPidResponse>("VerifyProductSubscriptionByPidResponse.xml");
        }
        
        /// <summary>
        /// Get Active Subscriptions By Pid 
        /// </summary>
        /// <param name="action">Get Active Subscriptions By Pid  action</param>
        /// <returns>Get Active Subscriptions By Pid  Response from the service</returns>
        /// <remarks>
        /// Gets all products that an end user is currently subscribed to for products owned by the requesting party.  Note this is an authenticated request, and as such it must be signed.
        ///   
        /// </remarks>
        public GetActiveSubscriptionsByPidResponse GetActiveSubscriptionsByPid(GetActiveSubscriptionsByPid action) 
        {
            return Invoke<GetActiveSubscriptionsByPidResponse>("GetActiveSubscriptionsByPidResponse.xml");
        }

        // Private API ------------------------------------------------------------//

        private T Invoke<T>(String xmlResource)
        {
            XmlSerializer serlizer = new XmlSerializer(typeof(T));
            Stream xmlStream = Assembly.GetAssembly(this.GetType()).GetManifestResourceStream(xmlResource);
            return (T)serlizer.Deserialize(xmlStream);
        }
    }
}