// This software code is made available "AS IS" without warranties of any        
// kind.  You may copy, display, modify and redistribute the software            
// code either by itself or as incorporated into your code; provided that        
// you do not remove any proprietary notices.  Your use of this software         
// code is at your own risk and you waive any claim against Amazon               
// Digital Services, Inc. or its affiliates with respect to your use of          
// this software code. (c) 2006-2007 Amazon Digital Services, Inc. or its             
// affiliates.          


using System;
using System.Collections;
using System.Net;
using System.IO;

namespace ImageNexus.BenScharbach.ICLTools.S3Sample
{
    public class GetResponse : Response
    {
        private readonly S3Object _obj;
        public S3Object Object {
            get {
                return _obj;
            }
        }

        // Events
        public event EventHandler<WriteObjectProgressArgs> WriteObjectProgressEvent;

        // Methods
        internal void OnRaiseProgressEvent(long incrementTransferred, long transferred, long total)
        {
            EventHandler<WriteObjectProgressArgs> writeObjectProgressEvent = this.WriteObjectProgressEvent;
            try
            {
                if (writeObjectProgressEvent != null)
                {
                    writeObjectProgressEvent(this, new WriteObjectProgressArgs(incrementTransferred, transferred, total));
                }
            }
            catch
            {
            }
        }



        public GetResponse(WebResponse response)
            : base(response)
        {
            SortedList metadata = extractMetadata( response );
            byte [] data = Utils.SlurpInputStream( response.GetResponseStream() );
            _obj = new S3Object( data, metadata );
        }

        private static SortedList extractMetadata( WebResponse response )
        {
            var metadata = new SortedList();
            foreach ( string key in response.Headers.Keys ) {
                if ( key == null ) continue;
                if ( key.StartsWith( Utils.MetadataPrefix ) ) {
                    metadata.Add( key.Substring( Utils.MetadataPrefix.Length ), response.Headers[ key ] );
                }
            }
            return metadata;
        }

        // 4/13/2011
        public void WriteResponseStreamToFile(string filePath)
        {
            var info = new FileInfo(filePath);
            Directory.CreateDirectory(info.DirectoryName);
            Stream stream = new BufferedStream(File.OpenWrite(filePath));
            try
            {
                var transferred = 0L;
                var stream2 = new BufferedStream(new MemoryStream(Object.Bytes));
                var buffer = new byte[0x2000];
                var count = 0;
                while ((count = stream2.Read(buffer, 0, buffer.Length)) > 0)
                {
                    stream.Write(buffer, 0, count);
                    transferred += count;
                    OnRaiseProgressEvent((long)count, transferred, Object.Bytes.Length);
                }
            }
            finally
            {
                stream.Close();
            }
        }


    }
}
