// This software code is made available "AS IS" without warranties of any        
// kind.  You may copy, display, modify and redistribute the software            
// code either by itself or as incorporated into your code; provided that        
// you do not remove any proprietary notices.  Your use of this software         
// code is at your own risk and you waive any claim against Amazon               
// Digital Services, Inc. or its affiliates with respect to your use of          
// this software code. (c) 2006-2007 Amazon Digital Services, Inc. or its             
// affiliates.          


using System.Collections;
using System.Net;
using System.Xml;

namespace ImageNexus.BenScharbach.ICLTools.S3Sample
{
    public class ListAllMyBucketsResponse : Response
    {
        private readonly Owner _owner;
        public Owner Owner {
            get {
                return _owner;
            }
        }

        private readonly ArrayList _buckets;
        public ArrayList Buckets {
            get {
                return _buckets;
            }
        }

        public ListAllMyBucketsResponse( WebResponse response )
            : base(response)
        {
            _buckets = new ArrayList();
            var rawBucketXML = Utils.SlurpInputStreamAsString( response.GetResponseStream() );

            var doc = new XmlDocument();
            doc.LoadXml( rawBucketXML );
            foreach (XmlNode node in doc.ChildNodes)
            {
                if (!node.Name.Equals("ListAllMyBucketsResult")) continue;

                foreach (XmlNode child in node.ChildNodes)
                {
                    if (child.Name.Equals("Owner"))
                    {
                        _owner = new Owner(child);
                    }
                    else if (child.Name.Equals("Buckets"))
                    {
                        foreach (XmlNode bucket in child.ChildNodes)
                        {
                            if (bucket.Name.Equals("Bucket"))
                            {
                                _buckets.Add(new Bucket(bucket));
                            }
                        }
                    }
                }
            }
        }
    }
}
