// This software code is made available "AS IS" without warranties of any        
// kind.  You may copy, display, modify and redistribute the software            
// code either by itself or as incorporated into your code; provided that        
// you do not remove any proprietary notices.  Your use of this software         
// code is at your own risk and you waive any claim against Amazon               
// Digital Services, Inc. or its affiliates with respect to your use of          
// this software code. (c) 2006 Amazon Digital Services, Inc. or its             
// affiliates.          


using System;
using System.Xml;

namespace ImageNexus.BenScharbach.ICLTools.S3Sample
{
    public class ListEntry
    {
        private readonly string _key;
        public string Key {
            get {
                return _key;
            }
        }

        private readonly DateTime _lastModified;
        public DateTime LastModified {
            get {
                return _lastModified;
            }
        }

        private readonly string _etag;
        public string ETag {
            get {
                return _etag;
            }
        }

        private readonly long _size;
        public long Size {
            get {
                return _size;
            }
        }

        private readonly string _storageClass;
        public string StorageClass {
            get {
                return _storageClass;
            }
        }

        private readonly Owner _owner;
        public Owner Owner {
            get {
                return _owner;
            }
        }

        public ListEntry( string key,
                          DateTime lastModified,
                          string etag,
                          long size,
                          string storageClass,
                          Owner owner)
        {
            _key = key;
            _lastModified = lastModified;
            _etag = etag;
            _size = size;
            _storageClass = storageClass;
            _owner = owner;
        }

        public ListEntry(XmlNode node)
        {
            foreach (XmlNode child in node.ChildNodes)
            {
                if (child.Name.Equals("Key"))
                {
                    _key = Utils.GetXmlChildText(child);
                }
                else if (child.Name.Equals("LastModified"))
                {
                    var value = Utils.GetXmlChildText(child);
                    _lastModified = Utils.ParseDate(value);
                }
                else if ( child.Name.Equals("ETag" ) ) {
                    _etag = Utils.GetXmlChildText(child);
                }
                else if ( child.Name.Equals("Size" ) )
                {
                    _size = long.Parse( Utils.GetXmlChildText( child ) );
                }
                else if ( child.Name.Equals( "Owner" ) )
                {
                    _owner = new Owner( child );
                }
                else if ( child.Name.Equals( "StorageClass" ) )
                {
                    _storageClass = Utils.GetXmlChildText( child );
                }
            }
        }
    }
}
