// This software code is made available "AS IS" without warranties of any        
// kind.  You may copy, display, modify and redistribute the software            
// code either by itself or as incorporated into your code; provided that        
// you do not remove any proprietary notices.  Your use of this software         
// code is at your own risk and you waive any claim against Amazon               
// Digital Services, Inc. or its affiliates with respect to your use of          
// this software code. (c) 2006-2007 Amazon Digital Services, Inc. or its             
// affiliates.          


using System;
using System.Net;

namespace ImageNexus.BenScharbach.ICLTools.S3Sample
{
    public class Response : IDisposable
    {
        protected WebResponse _response;

        public WebResponse Connection
        {
            get
            {
                return _response;
            }
        }

        public HttpStatusCode Status
        {
            get
            {
                var wr = _response as HttpWebResponse;
                return wr.StatusCode;
            }
        }

        public string XAmzId
        {
            get
            {
                return _response.Headers.Get( "x-amz-id-2" );
            }
        }

        public string XAmzRequestId
        {
            get
            {
                return _response.Headers.Get("x-amz-request-id");
            }
        }

        public Response(WebResponse response)
        {
            this._response = response;
        }

        public String getResponseMessage()
        {
            string data = Utils.SlurpInputStreamAsString( _response.GetResponseStream() );
            _response.GetResponseStream().Close();
            return data;
        }

        public byte[] getResponseBytes()
        {
            byte[] data = Utils.SlurpInputStream(_response.GetResponseStream());
            _response.GetResponseStream().Close();
            return data;
        }

        ~Response()
        {
            Close();
        }

        public void Dispose() {
            Close();
        }

        public void Close() {
            if ( _response != null ) {
                _response.Close();
                _response = null;
            }
        }
    }
}
