// This software code is made available "AS IS" without warranties of any        
// kind.  You may copy, display, modify and redistribute the software            
// code either by itself or as incorporated into your code; provided that        
// you do not remove any proprietary notices.  Your use of this software         
// code is at your own risk and you waive any claim against Amazon               
// Digital Services, Inc. or its affiliates with respect to your use of          
// this software code. (c) 2006 Amazon Digital Services, Inc. or its             
// affiliates.          


using System;
using System.Collections;
using System.Text;

namespace ImageNexus.BenScharbach.ICLTools.S3Sample
{
    public class S3Object
    {
        private readonly byte [] _bytes;

        // 4/13/2011 - Taken from Amazon SDK
        // Events
        public event EventHandler<PutObjectProgressArgs> PutObjectProgressEvent;

        // 4/13/2011 - Taken from Amazon SDK
        internal void OnRaiseProgressEvent(long incrementTransferred, long transferred, long total)
        {
            var putObjectProgressEvent = PutObjectProgressEvent;
            try
            {
                if (putObjectProgressEvent != null)
                {
                    putObjectProgressEvent(this, new PutObjectProgressArgs(incrementTransferred, transferred, total));
                }
            }
            catch
            {
            }
        }

        /// <summary>
        /// Acquires the binary representation of an object.
        /// </summary>
        public byte [] Bytes {
            get {
                return _bytes;
            }
        }

        /// <summary>
        /// Acquires the ASCII Encoding representation of an object.
        /// </summary>
        public string Data {
            get {
                var encoder = new ASCIIEncoding();
                return encoder.GetString(_bytes, 0, _bytes.Length);
            }
        }

        private readonly SortedList _metadata;

        /// <summary>
        /// Acquires the metadata.
        /// </summary>
        public SortedList Metadata {
            get {
                return _metadata;
            }
        }

        /// <summary>
        /// Constructs an S3Object.
        /// </summary>
        /// <param name="bytes">Byte array representing the object</param>
        /// <param name="metadata">Metadata associated with the object</param>
        public S3Object( byte [] bytes, SortedList metadata ) {
            _bytes = bytes;
            _metadata = metadata;
        }

        /// <summary>
        /// Constructs an S3Object.
        /// </summary>
        /// <param name="data">String representation of the data; this will be decoded via ASCII</param>
        /// <param name="metadata">Metadata associated with the object</param>
        public S3Object(string data, SortedList metadata)
        {
            var encoder = new ASCIIEncoding();
            _bytes = encoder.GetBytes( data.ToCharArray() );
            _metadata = metadata;
        }
    }
}
