/******************************************************************************* 
 *  Copyright 2007 Amazon Technologies, Inc.  
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  
 *  You may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at: http://aws.amazon.com/apache2.0
 *  This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
 *  CONDITIONS OF ANY KIND, either express or implied. See the License for the 
 *  specific language governing permissions and limitations under the License.
 *******************************************************************************/

using System;

namespace ImageNexus.BenScharbach.ICLTools.CredentialStore.Security
{
    /// <summary>
    /// This class provides and interface and helper methods for
    /// credential stores.
    /// </summary>
    public abstract class AbstractCredentialStore
    {
        private readonly Encryption _enc;
        protected string Password;

        /// <summary>
        /// We assume that all we need is a password to the store.  Derived
        /// stores can receive what they need in addition.
        /// </summary>
        /// <param name="password">The password to the store.</param>
        protected AbstractCredentialStore(string password)
        {
            Password = password;
            _enc = new Encryption(Password);
        }

        public abstract void Store();
        public abstract bool Read();

        /// <summary>
        /// Encrypts.
        /// </summary>
        /// <param name="plainText">The paintext to encrypt.</param>
        /// <returns>The plaintext encrypted.</returns>
        protected String Encrypt(String plainText)
        {
            if (string.IsNullOrEmpty(plainText))
            {
                return "";
            }
            return _enc.Encrypt(plainText);
        }

        /// <summary>
        /// Decrypts.
        /// </summary>
        /// <param name="cipherText">The encrypted text.</param>
        /// <returns>The plaintext.</returns>
        protected String Decrypt(String cipherText)
        {
            if (string.IsNullOrEmpty(cipherText))
            {
                return "";
            }
            return _enc.Decrypt(cipherText);
        }
    }
}
