﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Threading;
using System.Windows.Media.Imaging;

namespace ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces
{
    // 4/16/2011
    /// <summary>
    /// Interface for a directoryItem type.
    /// </summary>
    public interface IDirectoryItem 
    {
        event EventHandler DirectoryAccessError;

        /// <summary>
        /// Gets the current directory name.
        /// </summary>
        string DirectoryName { get; }

        /// <summary>
        /// Gets the folder icon Open image.
        /// </summary>
        BitmapImage FolderIconOpen { get; }

        /// <summary>
        /// Gets the folder icon Closed image.
        /// </summary>
        BitmapImage FolderIconClosed { get; }

        /// <summary>
        /// Gets the full path of the directory.
        /// </summary>
        string FullName { get; }

        /// <summary>
        /// Gets the name of this directory instance.
        /// </summary>
        string Name { get; }

        /// <summary>
        /// Is directory password protected?
        /// </summary>
        bool IsPasswordProtected { get; }

        /// <summary>
        /// Returns the current instance sub-directory count.
        /// </summary>
        int GetDirectoriesCount { get; }

        /// <summary>
        ///  Gets the parent instance for this data item.
        /// </summary>
        IDirectoryItemViewModel Parent { get; set; }

        /// <summary>
        /// Returns a enumerable collection for the DirectoryInfoViewModel instances.
        /// </summary>
        /// <returns>Instance of <see cref="IEnumerable{DirectoryInfoViewModel}"/>.</returns>
        IEnumerable<IDirectoryItem> GetCurrentDirectoryChildDirectories();

        /// <summary>
        /// Returns a enumerable collection for the FileInfoViewModel instances.
        /// </summary>
        /// <returns>Instance of <see cref="IEnumerable{FileInfoViewModel}"/>.</returns>
        IEnumerable<IFileItem> GetCurrentDirectoryFiles(Func<FileInfo, bool> filterBy);

        /// <summary>
        /// Helper method used to obtain recursively the total count of all children items.
        /// </summary>
        /// <param name="cancellationToken">Instance of <see cref="CancellationToken"/>.</param>
        /// <param name="setProgressValue"></param>
        /// <returns>Total children count value.</returns>
        int GetTotalCountOfAllChildren(CancellationToken cancellationToken, Action<double> setProgressValue);

        // 8/14/2011
        /// <summary>
        /// Returns a collection of <see cref="List{IDirectoryItem}"/>.
        /// </summary>
        /// <returns>Collection of <see cref="List{IDirectoryItem}"/>.</returns>
        List<IDirectoryItem> GetDirectories();

        // 8/14/2011
        /// <summary>
        /// Returns a collection of <see cref="List{IFileItem}"/>.
        /// </summary>
        /// <returns>Collection of <see cref="List{IDirectoryItem}"/>.</returns>
        List<IFileItem> GetFiles();
    }
}