﻿using System;
using System.IO;

namespace ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces
{
    // 4/16/2011
    /// <summary>
    /// Interface for a fileItem type.
    /// </summary>
    public interface IFileItem
    {
        /// <summary>
        /// Gets the current <see cref="FileInfo"/>
        /// </summary>
        FileInfo FileInfo { get; }

        /// <summary>
        /// Gets the current file name.
        /// </summary>
        string FileName { get; }

        /// <summary>
        /// Gets the full path name for given file.
        /// </summary>
        /// <remarks>
        /// Simply an alias for the internal <see cref="FileSystemInfo.FullName"/> property.
        /// </remarks>
        string FullPathName { get; }

        /// <summary>
        /// Gets the current file name with extension.
        /// </summary>
        string FileNameWithExtension { get; }

        /// <summary>
        /// Gets the file's extension.
        /// </summary>
        string FileExtension { get; }

        /// <summary>
        /// Gets the directory's name.
        /// </summary>
        string DirectoryName { get; }

        /// <summary>
        /// Gets the file's last modified date string.
        /// </summary>
        DateTime FileLastModified { get; }

        /// <summary>
        /// Gets the current unique Key used in cloud.
        /// </summary>
        string CloudKey { get; }

        // 5/14/2011
        /// <summary>
        /// Was this item deleted from cloud succesfully?
        /// </summary>
        /// <remarks>
        /// This is checked when the ObservableCollection is called to remove
        /// all checked items.  This guarantees that ONLY the items succesfully removed
        /// from the cloud are removed from the WPF view.
        /// </remarks>
        bool DeletedOnCloud { get; set; }

        /// <summary>
        /// Gets the current parent <see cref="IFileItemViewModel"/> instance.
        /// </summary>
        IFileItemViewModel Parent { get; set; }

        /// <summary>
        /// Gets the current file size.
        /// </summary>
        long FileSize { get; }
    }
}