using System;
using System.Windows.Media.Imaging;

namespace ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces
{
    public interface IFileItemViewModel : ITreeViewItemViewModel
    {
        /// <summary>
        /// Gets the full TreeView's directory path, starting from root directory.
        /// </summary>
        /// <example>
        /// If actual directory is => "c:\Users\Ben\MyDocs\Game\GameContent\Textures\DesertTextures\texture1.png", and
        /// the TreeView is set to start at => "Textures" directory level, then this property would return the following
        /// result for the given example; "Textures\DesertTextures\texture1".
        /// </example>
        string TreeViewDirectoryPath { get; }

        /// <summary>
        /// Gets the current file name.
        /// </summary>
        string FileName { get; }

        /// <summary>
        /// Gets the current file name with extension.
        /// </summary>
        string FileNameWithExtension { get; }

        /// <summary>
        /// Gets the current file size.
        /// </summary>
        long FileSize { get; }

        /// <summary>
        /// Gets the current file's <see cref="DateTime"/> it was last modified.
        /// </summary>
        DateTime FileLastModified { get; }

        /// <summary>
        /// Gets the current Icon image for this file.
        /// </summary>
        BitmapSource FileIcon { get; }

        /// <summary>
        /// Gets the current <see cref="IFileItem"/>.
        /// </summary>
        IFileItem FileItem { get; }
    }
}