﻿using System;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.CheckedDataModel;

namespace ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.CloudDataModel.CheckedDataModel
{
    // 10/26/2010; 11/8/2010 - Updated to use Interface.
    /// <summary>
    /// The <see cref="FileCloudCheckedItem"/> is used to track files which are checked
    /// by the user in the WPF treeView control.
    /// </summary>
    public class FileCloudCheckedItem : IFileCheckedItem<IFileItem>
    {
        // 12/10/2010
        /// <summary>
        /// Gets if the current instance is checked.
        /// </summary>
        public bool IsChecked { get; set; }

        // 11/9/2010
        /// <summary>
        /// Stores a reference to the original Generic 'T' type item, used
        /// to gain access to this item.
        /// </summary>
        public IFileItem OriginalItem { get; private set; }

        /// <summary>
        /// Gets the name of the file.
        /// </summary>
        public string Name { get; private set; }

        /// <summary>
        /// Gets the full path of the file.
        /// </summary>
        public string FullName { get; private set; }
        
        /// <summary>
        /// Gets a string representing the directory's full path.
        /// </summary>
        public string DirectoryName { get; private set; }

        // 12/7/2010
        /// <summary>
        /// Gets or sets a number representing a specific schedule to use.
        /// </summary>
        public int ScheduleNumber { get; set; }

        /// <summary>
        /// Constructor
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public FileCloudCheckedItem(IFileItem fileCloudItem)
        {
            if (fileCloudItem == null)
                throw new ArgumentNullException("fileCloudItem");

            // populate properties
            Name = fileCloudItem.FileName;
            FullName = fileCloudItem.FullPathName;
            DirectoryName = fileCloudItem.DirectoryName;
            OriginalItem = fileCloudItem;
        }

        /// <summary>
        /// constructor
        /// </summary>
        public FileCloudCheckedItem()
        {
            // Empty
        }
    }
}
