﻿using System.IO;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.CheckedDataModel;

namespace ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.DesktopDataModel.CheckedDataModel
{
    // 10/26/2010; 11/8/2010 - Updated to use Interface.
    /// <summary>
    /// The <see cref="DirectoryCheckedItem"/> is used to track directories which are checked
    /// by the user in the WPF treeView control.
    /// </summary>
    public class DirectoryCheckedItem : IDirectoryCheckedItem<IDirectoryItem>
    {

        #region Properties

        // 12/10/2010
        /// <summary>
        /// Gets if the current instance is checked.
        /// </summary>
        public bool IsChecked { get; set; }

        // 11/9/2010
        /// <summary>
        /// Stores a reference to the original Generic 'T' type item, used
        /// to gain access to this item.
        /// </summary>
        public IDirectoryItem OriginalItem { get; set; }

        /// <summary>
        /// Gets the name of the directory.
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// Gets the full path of the directory. 
        /// </summary>
        public string FullName { get; set; }

        // 12/7/2010
        /// <summary>
        /// Gets or sets a number representing a specific schedule to use.
        /// </summary>
        public int ScheduleNumber { get; set; }

        // 12/9/2010
        /// <summary>
        /// Gets or sets if this directory is in indeterminate mode - 
        /// In other words, some children items are checked.
        /// </summary>
        /// <remarks>
        /// This is used in the UI to let the end-user know when some
        /// directory has items checked.
        /// </remarks>
        public bool IsIndeterminate { get; set; }

        #endregion

        /// <summary>
        /// constructor
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public DirectoryCheckedItem(IDirectoryItem directoryItem)
        {
            // populate properties
            Name = directoryItem.DirectoryName; //Name;
            FullName = directoryItem.FullName;
            OriginalItem = directoryItem;
        }

        /// <summary>
        /// constructor
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public DirectoryCheckedItem(DirectoryInfo directoryInfo)
        {
            // populate properties
            Name = directoryInfo.Name;
            FullName = directoryInfo.FullName;
        }

        /// <summary>
        /// Constructor
        /// </summary>
        public DirectoryCheckedItem()
        {
            // Empty
        }
    }
}
