﻿using System.IO;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.CheckedDataModel;

namespace ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.DesktopDataModel.CheckedDataModel
{
    // 10/26/2010; 11/8/2010 - Updated to use Interface.
    /// <summary>
    /// The <see cref="FileCheckedItem"/> is used to track files which are checked
    /// by the user in the WPF treeView control.
    /// </summary>
    public class FileCheckedItem : IFileCheckedItem<IFileItem>
    {
        // 12/10/2010
        /// <summary>
        /// Gets if the current instance is checked.
        /// </summary>
        public bool IsChecked { get; set; }

        // 11/9/2010
        /// <summary>
        /// Stores a reference to the original Generic 'T' type item, used
        /// to gain access to this item.
        /// </summary>
        public IFileItem OriginalItem { get; set; }

        /// <summary>
        /// Gets the name of the file.
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// Gets the full path of the file.
        /// </summary>
        public string FullName { get; set; }
        
        /// <summary>
        /// Gets a string representing the directory's full path.
        /// </summary>
        public string DirectoryName { get; set; }

        // 12/7/2010
        /// <summary>
        /// Gets or sets a number representing a specific schedule to use.
        /// </summary>
        public int ScheduleNumber { get; set; }

        /// <summary>
        /// Constructor
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public FileCheckedItem(IFileItem fileItem)
        {
            // populate properties
            Name = fileItem.FileName;
            FullName = fileItem.FullPathName; //FileInfo.FullName;
            DirectoryName = fileItem.DirectoryName;
            OriginalItem = fileItem;
        }

        /// <summary>
        /// Constructor
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public FileCheckedItem(FileInfo fileInfo)
        {
            // populate properties
            Name = fileInfo.Name;
            FullName = fileInfo.FullName;
            DirectoryName = fileInfo.DirectoryName;
        }

        /// <summary>
        /// constructor
        /// </summary>
        public FileCheckedItem()
        {
            // Empty
        }
    }
}
