﻿using System;
using System.IO;
using System.Windows.Media.Imaging;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;

namespace ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.DesktopDataModel.DriveDataModel
{
    // 7/2/2010
    /// <summary>
    /// The <see cref="DriveInfoItem"/> class holds a single instance of the <see cref="DriveInfo"/>.
    /// </summary>
    public class DriveInfoItem : IDriveItem
    {
        #region Fields

        private readonly DriveInfo _driveInfoI;

        // 7/4/2010 - Drive Icons
        private static bool _driveIconsLoaded;

        // 8/20/2011: Note: Removed, since can cause cross-thread errors if resource
        //            Note: Created on seperate thread.  Furthermore, the XAML icons
        //            Note: are now used instead of the old bitmap images. - Ben
        // 11/16/2010 - Converted to BitmapSource, instead of BitmapImage for transparency to work.
        //private static BitmapSource _driveIcon;

        #endregion

        #region Properties

        /// <summary>
        /// Returns the root directory as <see cref="System.IO.DirectoryInfo"/>.
        /// </summary>
        public DirectoryInfo DirectoryInfo
        {
            get
            {
                return _driveInfoI != null ? _driveInfoI.RootDirectory : null;
            }
        }

        /// <summary>
        /// Returns an instance of the DriveInfo.
        /// </summary>
        public DriveInfo DriveInfoI
        {
            get { return _driveInfoI; }
        }

        /// <summary>
        /// Returns the root directory name.
        /// </summary>
        public string GetRootDirectory
        {
            get
            {
                return _driveInfoI != null ? _driveInfoI.RootDirectory.Name : string.Empty;
            }
        }

        /// <summary>
        /// Gets the current drive name.
        /// </summary>
        public string DriveName { get; private set; }

        // 11/16/2010 - Converted to BitmapSource, instead of BitmapImage for transparency to work.
        /// <summary>
        /// Gets the drive icon image.
        /// </summary>
        [Obsolete("(8.20.2011) Removed, since can cause cross-thread errors if resource created on seperate thread.  Furthermore, the XAML icons are now used instead of the old bitmap images.", true)]
        public BitmapSource DriveIcon { get; private set; }

        

        #endregion

        /// <summary>
        /// constructor for Desktop
        /// </summary>
        /// <param name="driveInfo">Instance of <see cref="DriveInfo"/></param>
        [CatchAllExceptionHandling(false, null, null)]
        public DriveInfoItem(DriveInfo driveInfo)
        {
            _driveInfoI = driveInfo;
            DriveName = driveInfo.Name;

            // 7/4/2010 - Load Drive Icons
            if (_driveIconsLoaded) return;

            // 8/20/2011: Note: Removed, since can cause cross-thread errors if resource
            //            Note: Created on seperate thread.  Furthermore, the XAML icons
            //            Note: are now used instead of the old bitmap images. - Ben
            //var driveOpen = new Bitmap(Resources.Hard_Disk_32);
            // 11/16/2010 - Convert to Transparent bitmapsource.
            //_driveIcon = driveOpen.ConvertToTransparentBitmapSource();

            _driveIconsLoaded = true;
        }
    }
}