﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows.Controls;
using System.Windows.Threading;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.CloudDataModel.DirectoryDataModel;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.CloudDataModel.DriveDataModel;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.Delegates;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.Enums;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.ViewModel.DirectoryViewModel;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.ViewModel.DriveViewModel;
using ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.BaseClasses;
using ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.Enums;
using ImageNexus.BenScharbach.ICLCommon.ICLInterface;
using ImageNexus.BenScharbach.ICLCommon.TreeViewInterface;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.Interfaces.DriveViewInterfaces;
using Microsoft.Practices.Unity;

// 7/4/2011
/*[assembly: MefLogger(LoggerTypeEnum.TraceLog, "ICLFrontEndConsole", LogPropertyCalls = true,
        LoggerVerbosity = LoggerVerbosityEnum.LogEntry | LoggerVerbosityEnum.LogExceptions,
        TraceLoggerOutputPath = @"ICL_Trace_Logs\LoggerDirCloudTreeViewModel.txt", WpfLogWindowInstanceId = 1, 
        AttributeTargetTypes = "DirCloudTreeViewModel.*",
        AttributeTargetTypeAttributes = MulticastAttributes.Public)]*/

namespace ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel
{
    // 11/7/2010
    /// <summary>
    /// The <see cref="DirCloudTreeViewModelAdapter"/> provides the data-view model to adapt to the
    /// WPF TreeView 'Directory' user control.
    /// </summary>
    public class DirCloudTreeViewModelAdapter : DirectoryTreeViewModelAdapterBase
    {
        // 11/7/2010 - References to the WPF controls
        private readonly TreeViewUserControl _treeViewUserControl;
        private readonly ListView _directoryListView;
        private readonly TreeView _treeView;

        // 10/25/2010 - Store instance of DriveViewModel.
        private readonly DriveViewModel _driveCloudViewModel; 

        // 8/30/2011 - Store instance to current Directories for selected drive.
        private DirectoryViewModel _directoryCloudViewModel; 

        private readonly ILockUnlockUI _lockUnlockUi; // 8/14/2011

        #region Events

        // 10/25/2010 - Drive Selected event.
        /// <summary>
        /// Occurs when the current item 'IsSelected' in the ListView control.
        /// </summary>
        public event SelectedItemEventHandler DriveSelected;

        #endregion

        #region Properties 

        // 11/7/2010
        /// <summary>
        /// Gets an instance of the Directory <see cref="TreeViewUserControl"/>.
        /// </summary>
        public TreeViewUserControl TreeViewUserControl
        {
            get { return _treeViewUserControl; }
        }

        // 11/7/2010
        /// <summary>
        /// Gets an instance of the <see cref="DriveViewModel"/>.
        /// </summary>
        public DriveViewModel DriveCloudViewModel
        {
            get { return _driveCloudViewModel; }
        }

        #endregion

        // 8/14/2011: Updated with ILockUnlockUI Param.
        /// <summary>
        /// Default constructor
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public DirCloudTreeViewModelAdapter(IMainWindow mainWindow, TreeViewUserControl treeViewUserControl, ListView listView, TreeView treeView, string[] drives)
            : base(mainWindow, ViewModelTypeEnum.CloudViewModel, (int)DirectoryTreeViewModelAdapterEnum.Cloud)
        {
            // 11/7/2010 - Store WPF Ref
            _treeViewUserControl = treeViewUserControl;
            _directoryListView = listView;
            _treeView = treeView;
            _lockUnlockUi = (ILockUnlockUI)mainWindow; // 8/14/2011

            // 1/9/2012 - Add this to Unity container
            mainWindow.UnityContainer.RegisterInstance(typeof(DirCloudTreeViewModelAdapter), ViewModelTypeEnum.CloudViewModel.ToString(), this);
            mainWindow.UnityContainer.RegisterInstance(typeof(DirectoryTreeViewModelAdapterBase), ViewModelTypeEnum.CloudViewModel.ToString(), this);
           
            // create instance of DriveViewModel.
            // Populate the internal 'Drives' collection with given cloud data.
            _driveCloudViewModel = new DriveViewModel(mainWindow, ViewModelTypeEnum.CloudViewModel);
            var cloudDrives = new ObservableCollection<IDriveItemViewModel>(
                (from driveName in drives
                 select (IDriveItemViewModel)new DriveInfoViewModel(mainWindow, ViewModelTypeEnum.CloudViewModel, _driveCloudViewModel, new DriveCloudItem(driveName)))
                    .ToList());
            _driveCloudViewModel.Drives = cloudDrives;

            // 8/8/2011
            listView.Dispatcher.BeginInvoke(DispatcherPriority.Normal,
                                            new Action<ListView, DriveViewModel>(SetDataContextCallback),
                                            _directoryListView, _driveCloudViewModel);

            // Connect the Selected event.
            _driveCloudViewModel.SelectedItemEvent += _driveCloudViewModel_SelectedItemEvent;

            
        }

        // 8/6/2011
        private void SetDataContextCallback(ListView listView, DriveViewModel driveCloudViewModel)
        {
            // Connect the ListView to the 'DriveViewModel'.
            listView.DataContext = driveCloudViewModel;
        }

        // 11/8/2010
        /// <summary>
        /// EventHandler to capture the 'IsSelected' event from listview, which then populates the
        /// treeView with the current Directories collection.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void _driveCloudViewModel_SelectedItemEvent(object sender, SelectedItemEventArgs e)
        {
            // 10/25/2010 - Store prior selected drive.
            TreeViewUserControl.PriorDriveSelected = TreeViewUserControl.CurrentDriveSelected;

            // 10/24/2010 - Store current selected drive
            TreeViewUserControl.CurrentDriveSelected = e.DirectoryName;

            // Populate TreeView with current RootDirectories.
            var directoryCloudItem = new DirectoryCloudItem[1];
            directoryCloudItem[0] = new DirectoryCloudItem(MainWindowParent, null, e.DirectoryName, e.DirectoryName);
            
            // Create Directory's View-Model
            var directoryCloudItems = (IEnumerable<DirectoryCloudItem>)directoryCloudItem;
            var directories = new ObservableCollection<IDirectoryItemViewModel>(
               (from directoryItem in directoryCloudItems
                select (IDirectoryItemViewModel)new DirectoryInfoViewModel(MainWindowParent, ViewModelType, TreeViewUserControl, directoryItem, null, null))
                   .ToList());
            _directoryCloudViewModel = new DirectoryViewModel(MainWindowParent, ViewModelType, directories);

            // 8/8/2011
            _treeView.Dispatcher.Invoke(DispatcherPriority.Normal,
                                        new Action<TreeView, DirectoryViewModel>(SetTreeViewContext),
                                        _treeView, _directoryCloudViewModel);
            

            // 10/25/2010 - Trigger DriveSelected.
            OnDriveSelected(e);
            
        }

        // 8/6/2011
        private void SetTreeViewContext(TreeView treeView, DirectoryViewModel directoryCloudViewModel)
        {
            treeView.DataContext = directoryCloudViewModel;
        }

        // 10/25/2010
        /// <summary>
        /// Triggers the 'DriveSelected' event.
        /// </summary>
        /// <param name="e">Instance of <see cref="SelectedItemEventArgs"/>.</param>
        [CatchAllExceptionHandling(false, null, null)]
        private void OnDriveSelected(SelectedItemEventArgs e)
        {
            if (DriveSelected != null)
                DriveSelected(this, e);
        }

        #region Set ProgressBar Callbacks

        // 11/14/2010
        /// <summary>
        /// Used to set up the callback method for the progress-bar reset call.
        /// </summary>
        /// <param name="resetProgressBarCallback">Instance of <see cref="Action{T}"/>.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void SetProgressBarResetCallback(EventHandler<ProgressEventArgs> resetProgressBarCallback)
        {
            if (resetProgressBarCallback == null)
                throw new ArgumentNullException("resetProgressBarCallback");

            // Subscribe to event
            TreeViewItemViewModel.ResetProgressBar += resetProgressBarCallback;
        }

        // 11/14/2010
        /// <summary>
        /// Used to set up the callback method for the progress-bar set value call.
        /// </summary>
        /// <param name="setProgressValueCallback">Instance of <see cref="Action{T}"/>.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void SetProgressBarValueCallback(EventHandler<ProgressEventArgs> setProgressValueCallback)
        {
            if (setProgressValueCallback == null)
                throw new ArgumentNullException("setProgressValueCallback");

            // Subscribe to event
            TreeViewItemViewModel.SetProgressValue += setProgressValueCallback;
        }

        // 11/15/2010
        /// <summary>
        /// Used to set up the callback method for the progress-bar set value call.
        /// </summary>
        /// <param name="setEndProgressCallback">Instance of <see cref="Action"/>.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void SetEndProgressBarCallback(EventHandler setEndProgressCallback)
        {
            if (setEndProgressCallback == null)
                throw new ArgumentNullException("setEndProgressCallback");

            // Subscribe to event
            TreeViewItemViewModel.EndProgressBar += setEndProgressCallback;
        }

        // 12/3/2010
        /// <summary>
        /// Used to set up the callback method for the footer's header text setting.
        /// </summary>
        /// <param name="setFooterHeaderCallback">Instance of <see cref="Action"/>.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void SetFooterHeaderTextCallback(EventHandler<ProgressEventArgs> setFooterHeaderCallback)
        {
            if (setFooterHeaderCallback == null)
                throw new ArgumentNullException("setFooterHeaderCallback");

            // Subscribe to event
            TreeViewItemViewModel.SetFooterHeaderText += setFooterHeaderCallback;
        }

        #endregion
    }
}
