﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Windows.Controls;
using System.Windows.Threading;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.Delegates;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.DesktopDataModel.DirectoryDataModel;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.DesktopDataModel.DriveDataModel;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.Enums;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.ViewModel.DirectoryViewModel;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.ViewModel.DriveViewModel;
using ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.BaseClasses;
using ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.Enums;
using ImageNexus.BenScharbach.ICLCommon.ICLInterface;
using ImageNexus.BenScharbach.ICLCommon.TreeViewInterface;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.Interfaces.DriveViewInterfaces;
using Microsoft.Practices.Unity;

// Adds the Unity Extenstions.

// 7/9/2011
/*[assembly: MefLogger(LoggerTypeEnum.TraceLog, "ICLFrontEndConsole", LogPropertyCalls = false,
        LoggerVerbosity = LoggerVerbosityEnum.LogEntry | LoggerVerbosityEnum.LogExceptions,
        TraceLoggerOutputPath = @"ICL_Trace_Logs\LoggerDirectoryTreeViewModel.txt", WpfLogWindowInstanceId = 1,
        AttributeTargetTypes = "DirectoryTreeViewModel.*",
        AttributeTargetTypeAttributes = MulticastAttributes.Public)]*/

namespace ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel
{
    // 11/7/2010
    public class DirectoryTreeViewModelAdapter : DirectoryTreeViewModelAdapterBase
    {
        // 11/7/2010 - References to the WPF controls
        private readonly TreeViewUserControl _treeViewUserControl;
        private readonly ListView _directoryListView;
        private readonly TreeView _treeView;

        // 10/25/2010 - Store instance of DriveViewModel.
        private readonly DriveViewModel _driveViewModel;
        private readonly ILockUnlockUI _lockUnlockUi; // 12/14/2010

        #region Events

        // 10/25/2010 - Drive Selected event.
        /// <summary>
        /// Occurs when the current item 'IsSelected' in the ListView control.
        /// </summary>
        public event SelectedItemEventHandler DriveSelected;

        #endregion

        #region Properties

        // 11/7/2010
        /// <summary>
        /// Gets an instance of the TreeViewUserControl.
        /// </summary>
        public TreeViewUserControl TreeViewUserControl
        {
            get { return _treeViewUserControl; }
        }

        #endregion

        // 8/14/2011: Updated with ILockUnlockUI Param.
        /// <summary>
        /// Defualt constructor
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public DirectoryTreeViewModelAdapter(IMainWindow mainWindow, TreeViewUserControl treeViewUserControl, ListView listView, TreeView treeView)
            : base(mainWindow, ViewModelTypeEnum.DesktopViewModel, (int)DirectoryTreeViewModelAdapterEnum.Desktop)
        {
            // 11/7/2010 - Store WPF Ref
            _treeViewUserControl = treeViewUserControl;
            _directoryListView = listView;
            _treeView = treeView;
            _lockUnlockUi = (ILockUnlockUI)mainWindow; // 12/14/2010

            // 1/9/2012 - Add this to Unity container
            mainWindow.UnityContainer.RegisterInstance(typeof(DirectoryTreeViewModelAdapter), ViewModelTypeEnum.DesktopViewModel.ToString(), this);
            mainWindow.UnityContainer.RegisterInstance(typeof(DirectoryTreeViewModelAdapterBase), ViewModelTypeEnum.DesktopViewModel.ToString(), this);

            // create instance of DriveViewModel.
            _driveViewModel = new DriveViewModel(mainWindow, ViewModelTypeEnum.DesktopViewModel);
            var drives = new ObservableCollection<IDriveItemViewModel>(
               (from driveInfo in DriveInfo.GetDrives()
                select (IDriveItemViewModel)new DriveInfoViewModel(mainWindow, ViewModelTypeEnum.DesktopViewModel, _driveViewModel, new DriveInfoItem(driveInfo)))
                   .ToList());
            _driveViewModel.Drives = drives;

            // 8/6/2011
            treeViewUserControl.Dispatcher.BeginInvoke(DispatcherPriority.Normal, 
                new Action<ListView, DriveViewModel>(SetDataContext), 
                _directoryListView, _driveViewModel);

            // Connect the Selected event.
            _driveViewModel.SelectedItemEvent += DriveInfoViewModel_SelectedItemEvent;
        }

        // 8/6/2011
        private void SetDataContext(ListView listView, DriveViewModel driveViewModel)
        {
            // Connect the ListView to the 'DriveViewModel'.
            listView.DataContext = driveViewModel;
        }
       

        // 10/23/2010
        /// <summary>
        /// EventHandler to capture the 'IsSelected' event from listview, which then populates the
        /// treeView with the current Directories collection.
        /// </summary>
        // ReSharper disable InconsistentNaming
        //[CatchAllExceptionHandling(false, null, null)]
        private void DriveInfoViewModel_SelectedItemEvent(object sender, SelectedItemEventArgs e)
        // ReSharper restore InconsistentNaming
        {
            // Populate TreeView with current RootDirectories.
            var directoryInfoItem = new DirectoryInfoItem[1];
            directoryInfoItem[0] = new DirectoryInfoItem(e.DirectoryInfo);

            // 10/25/2010 - Store prior selected drive.
            TreeViewUserControl.PriorDriveSelected = TreeViewUserControl.CurrentDriveSelected;

            // 10/24/2010 - Store current selected drive
            TreeViewUserControl.CurrentDriveSelected = e.DriveInfo.Name;

            // Create Directory's View-Model
            var directoryInfoItems = (IEnumerable<DirectoryInfoItem>) directoryInfoItem;
            var directories = new ObservableCollection<IDirectoryItemViewModel>(
                (from directoryInfo in directoryInfoItems
                 select (IDirectoryItemViewModel)new DirectoryInfoViewModel(MainWindowParent, ViewModelType, TreeViewUserControl, directoryInfo, null, null))
                    .ToList());

            var directoryVieWModel = new DirectoryViewModel(MainWindowParent, ViewModelType, directories);

            // 8/6/2011
            //_treeView.DataContext = directoryVieWModel;
            TreeViewUserControl.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action<TreeView, DirectoryViewModel>(SetTreeViewContext), _treeView, directoryVieWModel);
            
            // 10/25/2010 - Trigger DriveSelected.
            OnDriveSelected(e);

        }

        // 8/6/2011
        private void SetTreeViewContext(TreeView treeView, DirectoryViewModel directoryViewModel)
        {
            //Debugger.Break();
            treeView.DataContext = directoryViewModel;
        }

        // 10/25/2010
        /// <summary>
        /// Triggers the 'DriveSelected' event.
        /// </summary>
        /// <param name="e">Instance of <see cref="SelectedItemEventArgs"/>.</param>
        [CatchAllExceptionHandling(false, null, null)]
        private void OnDriveSelected(SelectedItemEventArgs e)
        {
            if (DriveSelected != null)
                DriveSelected(this, e);
        }

        #region Set ProgressBar Callbacks

        // 11/14/2010
        /// <summary>
        /// Used to set up the callback method for the progress-bar reset call.
        /// </summary>
        /// <param name="resetProgressBarCallback">Instance of <see cref="Action{T}"/>.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void SetProgressBarResetCallback(EventHandler<ProgressEventArgs> resetProgressBarCallback)
        {
            if (resetProgressBarCallback == null)
                throw new ArgumentNullException("resetProgressBarCallback");

            // Subscribe to event
            TreeViewItemViewModel.ResetProgressBar += resetProgressBarCallback;
        }

        // 11/14/2010
        /// <summary>
        /// Used to set up the callback method for the progress-bar set value call.
        /// </summary>
        /// <param name="setProgressValueCallback">Instance of <see cref="Action{T}"/>.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void SetProgressBarValueCallback(EventHandler<ProgressEventArgs> setProgressValueCallback)
        {
            if (setProgressValueCallback == null)
                throw new ArgumentNullException("setProgressValueCallback");

            // Subscribe to event
            TreeViewItemViewModel.SetProgressValue += setProgressValueCallback;
        }

        // 11/15/2010
        /// <summary>
        /// Used to set up the callback method for the progress-bar set value call.
        /// </summary>
        /// <param name="setEndProgressCallback">Instance of <see cref="Action"/>.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void SetEndProgressBarCallback(EventHandler setEndProgressCallback)
        {
            if (setEndProgressCallback == null)
                throw new ArgumentNullException("setEndProgressCallback");

            // Subscribe to event
            TreeViewItemViewModel.EndProgressBar += setEndProgressCallback;
        }

        // 12/3/2010
        /// <summary>
        /// Used to set up the callback method for the footer's header text setting.
        /// </summary>
        /// <param name="setFooterHeaderCallback">Instance of <see cref="Action"/>.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void SetFooterHeaderTextCallback(EventHandler<ProgressEventArgs> setFooterHeaderCallback)
        {
            if (setFooterHeaderCallback == null)
                throw new ArgumentNullException("setFooterHeaderCallback");

            // Subscribe to event
            TreeViewItemViewModel.SetFooterHeaderText += setFooterHeaderCallback;
        }

        #endregion

    }
}
