﻿using System;
using System.Windows.Media.Imaging;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.Delegates;
using ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.BaseClasses;
using ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.Enums;
using ImageNexus.BenScharbach.ICLCommon.ICLInterface;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.Interfaces.DriveViewInterfaces;

namespace ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.ViewModel.DriveViewModel
{
    // 7/2/2010
    public sealed class DriveInfoViewModel : ListViewItemViewModel, IDriveItemViewModel
    {
        #region Fields

        // instance of DriveInfo
        private readonly IDriveItem _driveInfoItem;
        // 10/25/2010 - Parent instance of 'DriveViewModel'.
        private readonly IDriveViewModel _driveViewModel;

        #endregion

        #region Properties

        /// <summary>
        /// Gets the current drive name.
        /// </summary>
        public string DriveName
        {
            get
            {
                return _driveInfoItem != null ? _driveInfoItem.DriveName : string.Empty;
            }
        }

        // 8/20/2011: Note: Removed, since can cause cross-thread errors if resource
        //            Note: Created on seperate thread.  Furthermore, the XAML icons
        //            Note: are now used instead of the old bitmap images. - Ben
        // 7/4/2010; // 11/16/2010 - Converted to BitmapSource, instead of BitmapImage for transparency to work.
        /// <summary>
        /// Gets the current drive image.
        /// </summary>
        [Obsolete("(8.20.2011) Removed, since can cause cross-thread errors if resource created on seperate thread.  Furthermore, the XAML icons are now used instead of the old bitmap images.", true)]
        public BitmapSource DriveImage
        {
            get
            {
                return _driveInfoItem != null ? _driveInfoItem.DriveIcon : null;
            }
        }

        #endregion

        /// <summary>
        /// constructor
        /// </summary>
        /// <param name="mainWindow"> </param>
        /// <param name="viewModelType"> </param>
        /// <param name="driveViewModel">Parent instance of <see cref="IDriveViewModel"/>.</param>
        /// <param name="driveInfoItem">Instance of <see cref="IDriveItem"/></param>
        [CatchAllExceptionHandling(false, null, null)]
        public DriveInfoViewModel(IMainWindow mainWindow, ViewModelTypeEnum viewModelType, IDriveViewModel driveViewModel, IDriveItem driveInfoItem)
            : base(mainWindow, viewModelType)
        {
            _driveViewModel = driveViewModel; // 10/25/2010
            _driveInfoItem = driveInfoItem;
        }

        // 10/23/2010; 10/25/2010 - Add 'IsSelected==true' check.
        /// <summary>
        /// Triggered by WPF when some Property is changed via the interface INotifyProperty,
        /// which then triggers the event <see cref="DriveViewModel.SelectedItemEvent"/> in parent class.
        /// </summary>
        /// <param name="propertyName">Property updated by WPF</param>
        [CatchAllExceptionHandling(false, null, null)]
        protected override void OnPropertyChanged(string propertyName)
        {
            base.OnPropertyChanged(propertyName);

            // Trigger when 'IsSelected' property is set to True.
            if (propertyName == "IsSelected" && IsSelected)
            {
                // Trigger 'SelectedItem' event.
                if (_driveViewModel != null)
                    ((global::ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.ViewModel.DriveViewModel.DriveViewModel)_driveViewModel).OnSelectedItem(new SelectedItemEventArgs
                                                       {
                                                           DirectoryName = (_driveInfoItem != null) ? _driveInfoItem.GetRootDirectory : string.Empty,
                                                           DirectoryInfo = (_driveInfoItem != null) ? _driveInfoItem.DirectoryInfo : null,
                                                           DriveInfo = (_driveInfoItem != null) ? _driveInfoItem.DriveInfoI : null

                                                       });
            }
        }
        
    }
}