﻿using System.Collections.ObjectModel;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.Delegates;
using ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.BaseClasses;
using ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.Enums;
using ImageNexus.BenScharbach.ICLCommon.ICLInterface;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.Interfaces.DriveViewInterfaces;

namespace ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.ViewModel.DriveViewModel
{
    /// <summary>
    /// The ViewModel for the LoadOnDemand demo.  This simply
    /// exposes a read-only collection of <see cref="DriveViewModel"/>.
    /// </summary>
    public sealed class DriveViewModel : ViewModelBase, IDriveViewModel
    {
        #region Fields

        private ObservableCollection<IDriveItemViewModel> _drives;

        #endregion

        #region Events

        // 10/25/2010
        /// <summary>
        /// Occurs when the current item 'IsSelected' in the ListView control.
        /// </summary>
        public event SelectedItemEventHandler SelectedItemEvent;

        #endregion

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="mainWindow"> </param>
        /// <param name="viewModelType"> </param>
        [CatchAllExceptionHandling(false, null, null)]
        public DriveViewModel(IMainWindow mainWindow, ViewModelTypeEnum viewModelType)
            : base(mainWindow, viewModelType)
        {
            // Empty
        }

        /// <summary>
        /// Returns <see cref="DriveInfoViewModel"/> observable collection.
        /// </summary>
        public ObservableCollection<IDriveItemViewModel> Drives
        {
            get { return _drives; }
            set { _drives = value; }
        }

        // 10/25/2010
        /// <summary>
        /// Called to trigger the 'SelectedItem' event.
        /// </summary>
        /// <param name="selectedItemEventArgs">Instance of <see cref="SelectedItemEventArgs"/>.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void OnSelectedItem(SelectedItemEventArgs selectedItemEventArgs)
        {
            // trigger event
            if (SelectedItemEvent != null)
                SelectedItemEvent(this, selectedItemEventArgs);
        }
    }
}