using System.ComponentModel;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.Enums;
using ImageNexus.BenScharbach.ICLCommon.ICLInterface;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.Interfaces;

namespace ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.BaseClasses
{
    /// <summary>
    /// Base class for all ViewModel classes displayed by ListViewItems.  
    /// This acts as an adapter between a raw data object and a ListViewItem.
    /// </summary>
    public class ListViewItemViewModel : ViewModelBase, INotifyPropertyChanged, IListViewItemViewModel
    {
        private bool _isSelected;
        private bool _isChecked; // 11/3/2010

        #region Constructors

        /// <summary>
        /// constructor
        /// </summary>
        /// <param name="mainWindow">Instance of <see cref="IMainWindow"/></param>
        /// <param name="viewModelType"> </param>
        protected ListViewItemViewModel(IMainWindow mainWindow, ViewModelTypeEnum viewModelType)
            : base(mainWindow, viewModelType)
        {
            // empty
        }

        #endregion // Constructors

        #region Presentation Members
        
        /// <summary>
        /// Gets/sets whether the TreeViewItem 
        /// associated with this object is selected.
        /// </summary>
        public virtual bool IsSelected
        {
            get { return _isSelected; }
            set
            {
                if (value != _isSelected)
                {
                    _isSelected = value;
                    OnPropertyChanged("IsSelected");
                }
            }
        }

        #region IsChecked

        // 11/3/2010
        /// <summary>
        /// Gets/sets whether the ListViewItem 
        /// associated with this object is checked.
        /// </summary>
        public virtual bool IsChecked
        {
            get { return _isChecked; }
            set
            {
                if (value != _isChecked)
                {
                    _isChecked = value;
                    OnPropertyChanged("IsChecked");
                }
            }
        }

        #endregion // IsChecked
        

        #endregion // Presentation Members

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        [CatchAllExceptionHandling(false, null, null)]
        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion // INotifyPropertyChanged Members
    }
}