﻿using System;
using System.Collections.Generic;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;
using Microsoft.Practices.Unity;

namespace ImageNexus.BenScharbach.ICLCommon.ICLInterface
{
    // 2/11/2011
    public interface IMainWindow
    {
        // 1/9/2012
        /// <summary>
        /// Gets the current <see cref="UnityContainer"/> instance.
        /// </summary>
        UnityContainer UnityContainer { get; }

        /// <summary>
        /// Occurs when the user clicks on the Cancel button request during
        /// some asnychronous operation.
        /// </summary>
        event EventHandler CancelCurrentOperation;

        /// <summary>
        /// Occurs when the user clicks on the Delete button.
        /// </summary>
        event EventHandler DeleteItems;

        /// <summary>
        /// OnGui WPF UI thread call.
        /// </summary>
        /// <typeparam name="T">Generic Type</typeparam>
        /// <param name="action">Action delegate</param>
        /// <param name="paramItem">Parameter to pass into Action delegate</param>
        void OnGuiThreadMethodCall<T>(Action<T> action, T paramItem) where T : class;

        /// <summary>
        /// OnGui WPF UI thread call.
        /// </summary>
        /// <param name="action">Action delegate</param>
        void OnGuiThreadMethodCall(Action action);
        
        // 4/17/2011
        /// <summary>
        /// Gets the current cloud files for the given directory.
        /// </summary>
        /// <param name="directoryItemViewModel">Instance of <see cref="IDirectoryItemViewModel"/></param>
        /// <param name="directoryName">The directory name to retrieve the files for.</param>
        /// <returns>Collection of <see cref="IFileItem"/></returns>
        List<IDirectoryItem> GetCloudDirectoriesForDirectory(IDirectoryItemViewModel directoryItemViewModel, string directoryName);

        // 4/17/2011
        /// <summary>
        /// Gets the current cloud directories for the given directory.
        /// </summary>
        /// <param name="directoryItemViewModel">Instance of <see cref="IDirectoryItemViewModel"/></param>
        /// <param name="directoryName">The directory name to retrieve the directories for.</param>
        /// <returns>Collection of <see cref="IDirectoryItem"/></returns>
        List<IFileItem> GetCloudFilesForDirectory(IDirectoryItemViewModel directoryItemViewModel, string directoryName);
    }
}