﻿using System;
using System.Windows;
using ImageCloudLock;

namespace ImageNexus.BenScharbach.ImageCloudLock
{
    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App : Application
    {
        public App()
        {
        }

        // 7/18/2011 - Captures the StartUp event to populate with params.
        private void Application_Startup(object sender, StartupEventArgs e)
        {
            try // 1/31/2012
            {
#if Azure
                MessageBox.Show("ICL Beta program starting... ");
#endif

                // Retrieve Args
                var args = e.Args;

                // Iterate given Args.
                foreach (var arg in args)
                {
                    var argLowerCase = arg.ToLower();

                    // Check if '/UseLogger'
                    if (argLowerCase == "/UseLogger".ToLower())
                    {
#if DEBUG
                        //MessageBox.Show("The '/UseLogger' was entered!");
#endif

                        // Set Application dictionary with 'UseLogger'.
                        Properties.Add("UseLogger", true);
                    }
                }

                // 1/31/2012 - Subscribe to the Dispatcher exception handling
                Application.Current.DispatcherUnhandledException += Current_DispatcherUnhandledException;

                // 1/31/2012 - Instantiate the MainWindow here
                // Note: Prior, the MainWindow was instantiated through the use of the StartupUri.
                var mainWindow = new MainWindow();
                mainWindow.Show();
            }
            catch (Exception exception)
            {
                Console.WriteLine(exception);
                var messageBoxText = string.Format("Application Error {0}", exception);
                MessageBox.Show(messageBoxText, "ImageCloudLock Error", MessageBoxButton.OK,
                                MessageBoxImage.Error);

                Application.Current.ShutdownMode = ShutdownMode.OnExplicitShutdown;
                Application.Current.Shutdown();
            }

        }

        // 1/31/2012
        /// <summary>
        /// Captures any Dispatcher Unhandled exception within the ImageCloudLock application.
        /// </summary>
        private void Current_DispatcherUnhandledException(object sender, System.Windows.Threading.DispatcherUnhandledExceptionEventArgs e)
        {
            Console.WriteLine(e.Exception.Message);
            var messageBoxText = string.Format("Application Error {0}", e.Exception.Message);

            MessageBox.Show(messageBoxText, "ImageCloudLock Error", MessageBoxButton.OK,
                            MessageBoxImage.Error);

            e.Handled = true;

            Application.Current.ShutdownMode = ShutdownMode.OnExplicitShutdown;
            Application.Current.Shutdown();
        }

    }
}
