﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.IO;
using System.Linq;
using System.ServiceModel;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media;
using System.Windows.Threading;
using ImageCloudLock;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsMisc;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsThreading;
using Attributes;
using ImageNexus.BenScharbach.Common.AttsInjectorHelper;
using ImageNexus.BenScharbach.Common.ProgressBarInterface;
using ImageNexus.BenScharbach.Common.ProgressBarWindow.Enums;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.Delegates;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.DesktopDataModel.DriveDataModel;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.ViewModel.DriveViewModel;
using ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.Enums;
using ImageNexus.BenScharbach.ICLCommon.ICLInterface;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.Interfaces.DriveViewInterfaces;
using ImageNexus.BenScharbach.ICLTools.StorageTools.BackupData;
using ImageNexus.BenScharbach.ImageCloudLock.Enums;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;
using ImageNexus.BenScharbach.ImageCloudLock.Interfaces;
using ImageNexus.BenScharbach.ImageCloudLock.ProgressBar;

namespace ImageNexus.BenScharbach.ImageCloudLock
{
    // 11/3/2010
    /// <summary>
    /// Interaction logic for BackupNowWindow.xaml
    /// </summary>
    public partial class BackupNowWindow : Window
    {
        // 12/6/2010 - Tracks if Backup-Now upload is currently in process.
        private bool _doingBackupNowUploadOp;

        // Stores instance of driveViewModel.
        private readonly DriveViewModel _driveViewModel;
        private readonly IBackupTreeViewItems _backupTreeViewItems; // 8/20/2011

        // 9/4/2011
        private readonly ProgressBarService _progressBarService;

        #region Properties

        /// <summary>
        /// Owner of this window.
        /// </summary>
        public MainWindow MainWindowParent { get; private set; }

        // 12/6/2010
        /// <summary>
        /// Gets the current instance of <see cref="MiniProgressBar"/>.
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(IMiniProgressBar), null)]
        public IMiniProgressBar MiniProgressBar
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.MiniProgressBar : null;
            }
            set
            {
                // If null, then just wait in parallel thread.
                // Note: This will always occur when window created because PostSharp attribute tries
                // note: to set before the constructor has run.
                if (MainWindowParent == null)
                    Task.Factory.StartNew(() =>
                                              {
                                                  while (MainWindowParent == null)
                                                  {
                                                      // Just spin wait. 
                                                  }

                                                  // Not null anymore, so set value.
                                                  MainWindowParent.MiniProgressBar = value;

                                              });
                else
                    MainWindowParent.MiniProgressBar = value;
            }
        }

        // 12/6/2010
        /// <summary>
        /// Gets the current <see cref="DuplexChannelFactory{IclWebCommService}"/>.
        /// </summary>
        public DuplexChannelFactory<IclWebCommService> IclDuplexChannelFactory
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.IclDuplexChannelFactory : null;
            }
            set { if (MainWindowParent != null) MainWindowParent.IclDuplexChannelFactory = value; }
        }

        // 12/6/2010
        /// <summary>
        /// Gets the current communication channel.
        /// </summary>
        public IclWebCommService CurrentChannel { get; set; }
        

        // 12/6/2010
        // NOTE: The 'Set' is required to keep PostSharp from crashing - bug.
        /// <summary>
        /// Gets the <see cref="ICommunicationObject"/> interface for the <see cref="IclDuplexChannelFactory"/>
        /// instance.  
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(ICommunicationObject), null)]
        public ICommunicationObject CommunicationObject
        {
            get { return IclDuplexChannelFactory; }
            private set { throw new InvalidOperationException(); }
        }

        #endregion

        /// <summary>
        /// Default constructor
        /// </summary>
        public BackupNowWindow()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="mainWindow">Instance of <see cref="MainWindow"/>.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public BackupNowWindow(MainWindow mainWindow)
        {
            InitializeComponent();

            // Save MainWindow ref
            MainWindowParent = mainWindow;

            // 2/19/2011
            // Check for Valid License.
            /*var license = new LicenseHelper();
            if (!license.Check(false))
                Application.Current.Shutdown();*/

            // 1/27/2012 - Set CloudService to Use in Wcf.
            mainWindow.SetCloudServiceToUse();

            // 3/1/2012
            _progressBarService = new ProgressBarService(); 

            // Retrieve IBackupTreeViewItems
            _backupTreeViewItems = (IBackupTreeViewItems) Application.Current.Properties["BackupTreeViewItems"];

            // Subscribe to the 'TransferCompleted' event.
            MainWindowParent.IclWebDuplexCallbacks.OperationCompleted += IclWebDuplexCallbacks_TransferCompleted;
            
            // create instance of DriveViewModel.
            _driveViewModel = new DriveViewModel((IMainWindow) mainWindow, ViewModelTypeEnum.DesktopViewModel);
            var drives = new ObservableCollection<IDriveItemViewModel>(
                (from driveInfo in DriveInfo.GetDrives()
                 select (IDriveItemViewModel)new DriveInfoViewModel((IMainWindow)mainWindow, ViewModelTypeEnum.DesktopViewModel, _driveViewModel, new DriveInfoItem(driveInfo)))
                    .ToList());
            _driveViewModel.Drives = drives;
            

            // Connect the Selected event.
            _driveViewModel.SelectedItemEvent += DriveInfoViewModel_SelectedItemEvent;

            // Connect the ListView to the 'DriveViewModel'.
            listViewBackupLists.DataContext = _driveViewModel;
        }

        /// <summary>
        /// Occurs when the transfer is complete.
        /// </summary>
// ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void IclWebDuplexCallbacks_TransferCompleted(object sender, IOperationCompletedEventArgs e)
// ReSharper restore InconsistentNaming
        {
            // 12/6/2010 - Set flag
            _doingBackupNowUploadOp = false;

            // 11/10/2010 - First check if cancelled.
            if (e.OperationCancelled)
            {
                txtResults.Foreground = new SolidColorBrush(Colors.OrangeRed);
                txtResults.Text = "Cancelled";
                return;
            }

            // Check result of transfer
            if (!e.OperationResult)
            {
                txtResults.Foreground = new SolidColorBrush((e.FailedCount == e.TotalCount) ? Colors.Red : Colors.Orange);
                txtResults.Text = (e.FailedCount == e.TotalCount) ? "Failed" : "Success with failures";
            }
            else
            {
                txtResults.Foreground = new SolidColorBrush(Colors.Green);
                txtResults.Text = "Success";
            }

        }

        // 12/6/2010 - Updated to use parallel Task.
        // 11/3/2010; 12/2/2010 - Refactored code out to 'GetFilesforBackupList' in main window.
        /// <summary>
        /// EventHandler to capture the 'IsSelected' event from listview, which then populates the
        /// other listView with the current files collection.
        /// </summary>
        // ReSharper disable InconsistentNaming
        [ShowWaitCursor]
        [CatchAllExceptionHandling(false, null, null)]
        private void DriveInfoViewModel_SelectedItemEvent(object sender, SelectedItemEventArgs e)
        // ReSharper restore InconsistentNaming
        {
            // 12/2/2010
            var filesForBackupList = _backupTreeViewItems.GetFilesForBackupList(e.DriveInfo.Name);

            // Set data to 2nd listView.
            SetListViewItemSource(filesForBackupList);

            // 2/24/2012 - Display Count
            var backupCount = (filesForBackupList != null) ? filesForBackupList.Count.ToString(CultureInfo.InvariantCulture) : "Empty Set";
            txtResults.Text = string.Format("Backup Count = {0}", backupCount);
        }

        // 12/6/2010
        /// <summary>
        /// Sets collection as ItemSource for Listview.
        /// </summary>
        /// <param name="filesForBackupList">Collection of <see cref="FileBackupItem"/></param>
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = true)]
        [CatchAllExceptionHandling(false, null, null)]
        private void SetListViewItemSource(IEnumerable<FileBackupItem> filesForBackupList)
        {
            // Set data to 2nd listView.
            listViewBackupListDetails.ItemsSource = filesForBackupList;
        }


        // 11/3/2010; // 12/6/2010: Updated with new Attributes.
        /// <summary>
        /// Occurs when user presses the 'Backup Now' button.
        /// </summary>
// ReSharper disable InconsistentNaming
        [ShowWaitCursor]
        [CatchAllExceptionHandling(false, null, null)]
        private void btnStartBackupNow_Click(object sender, RoutedEventArgs e)
// ReSharper restore InconsistentNaming
        {
            // Get all Checked items using LINQ;
            var checkedDrives = (from driveInfoViewModel in _driveViewModel.Drives
                                 where driveInfoViewModel.IsChecked
                                 select driveInfoViewModel.DriveName).ToArray();

            // Where any files selected.
            if (checkedDrives.Length == 0)
            {
                MessageBox.Show("You must select some drive to backup first.");
                return;
            }

            // Show WCF ProgressBar Window
            _progressBarService.Start(ProgressBarEventHandlerTypeEnum.BackupEventHandler, ProgressBarType.StandardDualBar);

            txtResults.Foreground = new SolidColorBrush(Colors.Orange);
            txtResults.Text = "Processing...";

            // 8/28/2011 - Note: 1st param can be null, since it will be set by postsharp.
            DoWCFStartBackupForDrives(null, checkedDrives);

            // 12/6/2010 - Set flag
            _doingBackupNowUploadOp = true;
        }

        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.BeginCall)]
        private void DoWCFStartBackupForDrives(IclWebCommService currentChannel, string[] checkedDrives)
        {
            // Call WCF service to start backup.
            currentChannel.StartBackupForDrives(checkedDrives);
        }

        // 12/6/2010
        /// <summary>
        /// Occurs when user is attempting to close window.
        /// </summary>
// ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void BackupNowTool_Closing(object sender, System.ComponentModel.CancelEventArgs e)
// ReSharper restore InconsistentNaming
        {
            if (!_doingBackupNowUploadOp) return;

            // Turn Off the progressBar from being top window.
            if (_progressBarService != null) 
                _progressBarService.SetTopMostWindow(false);

            var dialogResult = MessageBox.Show("This will cancel the current operation!  Do you want to continue?",
                                               "Cancel Operation",
                                               MessageBoxButton.OKCancel, MessageBoxImage.Exclamation,
                                               MessageBoxResult.Cancel,
                                               MessageBoxOptions.None);

            // Should operation cancel.
            switch (dialogResult)
            {
                case MessageBoxResult.OK:
                    e.Cancel = false;
                    // Cancel Operation.
                    _progressBarService.DoCancelOperation();
                   
                    break;
                case MessageBoxResult.Cancel:
                    e.Cancel = true;
                    break;
            }
        }
    }
}
