﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Threading;
using ImageCloudLock;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsMisc;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsThreading;
using Attributes;
using ImageNexus.BenScharbach.Common.AttsInjectorHelper;
using ImageNexus.BenScharbach.Common.ProgressBarInterface;
using ImageNexus.BenScharbach.Common.ProgressBarWindow.Enums;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel;
using ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.Enums;
using ImageNexus.BenScharbach.ICLCommon.ICLInterface;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.CheckedDataModel;
using ImageNexus.BenScharbach.ICLTools.StorageTools.CloudData;
using ImageNexus.BenScharbach.Common.LogWinEvents;
using ImageNexus.BenScharbach.ImageCloudLock.CloudCache;
using ImageNexus.BenScharbach.ImageCloudLock.Enums;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;
using ImageNexus.BenScharbach.ImageCloudLock.ParallelActions;
using ImageNexus.BenScharbach.ImageCloudLock.ProgressBar;
using Microsoft.Practices.Unity;

// Adds the Unity Extenstions.

namespace ImageNexus.BenScharbach.ImageCloudLock
{
    /// <summary>
    /// Interaction logic for DeleteEmptyWindow.xaml
    /// </summary>
    public partial class DeleteEmptyWindow : Window
    {
        // 9/4/2011
        private ProgressBarService _progressBarService;

        // 1/11/2012 - List of files to delete
        private readonly List<IFileCheckedItem<IFileItem>> _fileCheckedItems;

        // 1/14/2012 - List of files to delete in WCF format
        private List<GetFileCloudItem> _deleteFilesToSend;

        // 1/14/2012 - Batch Count
        private int _batchCount;

        #region Properties

        /// <summary>
        /// Owner of this window.
        /// </summary>
        public MainWindow MainWindowParent { get; private set; }

        // 12/6/2010
        /// <summary>
        /// Gets the current instance of <see cref="MiniProgressBar"/>.
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(IMiniProgressBar), null)]
        public IMiniProgressBar MiniProgressBar
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.MiniProgressBar : null;
            }
            set
            {
                // If null, then just wait in parallel thread.
                // Note: This will always occur when window created because PostSharp attribute tries
                // note: to set before the constructor has run.
                MainWindow windowParent = MainWindowParent; // 7/22/2011 - Cache
                if (windowParent == null)
                    Task.Factory.StartNew(() =>
                    {
                        while (windowParent == null)
                        {
                            // Just spin wait. 
                        }

                        // Not null anymore, so set value.
                        windowParent.MiniProgressBar = value;

                    });
                else
                    windowParent.MiniProgressBar = value;
            }
        }

        // 12/6/2010
        /// <summary>
        /// Gets the current <see cref="DuplexChannelFactory{IclWebCommService}"/>.
        /// </summary>
        public DuplexChannelFactory<IclWebCommService> IclDuplexChannelFactory
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.IclDuplexChannelFactory : null;
            }
            set { if (MainWindowParent != null) MainWindowParent.IclDuplexChannelFactory = value; }
        }

        // 12/6/2010
        /// <summary>
        /// Gets the current communication channel.
        /// </summary>
        public IclWebCommService CurrentChannel { get; set; }

        // 12/6/2010
        // NOTE: The 'Set' is required to keep PostSharp from crashing - bug.
        /// <summary>
        /// Gets the <see cref="ICommunicationObject"/> interface for the <see cref="IclDuplexChannelFactory"/>
        /// instance.  
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(ICommunicationObject), null)]
        public ICommunicationObject CommunicationObject
        {
            get { return IclDuplexChannelFactory; }
            private set { throw new InvalidOperationException(); }
        }
        
        #endregion
        
        /// <summary>
        /// Delete Action window constructor
        /// </summary>
        public DeleteEmptyWindow(MainWindow mainWindow, List<IFileCheckedItem<IFileItem>> fileCheckedItems)
        {
            InitializeComponent();

            // 1/11/2012
            if (fileCheckedItems == null)
            {
                MessageBox.Show("You MUST select an item first to delete.");
                Close();
                return;
            }

            // 1/11/2012
            if (fileCheckedItems.Count == 0)
            {
                MessageBox.Show("You MUST select an item first to delete.");
                Close();
                return;
            }

            // Save MainWindow ref
            MainWindowParent = mainWindow;

            // 1/27/2012 - Set CloudService to Use in Wcf.
            mainWindow.SetCloudServiceToUse();
            
            // 1/11/2012 - Save files to delete
            _fileCheckedItems = fileCheckedItems;

            // 1/11/2012 - Show files to delete in listbox
            ShowFilesToDeleteInListbox(string.Format("Files Selected for Deletion - {0};", _fileCheckedItems.Count), _fileCheckedItems);

            // Subscribe to the 'TransferCompleted' event.
            MainWindowParent.IclWebDuplexCallbacks.OperationCompleted += IclWebDuplexCallbacks_TransferCompleted;
        }

        // 1/11/2012
        /// <summary>
        /// Shows files to delete in the listbox
        /// </summary>
        /// <param name="header"> </param>
        /// <param name="itemsToDisplay"> </param>
        private void ShowFilesToDeleteInListbox(string header, IEnumerable<IFileCheckedItem<IFileItem>> itemsToDisplay)
        {
            // iterate the files to delete
            foreach (var item in itemsToDisplay)
            {
                deleteFeedBackListBox.Items.Add(new TextBlock() { Text = item.FullName, FontSize = 8, Foreground = Brushes.Navy });
            }

            // Set Footer for this section
            deleteFeedBackListBox.Items.Add(new TextBlock() { Text = header, FontSize = 9, Foreground = Brushes.Crimson });
        }

        // 1/11/2012
        /// <summary>
        /// Show files deleted successfully in the listbox
        /// </summary>
        /// <param name="header"> </param>
        /// <param name="doWcfEndDeleteFilesFromCloud"></param>
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = true)]
        private void ShowFilesDeletedSuccessInListbox(string header, IEnumerable<DeleteFileCloudItem> doWcfEndDeleteFilesFromCloud)
        {
            // iterate the files to delete
            foreach (var item in doWcfEndDeleteFilesFromCloud)
            {
                successFeedBackListBox.Items.Add(new TextBlock()
                                                     {Text = item.FullName, FontSize = 8, Foreground = Brushes.Navy});
            }

            // Set Footer for this section
            successFeedBackListBox.Items.Add(new TextBlock() { Text = header, FontSize = 9, Foreground = Brushes.Green });
        }

        /// <summary>
        /// Occurs when the transfer is complete.
        /// </summary>
        // ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void IclWebDuplexCallbacks_TransferCompleted(object sender, IOperationCompletedEventArgs e)
        // ReSharper restore InconsistentNaming
        {

            // 11/10/2010 - First check if cancelled.
            if (e.OperationCancelled)
            {
                txtResults.Foreground = new SolidColorBrush(Colors.OrangeRed);
                txtResults.Text = "Cancelled";
                return;
            }

            // Check result of transfer
            if (!e.OperationResult)
            {
                txtResults.Foreground = new SolidColorBrush(Colors.Red);
                txtResults.Text = "Failed";
            }
            else
            {
                txtResults.Foreground = new SolidColorBrush(Colors.Green);
                txtResults.Text = "Success";
            }

        }

        // 1/11/2012 - Starts Delete operation
        private void btnStartDelete_Click(object sender, RoutedEventArgs e)
        {
            // Confirm deletion request
            var result =
                MessageBox.Show("This will DELETE the checked items from your cloud account.  Do you want to continue?",
                                "Delete items", MessageBoxButton.YesNo, MessageBoxImage.Question, MessageBoxResult.None);

            // If No, then return
            if (result == MessageBoxResult.No) return;

            // Show WCF ProgressBar Window
            _progressBarService = new ProgressBarService(); //MainWindow.UnityContainer.Resolve<ProgressBarWindow>();
            _progressBarService.Start(ProgressBarEventHandlerTypeEnum.DeleteEventHandler, ProgressBarType.StandardDualBar);

            // Get collection of files to delete
            var doParallelCreateFiles = new DoParallelForCreateDeleteFiles();
            _deleteFilesToSend = doParallelCreateFiles.CreateDeleteFilesCollection(_fileCheckedItems).ToList();

            // Start Delete Operation
            DeleteFilesFromCloud();
        }

        // 12/14/2010 - 
        [ShowWaitCursor]
        [CatchAllExceptionHandling(false, null, null)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        private void DeleteFilesFromCloud()
        {
            // if null, return
            if (_deleteFilesToSend == null)
            {
                return;
            }

            // if empty, return
            if (_deleteFilesToSend.Count == 0)
            {
                return;
            }
           
            // Transfer delete files in batches to WCF service.
            bool clearPriorData = true;
            do
            {
                // 1/14/2012 - Take a batch of files to send to WCF.  When the WCF EndCall is reached, this method will
                //             be recalled if there are remaining records.
                const int batchCount = 50;
                var deleteFilesToSendBatched = new GetFileCloudItem[batchCount];
                var copyCount = (_deleteFilesToSend.Count >= batchCount) ? batchCount : _deleteFilesToSend.Count;
                _deleteFilesToSend.CopyTo(0, deleteFilesToSendBatched, 0, copyCount);

                // Check if batch smaller than batchCount; resize if necessary
                if (_deleteFilesToSend.Count < batchCount)
                {
                    Array.Resize(ref deleteFilesToSendBatched, _deleteFilesToSend.Count);
                }

                // remove range
                _deleteFilesToSend.RemoveRange(0, copyCount);

                // 1/14/2012 - Clear prior data
                // Note: 1st param can be null, since it will be set by postsharp.
                DoWCFAddDeleteFilesBatched(null, deleteFilesToSendBatched, clearPriorData);
                clearPriorData = false;
            }
            while (_deleteFilesToSend.Count > 0);            
            

            // 12/14/2010 - Create AsyncCallObject to pass to callback.
            var asyncCallObject = new AsyncDeleteCallObject<IclWebCommService>
            {
                FileCheckedItems = _fileCheckedItems, // 5/14/2011
                CurrentChannel = null,
                ResultObject = null,
            };

            // Note: 1st param can be null, since it will be set by postsharp.
            DoWCFDeleteFilesCall(null, asyncCallObject);
        }

        // 1/14/2012
        /// <summary>
        /// Method helper for WCF call to add deletion batch files.
        /// </summary>
        /// <param name="currentChannel"></param>
        /// <param name="filesToDelete">Collection of files to delete batched.</param>
        /// <param name="clearPriorData">Clears the prior data from service's internal collection.</param>
        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private void DoWCFAddDeleteFilesBatched(IclWebCommService currentChannel, GetFileCloudItem[] filesToDelete, bool clearPriorData)
        {
            // Call WCF service call.
            currentChannel.AddDeleteFilesBatched(filesToDelete, clearPriorData);
        }

        /// <summary>
        /// Method helper for WCF call.
        /// </summary>
        /// <param name="currentChannel"></param>
        /// <param name="asyncCallObject"></param>
        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.BeginCall)]
        private void DoWCFDeleteFilesCall(IclWebCommService currentChannel, AsyncDeleteCallObject<IclWebCommService> asyncCallObject)
        {
            // Call WCF service call.
            asyncCallObject.CurrentChannel = currentChannel;
            currentChannel.BeginDeleteFilesFromCloud(OnEndDeleteFilesFromCloud, asyncCallObject);
        }

        // WCF Callback
        [OnWCFProxyCall(AsynchronousCallType = OnWCFProxyCallAttribute.AsynchronousCallEnum.EndCall)]
        private void OnEndDeleteFilesFromCloud(IAsyncResult asyncResult)
        {
            try
            {

                var asyncCallObject = asyncResult.AsyncState as AsyncDeleteCallObject<IclWebCommService>;
                if (asyncCallObject == null)
                    throw new InvalidOperationException("The AsyncCallObject cannot be null.");

                // Note: 3/20/2012 - The collection return now includes failures, or records which do not exist on the cloud.
                // Note: 1st param can be null, since it will be set by postsharp.
                DeleteFileCloudItem[] doWcfEndDeleteFilesFromCloud = asyncCallObject.CurrentChannel.EndDeleteFilesFromCloud(asyncResult);
                if (doWcfEndDeleteFilesFromCloud == null) return;

                // Convert to Dictionary format
                Dictionary<string, DeleteFileCloudItem> successfulDeletedFileCloudItems = doWcfEndDeleteFilesFromCloud.ToDictionary(key => key.Key);
                if (successfulDeletedFileCloudItems.Count == 0) return;

                // 3/20/2012 - Items marked TRUE for deleted; otherwise, the rest are items not found.
                DeleteFileCloudItem[] deleteFilesFromCloudSucessful = doWcfEndDeleteFilesFromCloud.Where(item => item.WasDeleted).ToArray();
                    
                //
                // Show Results to listbox
                //
                // Set Header for this section
                ShowFilesDeletedSuccessInListbox(string.Format("Files Deleted Successfully = {0};", deleteFilesFromCloudSucessful.Length), deleteFilesFromCloudSucessful);

                // Clear out the entire CheckedCollections for the cloud, since the restore window is closed.
                var checkedCollection = ((IMainWindow)MainWindowParent).UnityContainer.Resolve<DirectoryTreeViewModelAdapterBase>(ViewModelTypeEnum.CloudViewModel.ToString());
                checkedCollection.ClearAllCheckedItems();

                // 3/17/2012 - Delete 'cloudcache' mirror; these include successfuls and failures due to the file not existing on the cloud.
                CloudCacheDeleter.DeleteCloudCacheItems(doWcfEndDeleteFilesFromCloud);

                // Check for additional batches
                DeleteFilesFromCloud();

            }
            catch (Exception err)
            {
                LogWinEventsHelper.LogErrorMessage("MainWindow.OnEndDeleteFilesFromCloud method threw an exception - " + err.Message);
            }
        }

        // 12/5/2010 - AsyncState used for asynchronous calls.
        private class AsyncDeleteCallObject<TChannel>
        {
            // 5/14/20911
            public List<IFileCheckedItem<IFileItem>> FileCheckedItems;

            /// <summary>
            /// Stores the current WCF factory 'Channel'.
            /// </summary>
            public TChannel CurrentChannel;

            /// <summary>
            /// Stores some misc object state, if any.
            /// </summary>
            public object ResultObject;

        }

        /// <summary>
        /// Occurs when the user closes this window.
        /// </summary>
        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            // Turn Off the progressBar from being top window.
            if (_progressBarService != null)
                _progressBarService.SetTopMostWindow(false);

            _batchCount = 0;
        }
    }
}
