using System.Collections.Generic;
using ImageNexus.BenScharbach.ICLTools.StorageTools.BackupData;

namespace ImageNexus.BenScharbach.ImageCloudLock.Interfaces
{
    // 8/20/2011
    /// <summary>
    /// Inteface <see cref="IBackupTreeViewItems"/>
    /// </summary>
    public interface IBackupTreeViewItems
    {
        /// <summary>
        /// Collection of <see cref="FileBackupItem"/>.
        /// </summary>
        List<FileBackupItem> FilesBackupCollection { get; }

        /// <summary>
        /// Collection of <see cref="DirectoryBackupItem"/>.
        /// </summary>
        List<DirectoryBackupItem> DirectoriesBackupCollection { get; }

        /// <summary>
        /// Sets a given simple array into the internal collection.
        /// </summary>
        /// <param name="files">Array of <see cref="FileBackupItem"/>.</param>
        void SetFilesBackupCollection(FileBackupItem[] files);

        /// <summary>
        /// Sets a given simple array into the internal collection.
        /// </summary>
        /// <param name="directories">Array of <see cref="DirectoryBackupItem"/>.</param>
        void SetDirectoriesBackupCollection(DirectoryBackupItem[] directories);

        /// <summary>
        /// Called by main application when window closing event occurs, 
        /// which in turn, calls the SaveItemsForBackup.
        /// </summary>
        void TriggerSaveItemsForBackup();

        /// <summary>
        /// Retrieves the 'Files' backup list for the given <paramref name="driveName"/>.
        /// </summary>
        /// <param name="driveName">DriveName to obtain the backup list for.</param>
        /// <returns>Collection of <see cref="FileBackupItem"/>.</returns>
        List<FileBackupItem> GetFilesForBackupList(string driveName);

        /// <summary>
        /// Retrieves the 'Directories' backup list for the given <paramref name="driveName"/>.
        /// </summary>
        /// <param name="driveName">DriveName to obtain the backup list for.</param>
        /// <returns>Collection of <see cref="DirectoryBackupItem"/>.</returns>
        List<DirectoryBackupItem> GetDirectoriesForBackupList(string driveName);
    }
}