﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.ServiceModel;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Threading;
using ImageCloudLock;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsMisc;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsThreading;
using Attributes;
using ImageNexus.BenScharbach.Common.AttsInjectorHelper;
using ImageNexus.BenScharbach.Common.ProgressBarInterface;
using ImageNexus.BenScharbach.Common.ProgressBarWindow;
using ImageNexus.BenScharbach.Common.MEFLoggerInterfaces.Interfaces;
using ImageNexus.BenScharbach.Common.ReflectionHelper;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.ViewModel.DirectoryViewModel;
using ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.BaseClasses;
using ImageNexus.BenScharbach.ICLCommon.ICLInterface;
using ImageNexus.BenScharbach.ICLCommon.TreeViewInterface;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.EventArgs;
using ImageNexus.BenScharbach.ICLTools.ICLExtensions;
using ImageNexus.BenScharbach.ICLTools.StorageTools.BackupData;
using ImageNexus.BenScharbach.ImageCloudLock.CloudSetups;
using ImageNexus.BenScharbach.ImageCloudLock.Enums;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;
using ImageNexus.BenScharbach.ImageCloudLock.OSHelper;
using ImageNexus.BenScharbach.ImageCloudLock.OSHelper.Enums;
using ImageNexus.BenScharbach.ImageCloudLock.TaskActions;
using ImageNexus.BenScharbach.ImageCloudLock.TestClasses;
using ImageNexus.BenScharbach.ImageCloudLock.TreeViewItems;
using ImageNexus.BenScharbach.ImageCloudLock.WcfCallbacks;
using Microsoft.Practices.Unity;
// Adds the Unity Extenstions.

/*[assembly: MefLogger(LoggerTypeEnum.EventLog, "ICLFrontEndConsole", LogPropertyCalls = true,
        LoggerVerbosity = LoggerVerbosityEnum.LogEntry | LoggerVerbosityEnum.LogExceptions,
        TraceLoggerOutputPath = @"ICL_Trace_Logs\LoggerCallsOutput.txt", WpfLogWindowInstanceId = 1, AttributeTargetTypes = "ImageCloudLock.*",
        AttributeTargetTypeAttributes = MulticastAttributes.Public | MulticastAttributes.Abstract |
        MulticastAttributes.Protected | MulticastAttributes.Private | MulticastAttributes.Virtual)]*/

namespace ImageNexus.BenScharbach.ImageCloudLock
{
// LoggerVerbosityEnum.LogEntry | LoggerVerbosityEnum.LogExceptions
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window, ILockUnlockUI, IMainWindow
    {
        // 2/15/2011
        private const string ProjectAssemblyLocation =
            @"C:\Program Files (x86)\Image-Nexus\ImageCloudLock Backup Service\ICL\ImageCloudLock.exe";

        // 11/3/2010 - Stores selections made on the current 'Treeview' control for backups.
        private BackupTreeViewItems _backupTreeViewItems;

        // 8/13/2011
        private RestoreCloudWindow _restoreCloudWindow;

        // 10/29/2010 - ICL ProgressBar window.
        private ProgressBarWindow _progressBarWindow;

        // 10/29/2010 - WCF Communication service to CloudAPI.
        // 10/29/2010 - WCF Callbacks (Event handling)
        private IclWebDuplexCallbacks _iclWebDuplexCallbacks;

        // 11/7/2010 - Instantiate a directory TreeView model adapter.
        internal DirectoryTreeViewModelAdapter _directoryTreeViewModelAdapter;

        // 2/18/2011 - Checks if the proper account info was retrieved for the given cloud checked.
        private bool _didCheckAccountInfoPass;

        // 2/20/2011 - Store OS Version
        internal OperatingSystemTypeEnum _osType;

        // 1/23/2012 - Task to Poll the Wcf service.
        private DoPollWcfServiceTask _doPollWcfServiceTask; 
        // 1/23/2012 - Task to Poll the Wcf logger messages.
        private readonly DoPollLoggerMessages _doPollLoggerMessages;

        // 1/25/2012 - MessageWindow
        private MessageWindow _waitMessage;

        #region Events

        /// <summary>
        /// Occurs when the user clicks on the Cancel button request during
        /// some asnychronous operation.
        /// </summary>
        public event EventHandler CancelCurrentOperation;

        // 2/10/2011
        /// <summary>
        /// Occurs when the user clicks on the Delete button.
        /// </summary>
        public event EventHandler DeleteItems;

        #endregion
        
        #region Properties

        // 1/25/2012
        /// <summary>
        /// Gets the <see cref="MessageWindow"/>.
        /// </summary>
        public MessageWindow WaitMessage
        {
            get { return _waitMessage; }
        }

        // 1/24/2012
        /// <summary>
        /// The current <see cref="CloudService"/> in use.
        /// </summary>
        public CloudService CloudServiceInUse { get; internal set; }

        // 9/4/2011 - UnityContainer 2.1
        /// <summary>
        /// Gets the Unity container.
        /// </summary>
        public static UnityContainer UnityContainer { get; internal set; }

        // 12/3/2010
        /// <summary>
        /// Gets the current <see cref="IMainWindow.UnityContainer"/> instance.
        /// </summary>
        UnityContainer IMainWindow.UnityContainer 
        {
            get { return UnityContainer; }
        }

        // 12/2/2010
        /// <summary>
        /// Gets the current instance of <see cref="MiniProgressBar"/>.
        /// </summary>
        public IMiniProgressBar MiniProgressBar { get; set; }

        /// <summary>
        /// Gets the current <see cref="IclWebDuplexCallbacks"/> callback instance.
        /// </summary>
        public IclWebDuplexCallbacks IclWebDuplexCallbacks
        {
            get { return _iclWebDuplexCallbacks; }
        }

        // 12/5/2010
        /// <summary>
        /// Gets the current <see cref="DuplexChannelFactory{IclWebCommService}"/>.
        /// </summary>
        public DuplexChannelFactory<IclWebCommService> IclDuplexChannelFactory { get; set; }

        // 12/5/2010
        /// <summary>
        /// Gets the current communication channel.
        /// </summary>
        public IclWebCommService CurrentChannel { get; set; }

        // 12/6/2010
        // NOTE: The 'Set' is required to keep PostSharp from crashing - bug.
        /// <summary>
        /// Gets the <see cref="ICommunicationObject"/> interface for the <see cref="IclDuplexChannelFactory"/>
        /// instance.  
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public ICommunicationObject CommunicationObject
        {
            get { return IclDuplexChannelFactory; }
            private set { throw new InvalidOperationException(); }
        }

        // 6/19/2011
        /// <summary>
        /// The MEF <see cref="MEFLoggerInterfaces.Interfaces.ILogger"/> required for using the <see cref="MefLoggerAttribute"/>.
        /// </summary>
        public ILogger Logger { get; set; }

        // 7/18/2011
        /// <summary>
        /// Internal field set during startup from the App.xaml when
        /// then '/UseLogger' parameter is passed into the application.
        /// </summary>
        public static bool UseInternalLogger
        {
            get { return DoPollLoggerMessages.UseInternalLogger; }
        }

        #endregion

        #region Constructor

        /// <summary>
        /// Constructor
        /// </summary>
        [DoNotUseLogger]
        //[CatchAllExceptionHandling(false, null, null)]
        public MainWindow()
        {
            InitializeComponent();

            // 1/29/2012 - Lock main window until DoIclStartUpRoutinesTask complete in Windows_Load event.
            IsEnabled = false;

            // Check for Valid License.
            /*var license = new LicenseHelper();
            if (!license.Check(false))
                Application.Current.Shutdown();*/

            // Create Unity container 2.1
            UnityContainer = new UnityContainer();
            UnityContainer.RegisterInstance(typeof(MainWindow), "MainWindowService", this);
            UnityContainer.RegisterType<IShowMessageTestClass, ShowMessageTestClass>("ShowMessageService");

            // 7/18/2011 - 
            // Start LogWindow hidden.
            if (Application.Current.Properties.Contains("UseLogger"))
            {
                SetWpfLoggerVisibility();
                _doPollLoggerMessages = new DoPollLoggerMessages(this);
                _doPollLoggerMessages.StartAsyncPollWcfLoggingMessages();
            }
        }

        // 1/25/2012
        /// <summary>
        /// Occurs when the MainWindow is loaded.
        /// </summary>
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            // Show Wait Message
            _waitMessage = new MessageWindow("Running ICL Start-Up Routines...");
            _waitMessage.Show();

            // 1/25/2012 - Run Start-Up initilization routines in parallel task
            var startUpTasks = new DoIclStartUpRoutinesTask();
            startUpTasks.DoStartUpInitializeRoutines(this, Application.Current);
            startUpTasks.TaskCompleted += startUpTasks_TaskCompleted;
        }

        // 1/25/2012
        /// <summary>
        /// Task completed
        /// </summary>
        /// <param name="obj"></param>
        [CatchAllExceptionHandling(false, null, null)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        private void startUpTasks_TaskCompleted(System.Threading.Tasks.Task<DoIclStartUpRoutinesTask.TaskObjectResult> obj)
        {
            // Close the Start-Up Init-Routines Window
            if (_waitMessage != null)
            {
                _waitMessage.Close();
            }

            // Unlock the main window
            IsEnabled = true;

            // If true, then check AcctInfo and run application
            if (obj.Result.Result)
            {
                CheckAccountInfoExist();
                return;
            }

            return;

            // If failed, Check results
            switch (obj.Result.InitComponent)
            {
                case DoIclStartUpRoutinesTask.InitComponentEnum.CheckForXpOs:
                case DoIclStartUpRoutinesTask.InitComponentEnum.StartDirectoryTreeViewTask:
                case DoIclStartUpRoutinesTask.InitComponentEnum.SetEventHandlersForCallback:
                case DoIclStartUpRoutinesTask.InitComponentEnum.CreateServiceCallbacks:
                case DoIclStartUpRoutinesTask.InitComponentEnum.CreateWcfPoller:
                case DoIclStartUpRoutinesTask.InitComponentEnum.CreateServiceChannels:
                case DoIclStartUpRoutinesTask.InitComponentEnum.CreateBackupTreeView:
                case DoIclStartUpRoutinesTask.InitComponentEnum.CreateItemTemplate:
                    throw new InvalidOperationException(string.Format("Startup Init-Tasks failed: {0}.",obj.Result.Message));
                    break;
                default:
                    throw new ArgumentOutOfRangeException();
            }
            
        }

        // 1/25/2012
        /// <summary>
        /// Sets the ItemTemplates type in the TreeViewUserControl.
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(bool), false)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        internal bool SetItemTemplatesType()
        {
            treeViewUserControl1.SetItemTemplatesType(true);

            return true;
        }

        // 1/25/2012
        /// <summary>
        /// Sets the WPF logger visibility.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        internal void SetWpfLoggerVisibility()
        {
            miLogger.Visibility = Visibility.Visible;
        }

        // 1/25/2012
        /// <summary>
        /// Creates the instance of <see cref="IclWebDuplexCallbacks"/>
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(bool), false)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        internal bool CreateIclWebDuplexCallbacks()
        {
            _iclWebDuplexCallbacks = new IclWebDuplexCallbacks();

            return true;
        }

        // 1/25/2012
        /// <summary>
        /// Creates the instance of the <see cref="CreateDoPollWcfServiceTask"/>.
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(bool), false)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        internal bool CreateDoPollWcfServiceTask()
        {
            _doPollWcfServiceTask = new DoPollWcfServiceTask(this, _iclWebDuplexCallbacks);
            _doPollWcfServiceTask.StartAsyncPollWcfService();

            return true;
        }

        // 1/25/2012
        /// <summary>
        /// Creates the instance of the <see cref="BackupTreeViewItems"/>.
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(bool), false)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        internal bool CreateBackupTreeViewItems()
        {
            _backupTreeViewItems = new BackupTreeViewItems(this, _directoryTreeViewModelAdapter);
            Application.Current.Properties.Add("BackupTreeViewItems", _backupTreeViewItems);

            return true;
        }

        // 1/25/2012
        /// <summary>
        /// Starts the DirectoryTreeView tasks.
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(bool), false)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        internal bool StartDirectoryTreeViewTasks()
        {
            DirectoryTreeViewTasks.CreateDirectoryTreeViewTask(new DirectoryTreeViewTasks.DirectoryTreeViewTaskParams
            {
                MainWindow = this,
                ListViewInstance = treeViewUserControl1.DirectoryListView,
                TreeViewInstance = treeViewUserControl1.DetailsTreeView,
                TreeViewUserControl = treeViewUserControl1
            });

            // 3/19/2012 - Add eventHandler to splitter moved
            treeViewUserControl1.SplitterMoved += treeViewUserControl1_SplitterMoved;
            // 3/20/2012 - Set prior TreeView container width
            treeViewUserControl1.TreeViewContainerWidth = new GridLength((double)GetPropertySetting(PropertySettingEnums.TreeView1ContainerWidth));
            // 3/20/2012 - Set margin of btnCancel; 200 is subtracted, which is the MinWidth of the TreeViewContainerWidth
            btnCancel.Margin = new Thickness(treeViewUserControl1.TreeViewContainerWidth.Value - 200,
                                             btnCancel.Margin.Top,
                                             btnCancel.Margin.Right,
                                             btnCancel.Margin.Bottom);
            // 3/20/2012 - Set MaxWidth percent for the TreeView1Container width
            treeViewUserControl1.TreeViewContainerMaxWidthPercent = 70;

            return true;
        }
        
        // 3/19/2012
        /// <summary>
        /// Occurs when the TreeView's splitter is moved.
        /// </summary>
        private void treeViewUserControl1_SplitterMoved(object sender, SplitterEventArgs e)
        {
            var currentMarginLeft = btnCancel.Margin.Left + e.TreeViewSplitterHorizontalChange;
            currentMarginLeft = (currentMarginLeft < 0) ? 0 : currentMarginLeft;

            // Update cancel button margin
            btnCancel.Margin = new Thickness(currentMarginLeft, btnCancel.Margin.Top, btnCancel.Margin.Right,
                                             btnCancel.Margin.Bottom);
            // 3/20/2012 - save width
            Properties.Settings.Default.TreeView1ContainerWidth = treeViewUserControl1.TreeViewContainerWidth.Value;
        }

        // 3/21/2012
        /// <summary>
        /// Occurs when the Show files checked is clicked by user.
        /// </summary>
        private void btnShowFilesChecked_Click(object sender, RoutedEventArgs e)
        {
            string currentDriveSelected = _directoryTreeViewModelAdapter.TreeViewUserControl.CurrentDriveSelected;
            if (string.IsNullOrEmpty(currentDriveSelected))
            {
                return;
            }

            // Retrieve files checked for backup
            List<FileBackupItem> filesForBackupList = _backupTreeViewItems.GetFilesForBackupList(currentDriveSelected);

            if (filesForBackupList == null)
            {
                return;
            }

            if (filesForBackupList.Count == 0)
            {
                return;
            }

            // Open Checked list window
            var filesCheckedWindow = new FilesCheckedWindow(filesForBackupList);
            filesCheckedWindow.ShowDialog();
        }

        // 1/25/2012
        /// <summary>
        /// Set EventHandler for Cancel operation
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(bool), false)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        internal bool SetEventHandlerForCancelOrder()
        {
            CancelCurrentOperation += DirectoryInfoViewModel.CancelAsyncOperation;

            return true;
        }

        // 1/23/2012
        [DoNotUseLogger]
        [CatchAllExceptionHandling(false, null, null)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = true)]
        public void SetStatusEllipseColor(Color color)
        {
            wcfServiceStatusEllipse.Fill = new SolidColorBrush { Color = color };
        }

        // 11/2/2011
        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        internal void SetCloudService(IclWebCommService currentChannel, CloudService cloudService)
        {
            // Set to use the 'Internal Test DB' service
            currentChannel.UseCloudServiceType(cloudService);
        }

        // 8/7/2011
        /// <summary>
        /// Helper method which creates the <see cref="DuplexChannelFactory{T}"/>"/> and
        /// sets all default settings, like to use compression.
        /// </summary>
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        [CatchAllExceptionHandling(true, typeof(bool), false)]
        internal bool CreateChannelFactoryAndSetDefaultValues()
        {

            IclDuplexChannelFactory = new DuplexChannelFactory<IclWebCommService>(
                new InstanceContext(IclWebDuplexCallbacks), "NetNamedPipeBinding_IclWebCommService");
            IclDuplexChannelFactory.Open(); // 12/5/2010 - Good Practice to Explicilty open channel.
            CurrentChannel = IclDuplexChannelFactory.CreateChannel();

            // Note: 1st param can be null, since it will be set by postsharp.
            if (!DoWcfUseCompression(null, cbUseCompression.IsChecked))
                return false;

            // Note: 1st param can be null, since it will be set by postsharp.
            if (!DoWcfUseCompressionType(null, CompressionType.MicrosoftGZipStream))
                return false;

            // Note: 1st param can be null, since it will be set by postsharp.
            if (!DoWcfTransferInChunks(null, cbUseFileChunks.IsChecked))
                return false;

            return true;
        }

        // 1/31/2012
        /// <summary>
        /// Sets the Wcf DoCompression.
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(bool), false)]
        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private bool DoWcfUseCompression(IclWebCommService currentChannel, bool useCompression)
        {
            currentChannel.DoCompression(useCompression);

            return true;
        }

        // 1/31/2012
        /// <summary>
        /// Sets the Wcf Use-Compression type.
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(bool), false)]
        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private bool DoWcfUseCompressionType(IclWebCommService currentChannel, CompressionType microsoftGZipStream)
        {
            // Set Compression Algorithm
            currentChannel.UseCompressionType(microsoftGZipStream);

            return true;
        }

        // 1/31/2012
        /// <summary>
        /// Sets the Wcf transfer in chunks
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(bool), false)]
        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private bool DoWcfTransferInChunks(IclWebCommService currentChannel, bool isChecked)
        {
            // Transfer in Chunks?
            currentChannel.DoTransferInChunks(isChecked);

            return true;
        }

        // 2/20/2011
        /// <summary>
        /// Checks if windowsXp, and turns off scheduling.
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(bool), false)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        internal bool CheckIfWindowsXp()
        {
            _osType = OsInfoHelper.GetOsInfo();

            if (_osType != OperatingSystemTypeEnum.WindowsXp) return true;

            // Turn off schedule buttons
            btnScheduler.Visibility = Visibility.Hidden;
            btnScheduleBackups.Visibility = Visibility.Hidden;

            return true;
        }

        // 2/18/2011; 4/29/2011 - Updated to check DevPay credentials
        /// <summary>
        /// Checks if the cloud account checked has the proper account information set, before
        /// calling any WCF service calls.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        internal bool CheckAccountInfoExist()
        {
            // 12/13/2012 - Check if developer mode.
            if (cbDevMode.IsChecked)
                return true;

            // If logic already done, then just return.
            if (_didCheckAccountInfoPass)
            {
                return true;
            }

#if Amazon
            // Check if using AmazonS3 cloud service
            //if (CloudServiceInUse == CloudService.AmazonS3)
            {
                // NOTE: Non-DevPay check below
                // Check if reg values are empty or null
                /*string amazonAccessKey, amazonSecretAccessKey;
                GetAmazonS3AcctInfo(out amazonAccessKey, out amazonSecretAccessKey);

                if (string.IsNullOrEmpty(amazonAccessKey) || string.IsNullOrEmpty(amazonSecretAccessKey))
                {
                    // Show SetAmazonS3AcctInfo
                    var amazonAcctInfo = new SetAmazonS3AcctInfo(this);

                    // NOTE: 'ShowDialog' creates a Modal WPF window (special window type), 
                    // NOTE:  which disables all other windows until this window is closed.
                    amazonAcctInfo.ShowDialog();
                }*/

                // 4/29/2011 - Check for Amazon Credientials
                if (!SetAmazonDevPay.AmazonFileCredentialStore.CredentialsExist)
                {
                    // Show SetAmazonDevPay window
                    var amazonDevPay = new SetAmazonDevPay(this);

                    // NOTE: 'ShowDialog' creates a Modal WPF window (special window type), 
                    // NOTE:  which disables all other windows until this window is closed.
                    amazonDevPay.ShowDialog();
                }

                _didCheckAccountInfoPass = SetAmazonDevPay.AmazonFileCredentialStore.DoesFileStoreExist();
                return _didCheckAccountInfoPass;
            }
             
#endif

#if Azure
            // 1/17/2012 - Check if using Azure cloud service
            //if (CloudServiceInUse == CloudService.Azure)
            {
                // Check for Azure Credientials
                if (!SetAzureAcctInfo.AzureFileCredentialStore.CredentialsExist)
                {
                    // Show SetAzureAcctInfo window
                    var azureAcctInfo = new SetAzureAcctInfo(this);

                    // NOTE: 'ShowDialog' creates a Modal WPF window (special window type), 
                    // NOTE:  which disables all other windows until this window is closed.
                    azureAcctInfo.ShowDialog();
                }

                _didCheckAccountInfoPass = SetAzureAcctInfo.AzureFileCredentialStore.DoesFileStoreExist();
                return _didCheckAccountInfoPass;
            }
#endif
            return false;
        }


        /// <summary>
        /// Retrieves the Amazon account keys from the registry.
        /// </summary>
        /// <param name="accessKey">(OUT) Amazon Access key</param>
        /// <param name="secretAccessKey">(OUT) Amazon Secret Access key</param>
        private void GetAmazonS3AcctInfo(out string accessKey, out string secretAccessKey)
        {
            // Create collection of Registry keys to pass to WCF service
            var cloudAccountRegistry = new WcfCloudAccountRegistry
                                           {
                                               WcfCloudAccountRegistryKeys = new WcfCloudAccountRegistryKeys[2]
                                           };

            // Create Each Registry Record
            cloudAccountRegistry.WcfCloudAccountRegistryKeys[0] = new WcfCloudAccountRegistryKeys
                                                                      {
                                                                          RegistryName =
                                                                              CloudAcctRegistryEnums.AmazonAccessKey
                                                                      };
            cloudAccountRegistry.WcfCloudAccountRegistryKeys[1] = new WcfCloudAccountRegistryKeys
                                                                      {
                                                                          RegistryName =
                                                                              CloudAcctRegistryEnums.AmazonSecretAccessKey
                                                                      };

            // 8/28/2011 - Note: 1st param can be null, since it will be set by postsharp.
            cloudAccountRegistry = DoWCFGetAmazonAccountRegistryKeys(null, cloudAccountRegistry);

            // Set Values
            accessKey = (string) cloudAccountRegistry.WcfCloudAccountRegistryKeys[0].RegistryValue;
            secretAccessKey = (string) cloudAccountRegistry.WcfCloudAccountRegistryKeys[1].RegistryValue;
        }

        // 8/28/2011
        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private WcfCloudAccountRegistry DoWCFGetAmazonAccountRegistryKeys(IclWebCommService currentChannel, WcfCloudAccountRegistry cloudAccountRegistry)
        {
            // Retrieve current account info, if any, and populate fields.
            cloudAccountRegistry = currentChannel.GetCloudAccountRegistryKeys(cloudAccountRegistry);
            return cloudAccountRegistry;
        }

        #endregion

        #region OnGui Method Helpers

        // 2/10/2011
        /// <summary>
        /// OnGui WPF UI thread call.
        /// </summary>
        /// <typeparam name="T">Generic Type</typeparam>
        /// <param name="action">Action delegate</param>
        /// <param name="paramItem">Parameter to pass into Action delegate</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void OnGuiThreadMethodCall<T>(Action<T> action, T paramItem) where T : class
        {
            // The Work to perform on another thread
            ThreadStart start = delegate()
                                    {
                                        // ...

                                        // This will work as its using the dispatcher
                                        var op = Dispatcher.BeginInvoke(
                                            DispatcherPriority.Normal,
                                            action,
                                            paramItem);

                                        var status = op.Status;
                                        while (status != DispatcherOperationStatus.Completed)
                                        {
                                            status = op.Wait(TimeSpan.FromMilliseconds(1000));
                                            if (status == DispatcherOperationStatus.Aborted)
                                            {
                                                // Alert Someone
                                            }
                                        }
                                    };

            // Create the thread and kick it started!
            new Thread(start).Start();

        }

        // 2/10/2011
        /// <summary>
        /// OnGui WPF UI thread call.
        /// </summary>
        /// <param name="action">Action delegate</param>
        //[CatchAllExceptionHandling(false, null, null)]
        public void OnGuiThreadMethodCall(Action action)
        {
            // The Work to perform on another thread
            ThreadStart start = delegate()
                                    {
                                        // ...

                                        // This will work as its using the dispatcher
                                        var op = Dispatcher.BeginInvoke(
                                            DispatcherPriority.Normal,
                                            action);

                                        var status = op.Status;
                                        while (status != DispatcherOperationStatus.Completed)
                                        {
                                            status = op.Wait(TimeSpan.FromMilliseconds(1000));
                                            if (status == DispatcherOperationStatus.Aborted)
                                            {
                                                // Alert Someone
                                            }
                                        }
                                    };

            // Create the thread and kick it started!
            new Thread(start).Start();

        }

        #endregion


        #region Footer Mini-ProgressBar methods

        // 8/14/2011
        // 11/14/2010; 11/15/2010 - Updated with [OnGuiThread] attribute.
        /// <summary>
        /// Helper method which resets the Footer's mini-progressBar.
        /// </summary>
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = true)]
        [CatchAllExceptionHandling(false, null, null)]
        public void ResetFooterProgressBar(object sender, ProgressEventArgs e)
        {
            if (!(sender is DirectoryInfoViewModel)) return;

            // 11/14/2010 - setup footer progressbar
            footerProgressBar.Value = 0;
            footerProgressBar.Maximum = e.Value;
            footerProgressBar.Visibility = Visibility.Visible;
            footerTextInfo.Text = "Populating...";
        }

        // 12/14/2010: Test 'Asynchronous' mode when updating UI.
        // 11/14/2010; 11/15/2010 - Updated with [OnGuiThread] attribute.
        /// <summary>
        /// Helper method which set the current value into the footer's mini-progressBar.
        /// </summary>
        [OnGuiThread(DispatcherPriority.Send, Asynchronous = true)]
        [CatchAllExceptionHandling(false, null, null)]
        public void SetFooterProgress(object sender, ProgressEventArgs e)
        {
            if (!(sender is DirectoryInfoViewModel)) return;

            footerProgressBar.Value = e.Value;

            if (!btnCancel.IsVisible)
                btnCancel.Visibility = Visibility.Visible; // 2/20/2011
        }

        // 11/14/2010; 11/15/2010 - Updated with [OnGuiThread] attribute.
        /// <summary>
        /// Helper method which clears the footer text and hides the progress bar.
        /// </summary>
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = true)]
        [CatchAllExceptionHandling(false, null, null)]
        public void EndFooterProgressBar(object sender, EventArgs e)
        {
            if (!(sender is DirectoryInfoViewModel)) return;

            footerTextInfo.Text = "";
            footerProgressBar.Visibility = Visibility.Hidden;
            btnCancel.Visibility = Visibility.Hidden; // 2/20/2011
        }

        // 12/3/2010
        /// <summary>
        /// Helper method which sets the footer's Header control.
        /// </summary>
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = true)]
        [CatchAllExceptionHandling(false, null, null)]
        public void SetFooterHeaderText(object sender, ProgressEventArgs e)
        {
            if (!(sender is DirectoryInfoViewModel)) return;

            footerTextInfo.Text = e.Message;
            btnCancel.Visibility = Visibility.Visible; // 2/20/2011
        }

        #endregion
        

        // 11/1/2010; 12/6/2010 - Updated to use the 'ShowDialog'.
        /// <summary>
        /// Shows the 'Schedule Backups' window.
        /// </summary>
        [ShowWaitCursor]
        [CatchAllExceptionHandling(false, null, null)]
        // ReSharper disable InconsistentNaming
        private void btnScheduleBackups_Click(object sender, RoutedEventArgs e)
            // ReSharper restore InconsistentNaming
        {
            // Then show the 'Schedule Backups' window.
            ShowSchedulerWindow();

        }

        // 11/9/2010 - Multi-Function button, which changes depending on tab focus for either 'Backups' or 'Restores'.
        [CatchAllExceptionHandling(false, null, null)]
        private void btnMultiFunction_Click(object sender, RoutedEventArgs e)
        {
            // If 'Backup' tab is focused, then multiFunction is for 'BackupNow'
            if (!tabBackup.IsSelected) return;

            // 2/18/2011; 4/29/2011 - Returns if false
            if (!CheckAccountInfoExist()) return;

            // Save any possible changes user made to current backupList before opening next window.
            if (_backupTreeViewItems != null)
                _backupTreeViewItems.TriggerSaveItemsForBackup();

            // Show BackupNow Window
            var backupNowWindow = new BackupNowWindow(this);

            // 12/6/2010 - Updated to use 'ShowDialog', rather than 'Show'.
            // NOTE: 'ShowDialog' creates a Modal WPF window (special window type), 
            // NOTE:  which disables all other windows until this window is closed.
            backupNowWindow.ShowDialog();

        }

        // 11/3/2010; 12/6/2010 - Updated to use the 'ShowDialog'
        /// <summary>
        /// Occurs when the Single-File upload menuitem is selected.
        /// </summary>
        // ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void MenuItem_menuUploadSingleFile(object sender, RoutedEventArgs e)
            // ReSharper restore InconsistentNaming
        {
            // 2/18/2011; 4/29/2011 - Returns if false
            if (!CheckAccountInfoExist()) return;

            // Show 'Single-File' window to user.
            var uploadFileWindow = new UploadFileWindow(this);

            // 12/6/2010 - Updated to use 'ShowDialog', rather than 'Show'.
            // NOTE: 'ShowDialog' creates a Modal WPF window (special window type), 
            // NOTE:  which disables all other windows until this window is closed.
            uploadFileWindow.ShowDialog();
        }

        // 11/3/2010; 12/6/2010 - Updated to use the 'ShowDialog'
        /// <summary>
        /// Occurs when the Single-Directory upload menuitem is selected.
        /// </summary>
        // ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void MenuItem_menuUploadSingleDirectory(object sender, RoutedEventArgs e)
            // ReSharper restore InconsistentNaming
        {
            // 2/18/2011; 4/29/2011 - Returns if false
            if (!CheckAccountInfoExist()) return;

            // Show 'Directory' window to user.
            var uploadDirectoryWindow = new UploadDirectoryWindow(this);

            // 12/6/2010 - Updated to use 'ShowDialog', rather than 'Show'.
            // NOTE: 'ShowDialog' creates a Modal WPF window (special window type), 
            // NOTE:  which disables all other windows until this window is closed.
            uploadDirectoryWindow.ShowDialog();
        }

        // 11/3/2010
        /// <summary>
        /// Occurs when the user closes the MainWindow of the application.
        /// </summary>
        // ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
            // ReSharper restore InconsistentNaming
        {
            // Show Wait Message
            _waitMessage = new MessageWindow("Running ICL Shut-down Routines...");
            _waitMessage.Show();

            // 1/25/2012 - Run Start-Up initilization routines in parallel task
            var shutDownTasks = new DoIclShutDownRoutinesTask();
            shutDownTasks.DoShutDownRoutines(this, Application.Current);
            shutDownTasks.TaskCompleted += shutDownTasks_TaskCompleted;
        }

        // 1/25/2012
        /// <summary>
        /// Occurs when the <see cref="DoIclShutDownRoutinesTask"/> completes.
        /// </summary>
        /// <param name="obj"></param>
        [CatchAllExceptionHandling(false, null, null)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        private void shutDownTasks_TaskCompleted(System.Threading.Tasks.Task obj)
        {
            if (_waitMessage != null)
            {
                _waitMessage.Close();
            }

            // Shutdown this application
            Application.Current.Shutdown();
        }

        // 1/25/2012
        /// <summary>
        /// Dispose of Mini-ProgressBar.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        internal void DisposePollLoggerMessages()
        {
            if (_doPollLoggerMessages != null)
            {
                _doPollLoggerMessages.CloseLogWindow();
            }
        }

        // 1/25/2012
        /// <summary>
        /// Dispose of Mini-ProgressBar.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        internal void DisposeOfMiniProgressBar()
        {
            // Stop UI Thread, since operation completed.
            if (MiniProgressBar != null)
            {
                MiniProgressBar.CloseWindow();
            }
        }

        // 1/25/2012
        /// <summary>
        /// Save users settings.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        internal void DoSaveUserSettings()
        {
            Properties.Settings.Default.Save();
            if (treeViewUserControl1 != null)
            {
                treeViewUserControl1.SaveUserSettings();
            }
        }

        // 1/25/2012
        /// <summary>
        /// Set pointers to nulls.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        internal void NullPointerRefs()
        {
            _iclWebDuplexCallbacks = null;
            CurrentChannel = null;
            _progressBarWindow = null;
            _backupTreeViewItems = null;
        }

        // 1/25/2012
        /// <summary>
        /// Dispose of poll Wcf service.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        internal void CloseWcfChannels()
        {
            if (CurrentChannel != null)
            {
                ((ICommunicationObject) CurrentChannel).CloseConnection();
            }
        }

        // 1/25/2012
        /// <summary>
        /// Dispose of poll Wcf service.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        internal void DisposeOfPollWcfService()
        {
            if (_doPollWcfServiceTask != null)
            {
                _doPollWcfServiceTask.Cancel();
            }
        }

        // 1/25/2012
        /// <summary>
        /// Dispose of poll logger messages service.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        internal void DisposeOfPollLoggerMessages()
        {
            if (_doPollLoggerMessages != null)
            {
                _doPollLoggerMessages.Cancel();
            }
        }

        // 1/25/2012
        /// <summary>
        /// Dispose of the progress bar window.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        internal void DisposeOfProgressBarWindow()
        {
            if (_progressBarWindow != null)
            {
                // Note: Required, in order for the 'Close' command to take affect.
                _progressBarWindow.ApplicationClosing = true;
                _progressBarWindow.Close();
            }
        }

        // 1/25/2012
        /// <summary>
        /// Saves the backup checked items.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        internal void SaveBackupCheckedItems()
        {
            if (_backupTreeViewItems != null)
            {
                _backupTreeViewItems.TriggerSaveItemsForBackup();
            }
        }

        [CatchAllExceptionHandling(false, null, null)]
        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        internal void FlushAndCloseAllTraceListeners(IclWebCommService currentChannel)
        {
            // Close on Service
            currentChannel.FlushAndCloseAllTraceListeners();

            // Close here
            if (MefLoggerAttribute.Logger != null)
            {
                MefLoggerAttribute.Logger.FlushAndCloseAllTraceListeners();
            }
        }

        // 10/4/2011
        [CatchAllExceptionHandling(false, null, null)]
        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        internal void ClearBackupList(IclWebCommService currentChannel)
        {
            // Clear all backup lists from service
            currentChannel.ClearBackupLists();
        }

        // 11/3/2010
        /// <summary>
        /// Occurs when the user selects the 'Use Compression' menuItem.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        // ReSharper disable InconsistentNaming
        private void cbUseCompression_Click(object sender, RoutedEventArgs e)
        // ReSharper restore InconsistentNaming
        {
            // Note: 1st param can be null, since it will be set by postsharp.
            DoWCFDoCompression(null);   
        }

        // 1/4/2012
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private void DoWCFDoCompression(IclWebCommService currentChannel)
        {
            // Set to do compression in WCF service.
            currentChannel.DoCompression(cbUseCompression.IsChecked);
        }

        // 11/3/2010
        /// <summary>
        /// Occurs when the user selects the 'Windows G-Zip' menuItem.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        // ReSharper disable InconsistentNaming
        private void cbCompressionGZip_Click(object sender, RoutedEventArgs e)
            // ReSharper restore InconsistentNaming
        {
            // Note: 1st param can be null, since it will be set by postsharp.
            DoWCFUseCompressionType(null, (cbCompressionGZip.IsChecked)
                                              ? CompressionType.MicrosoftGZipStream
                                              : CompressionType.DotNetZip);
        }

        // 11/3/2010
        /// <summary>
        /// Occurs when the user selects the 'DotNetZip' menuItem.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        // ReSharper disable InconsistentNaming
        private void cbCompressionDotNetZip_Click(object sender, RoutedEventArgs e)
            // ReSharper restore InconsistentNaming
        {
            // Note: 1st param can be null, since it will be set by postsharp.
            DoWCFUseCompressionType(null, (cbCompressionDotNetZip.IsChecked)
                                              ? CompressionType.DotNetZip
                                              : CompressionType.MicrosoftGZipStream);
        }

        // 1/4/2012
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private void DoWCFUseCompressionType(IclWebCommService currentChannel, CompressionType useCompressionTypeEnum)
        {
            // Set to do compression in WCF service.
            currentChannel.UseCompressionType(useCompressionTypeEnum);
        }

        // 11/3/2010
        /// <summary>
        /// Occurs when the user selects the 'Use Encryption' menuItem.
        /// </summary>
        // ReSharper disable InconsistentNaming
        private void cbUseEncryption_Click(object sender, RoutedEventArgs e)
            // ReSharper restore InconsistentNaming
        {
            // Set to do encyption in WCF service.
            // TODO: Need to implement.
        }

        // 11/3/2010
        /// <summary>
        /// Occurs when the user selects the 'Use File Chunks' menuItem.
        /// </summary>
        // ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void cbUseFileChunks_Click(object sender, RoutedEventArgs e)
            // ReSharper restore InconsistentNaming
        {
            // Note: 1st param can be null, since it will be set by postsharp.
            DoWcfTransferInChunks(null, cbUseFileChunks.IsChecked);    
        }

        // 1/4/2012
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private void DoWCFUseCloudService(IclWebCommService currentChannel, CloudService useCloudService)
        {
            // Set to use the given cloud service
            currentChannel.UseCloudServiceType(useCloudService);
        }

        // 12/3/2010
        /// <summary>
        /// Occurs when the user clicks the Cancel button in the footer window section.
        /// </summary>
        // ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void btnCancel_Click(object sender, RoutedEventArgs e)
            // ReSharper restore InconsistentNaming
        {
            // trigger event
            if (CancelCurrentOperation != null)
                CancelCurrentOperation(this, EventArgs.Empty);
        }

        // 12/6/2010
        /// <summary>
        /// Occurs when user selects the 'Start' action in the contextMenu of 
        /// the Ellipse control.
        /// </summary>
        [ShowWaitCursor]
        [CatchAllExceptionHandling(false, null, null)]
        // ReSharper disable InconsistentNaming
        private void menuItem_StartService_Click(object sender, RoutedEventArgs e)
            // ReSharper restore InconsistentNaming
        {
            _doPollWcfServiceTask.StartWcfService();
        }

        // 12/6/2010
        /// <summary>
        /// Occurs when user selects the 'Stop' action in the contextMenu of 
        /// the Ellipse control.
        /// </summary>
        [ShowWaitCursor]
        [CatchAllExceptionHandling(false, null, null)]
        // ReSharper disable InconsistentNaming
        private void menuItem_StopService_Click(object sender, RoutedEventArgs e)
            // ReSharper restore InconsistentNaming
        {
            _doPollWcfServiceTask.StopWcfService();
        }

        // 7/2/2010 - TEST MEF WPF Logger
        private void miTest_Click(object sender, RoutedEventArgs e)
        {

        }

        // 12/12/2010
        /// <summary>
        /// Occurs when user clicks the Scheduler button.
        /// </summary>
        // ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void btnScheduler_Click(object sender, RoutedEventArgs e)
            // ReSharper restore InconsistentNaming
        {
            // Scheduler Window
            ShowSchedulerWindow();
        }

        // 2/10/2011
        /// <summary>
        /// Helper method which shows the scheduler window.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void ShowSchedulerWindow()
        {
            var scheduler = new SchedulerWindow(this);
            scheduler.ShowDialog();
        }

        // 12/14/2010
        /// <summary>
        /// Locks the MainWindow UI during concurrent operation.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        [OnGuiThread(DispatcherPriority.Normal)]
        public void LockUI()
        {
            // Lock TreeView1
            treeViewUserControl1.LockUnLockDrivesUi(false);
            treeViewUserControl1.LockUnLockTreeViewUi(false);
        }

        // 12/14/2010
        /// <summary>
        /// UnLocks the MainWindow UI during concurrent operation.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        [OnGuiThread(DispatcherPriority.Normal)]
        public void UnLockUI()
        {
            // Lock TreeView1
            treeViewUserControl1.LockUnLockDrivesUi(true);
            treeViewUserControl1.LockUnLockTreeViewUi(true);

        }

        // 5/14/2011
        /// <summary>
        /// Triggers the delete items event.
        /// </summary>
        internal void OnDeleteItems()
        {
            // 2/10/2011 - Trigger Event to delete items from display
            if (DeleteItems != null)
                DeleteItems(this, EventArgs.Empty);
        }

        // 2/14/2011
        /// <summary>
        /// Shows the About Window.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void miAbout_Click(object sender, RoutedEventArgs e)
        {
            // Get Project Assembly
            Assembly projectAssembly;
            ReflectionHelperTool.GetAssembly(ProjectAssemblyLocation, out projectAssembly);

            // Show About window
            var about = new About(projectAssembly);


            // NOTE: 'ShowDialog' creates a Modal WPF window (special window type), 
            // NOTE:  which disables all other windows until this window is closed.
            about.ShowDialog();
        }

        // 2/17/2011
        /// <summary>
        /// Shows the AmazonS3 AccountInfo Window.
        /// </summary>
        private void menuSetAmazonS3AcctInfo_Click(object sender, RoutedEventArgs e)
        {
            // Show SetAmazonS3AcctInfo
            var amazonAcctInfo = new SetAmazonS3AcctInfo(this);

            // NOTE: 'ShowDialog' creates a Modal WPF window (special window type), 
            // NOTE:  which disables all other windows until this window is closed.
            amazonAcctInfo.ShowDialog();
        }

        // 4/11/2011
        /// <summary>
        /// Shows the AmazonS3 DevPay Activation Window.
        /// </summary>
        private void menuSetProductActivationKey_Click(object sender, RoutedEventArgs e)
        {
            // Show SetAmazonDevPay
            var amazonDevPay = new SetAmazonDevPay(this);

            // NOTE: 'ShowDialog' creates a Modal WPF window (special window type), 
            // NOTE:  which disables all other windows until this window is closed.
            amazonDevPay.ShowDialog();
        }

        // 1/17/2012
        /// <summary>
        /// Shows the Azure Activation Window.
        /// </summary>
        private void menuSetAzureProductActivationKey_Click(object sender, RoutedEventArgs e)
        {
            // Show SetAzureAcctInfo
            var azureAcctInfo = new SetAzureAcctInfo(this);

            // NOTE: 'ShowDialog' creates a Modal WPF window (special window type), 
            // NOTE:  which disables all other windows until this window is closed.
            azureAcctInfo.ShowDialog();
        }

        // 7/18/2011
        /// <summary>
        /// Shows the LoggerWindow.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void miLogger_Click(object sender, RoutedEventArgs e)
        {
            if (_doPollLoggerMessages != null)
            {
                _doPollLoggerMessages.ShowLogWindow();
            }
        }

        /// <summary>
        /// Occurs when the 'Restore' tab is selected, which then calls the asynchronous retrieve root
        /// directories operation from the cloud.  Furthermore, the 'Restore' button is shown and the
        /// 'Backup' button is hidden.
        /// </summary>
        // ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void mainTabControl_SelectionChanged(object sender, SelectionChangedEventArgs e)
            // ReSharper restore InconsistentNaming
        {
            // check if 'SelectedIndex' is 'Backup' tab.
            switch (mainTabControl.SelectedIndex)
            {
                case 0: // 'SelectedIndex' is 'Backup' tab.

                    // Restore the schedule button.
                    if (_osType != OperatingSystemTypeEnum.WindowsXp)
                        btnScheduleBackups.Visibility = Visibility.Visible;

                    // Update the content for the multifunction button.
                    btnMultiFunction.Content = "Backup Now";


                    break;
            }
        }

        // 8/20/2011
        /// <summary>
        /// Starts the Restore window.
        /// </summary>
        private void btnRestore_Click(object sender, RoutedEventArgs e)
        {
            if (_restoreCloudWindow != null)
            {
                _restoreCloudWindow.Show();
                return;
            }

            _restoreCloudWindow = new RestoreCloudWindow(this);
            _restoreCloudWindow.Closed += _restoreCloudWindow_Closed;
            _restoreCloudWindow.Show();
        }

        // 8/19/2011
        /// <summary>
        /// Occurs when the <see cref="RestoreCloudWindow"/> closed.
        /// </summary>
        private void _restoreCloudWindow_Closed(object sender, EventArgs e)
        {
            _restoreCloudWindow = null;
        }

        /// <summary>
        /// Gets the current cloud files for the given directory.
        /// </summary>
        /// <param name="directoryItemViewModel">Instance of <see cref="IDirectoryItemViewModel"/></param>
        /// <param name="directoryName">The directory name to retrieve the files for.</param>
        /// <returns>Collection of <see cref="IFileItem"/></returns>
        public List<IFileItem> GetCloudFilesForDirectory(IDirectoryItemViewModel directoryItemViewModel,
                                                         string directoryName)
        {
            return _restoreCloudWindow.GetCloudFilesForDirectory(directoryItemViewModel, directoryName);
        }

        /// <summary>
        /// Gets the current cloud directories for the given directory.
        /// </summary>
        /// <param name="directoryItemViewModel">Instance of <see cref="IDirectoryItemViewModel"/></param>
        /// <param name="directoryName">The directory name to retrieve the directories for.</param>
        /// <returns>Collection of <see cref="IDirectoryItem"/></returns>
        public List<IDirectoryItem> GetCloudDirectoriesForDirectory(IDirectoryItemViewModel directoryItemViewModel,
                                                                    string directoryName)
        {
            return _restoreCloudWindow.GetCloudDirectoriesForDirectory(directoryItemViewModel, directoryName);
        }

        // 1/17/2012
        /// <summary>
        /// Occurs when the Azure development mode is checked on.
        /// </summary>
        private void cbDevMode_Checked(object sender, RoutedEventArgs e)
        {
            // 1/27/2012 - Save settings.
            DoSaveUserSettings();

            // Note: 1st param can be null, since it will be set by postsharp.
            DoWCFSetDevelopmentMode(null, true);
        }

        // 1/17/2012
        /// <summary>
        /// Occurs when the Azure development mode is checked off.
        /// </summary>
        private void cbDevMode_Unchecked(object sender, RoutedEventArgs e)
        {
            // 1/27/2012 - Save settings.
            DoSaveUserSettings();

            // Note: 1st param can be null, since it will be set by postsharp.
            DoWCFSetDevelopmentMode(null, false);
        }

        // 1/17/2012
        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private void DoWCFSetDevelopmentMode(IclWebCommService currentChannel, bool useDevMode)
        {
            currentChannel.UseDevelopmentMode(useDevMode);
        }

        // 1/24/2012
        /// <summary>
        /// Occurs when the user selects a specific <see cref="CloudService"/> for use in the comboBox.
        /// </summary>
        private void cmbCloudService_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var comboSelector = (ComboBox)sender;
            if (comboSelector == null)
            {
                return;
            }

            // Note: No longer used, since CloudService hard-coded!
            return;

            // 1/27/2012 - Save settings.
            /*DoSaveUserSettings();
            
            IList items = e.AddedItems;
            foreach (ComboBoxItem item in items)
            {
                if (item.Content.ToString().Equals("AmazonS3"))
                {
                    // Note: 1st param can be null, since it will be set by postsharp.
                    DoWCFUseCloudService(null, CloudService.AmazonS3);
                    CloudServiceInUse = CloudService.AmazonS3;
                }

                if (item.Content.ToString().Equals("Windows Azure"))
                {
                    // Note: 1st param can be null, since it will be set by postsharp.
                    DoWCFUseCloudService(null, CloudService.Azure);
                    CloudServiceInUse = CloudService.Azure;
                }
            }*/
        }

        // 1/27/2012
        /// <summary>
        /// Sets the <see cref="CloudService"/> value from in the Properties.Settings to the Wcf service.
        /// </summary>
        internal void SetCloudServiceToUse()
        {
            var useDevMode = (bool)GetPropertySetting(PropertySettingEnums.UseAzureDevMode);
            var cloudToUse = (CloudService)GetPropertySetting(PropertySettingEnums.CloudService);

            // Note: 1st param can be null, since it will be set by postsharp.
            DoWCFSetDevelopmentMode(null, useDevMode);
            // Note: 1st param can be null, since it will be set by postsharp.
            DoWCFUseCloudService(null, cloudToUse);
            CloudServiceInUse = cloudToUse;
        }

        // 1/24/2012
        /*/// <summary>
        /// Sets the given <see cref="CloudService"/> selection in the comboBox.
        /// </summary>
        /// <param name="useCloudService"><see cref="CloudService"/> to set.</param>
        internal void SetCloudServiceSelection(CloudService useCloudService)
        {
            switch (useCloudService)
            {
                case CloudService.AmazonS3:
                    cmbCloudService.SelectedIndex = 0;
                    break;
                case CloudService.Azure:
                    cmbCloudService.SelectedIndex = 1;
                    break;
                case CloudService.InternalTestDb:
                    cmbCloudService.SelectedIndex = -1;
                    break;
                default:
                    throw new ArgumentOutOfRangeException("useCloudService");
            }
        }*/

        // 1/27/2012
        /// <summary>
        /// Gets the current value for the given property settings enum.
        /// </summary>
        /// <param name="propertySetting"><see cref="PropertySettingEnums"/> to retrieve value for.</param>
        internal object GetPropertySetting(PropertySettingEnums propertySetting)
        {
            switch (propertySetting)
            {
                case PropertySettingEnums.UseFileChunks:
                    return Properties.Settings.Default.UseFileChunks;
                    break;
                case PropertySettingEnums.UseCompression:
                    return Properties.Settings.Default.UseCompression;
                    break;
                case PropertySettingEnums.GZipCompressionType:
                    return Properties.Settings.Default.GZipCompressionType;
                    break;
                case PropertySettingEnums.DotNetCompressionType:
                    return Properties.Settings.Default.DotNetCompressionType;
                    break;
                case PropertySettingEnums.UseEncryption:
                    return Properties.Settings.Default.UseEncryption;
                    break;
                case PropertySettingEnums.UseWindowState:
                    return Properties.Settings.Default.UseWindowState;
                    break;
                case PropertySettingEnums.WindowsHeight:
                    return Properties.Settings.Default.WindowsHeight;
                    break;
                case PropertySettingEnums.WindowsWidth:
                    return Properties.Settings.Default.WindowsWidth;
                    break;
                case PropertySettingEnums.UseAzureDevMode:
                    return Properties.Settings.Default.UseAzureDevMode;
                    break;
                case PropertySettingEnums.UseCloudCache:
                    return Properties.Settings.Default.RestoreWindowUseCloudCache;
                    break;
                case PropertySettingEnums.TreeView1ContainerWidth:
                    return Properties.Settings.Default.TreeView1ContainerWidth;
                    break;
                case PropertySettingEnums.TreeView2ContainerWidth:
                    return Properties.Settings.Default.TreeView2ContainerWidth;
                    break;
                case PropertySettingEnums.CloudService:
                    var cloudService = Properties.Settings.Default.CloudService;
                    switch (cloudService)
                    {
                        case 0:
                            return CloudService.AmazonS3;
                        case 1:
                            return CloudService.Azure;
                        default:
                            throw new ArgumentOutOfRangeException("cloudService");
                    }
                default:
                    throw new ArgumentOutOfRangeException("propertySetting");
            }
        }
    }
}