﻿using System;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.Common.AttsInjectorHelper;
using ImageNexus.BenScharbach.ImageCloudLock.OSHelper.Enums;

namespace ImageNexus.BenScharbach.ImageCloudLock.OSHelper
{
    /// <summary>
    /// Operating System Information helper class.
    /// </summary>
    public static class OsInfoHelper
    {

        /// <summary>
        /// Gets the current Operating System.
        /// </summary>
        /// <returns>Instance of <see cref="OperatingSystemTypeEnum"/></returns>
        [DoNotUseLogger]
        [CatchAllExceptionHandling(false, null, null)]
        public static OperatingSystemTypeEnum GetOsInfo()
        {
            //Get Operating system information.
            var os = Environment.OSVersion;
            //Get version information about the os.
            var vs = os.Version;

            //Variable to hold our return value
            var operatingSystem = OperatingSystemTypeEnum.Windows95;

            switch (os.Platform)
            {
                case PlatformID.Win32Windows:
                    switch (vs.Minor)
                    {
                        case 0:
                            operatingSystem = OperatingSystemTypeEnum.Windows95;
                            break;
                        case 10:
                            operatingSystem = vs.Revision.ToString() == "2222A" ? OperatingSystemTypeEnum.Windows98Se :
                                                                                                                          OperatingSystemTypeEnum.Windows98;
                            break;
                        case 90:
                            operatingSystem = OperatingSystemTypeEnum.WindowsMe;
                            break;
                        default:
                            break;
                    }
                    break;
                case PlatformID.Win32NT:
                    switch (vs.Major)
                    {
                        case 3:
                            operatingSystem = OperatingSystemTypeEnum.WindowsNt351;
                            break;
                        case 4:
                            operatingSystem = OperatingSystemTypeEnum.WindowsNt4;
                            break;
                        case 5:
                            operatingSystem = vs.Minor == 0 ? OperatingSystemTypeEnum.Windows2000 : OperatingSystemTypeEnum.WindowsXp;
                            break;
                        case 6:
                            operatingSystem = vs.Minor == 0 ? OperatingSystemTypeEnum.WindowsVista : OperatingSystemTypeEnum.Windows7;
                            break;
                        default:
                            break;
                    }
                    break;
            }
            //Make sure we actually got something in our OS check
            //We don't want to just return " Service Pack 2" or " 32-bit"
            //That information is useless without the OS version.
            /*if (operatingSystem != "")
            {
                //Got something.  Let's prepend "Windows" and get more info.
                operatingSystem = "Windows " + operatingSystem;
                //See if there's a service pack installed.
                if (os.ServicePack != "")
                {
                    //Append it to the OS name.  i.e. "Windows XP Service Pack 3"
                    operatingSystem += " " + os.ServicePack;
                }
                //Append the OS architecture.  i.e. "Windows XP Service Pack 3 32-bit"
                operatingSystem += " " + getOSArchitecture().ToString() + "-bit";
            }*/
            //Return the information we've gathered.
            return operatingSystem;
        }
    }
}
