﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.ComponentModel;
using System.ServiceModel;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Threading;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsMisc;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsThreading;
using ImageNexus.BenScharbach.Common.ProgressBarInterface;
using ImageNexus.BenScharbach.Common.ProgressBarWindow.Enums;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.ViewModel.DirectoryViewModel;
using ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.BaseClasses;
using ImageNexus.BenScharbach.ICLCommon.TreeViewInterface;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.CheckedDataModel;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.EventArgs;
using ImageNexus.BenScharbach.ImageCloudLock.CloudCache;
using ImageNexus.BenScharbach.ImageCloudLock.Enums;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;
using ImageNexus.BenScharbach.ImageCloudLock.ProgressBar;
using ImageNexus.BenScharbach.ImageCloudLock.TaskActions;

namespace ImageNexus.BenScharbach.ImageCloudLock
{
    // 8/11/2011
    /// <summary>
    /// Interaction logic for RestoreCloudWindow.xaml
    /// </summary>
    public partial class RestoreCloudWindow : Window, ILockUnlockUI
    {
        // Instantiate a cloud directory TreeView model adapter.
        internal DirCloudTreeViewModelAdapter _dirCloudTreeViewModelAdapter;

        private bool _isPropertiesUpdating; // 8/16/2011

        private readonly ConcurrentFooterListBox _concurrentFooterListBox; // 8/26/2011
        private readonly DoRetrieveFilesTask _retrieveTask;
        private readonly DoRetrieveDirectoriesTask _getDirectoryTask;
        private readonly DoRetrieveRootDirectoriesTask _getRootDirectoriesTask;

        // 9/4/2011
        private readonly ProgressBarService _progressBarService;

        // 3/2/2012 - FooterProgressBar for Directories
        internal readonly FooterProgressBar _footerProgressBarDirectories;
        // 3/2/2012 - FooterProgressBar for Files
        internal readonly FooterProgressBar _footerProgressBarFiles;

        // 3/17/2012 - Turns on/off the use of the 'cloudcache'
        private bool _useCloudCache;

        #region Events
        
        /// <summary>
        /// Occurs when the user clicks on the Cancel button request during
        /// some asnychronous operation.
        /// </summary>
        public event EventHandler CancelCurrentOperation;

        #endregion

        #region Properties

        /// <summary>
        /// Owner of this window.
        /// </summary>
        public MainWindow MainWindowParent { get; private set; }

        // 12/6/2010
        /// <summary>
        /// Gets the current instance of <see cref="MiniProgressBar"/>.
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(IMiniProgressBar), null)]
        public IMiniProgressBar MiniProgressBar
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.MiniProgressBar : null;
            }
            set
            {
                // If null, then just wait in parallel thread.
                // Note: This will always occur when window created because PostSharp attribute tries
                // note: to set before the constructor has run.
                if (MainWindowParent == null)
                    Task.Factory.StartNew(() =>
                                              {
                                                  while (MainWindowParent == null)
                                                  {
                                                      // Just spin wait. 
                                                  }

                                                  // Not null anymore, so set value.
                                                  MainWindowParent.MiniProgressBar = value;

                                              });
                else
                    MainWindowParent.MiniProgressBar = value;
            }
        }

        // 12/6/2010
        /// <summary>
        /// Gets the current <see cref="DuplexChannelFactory{IclWebCommService}"/>.
        /// </summary>
        public DuplexChannelFactory<IclWebCommService> IclDuplexChannelFactory
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.IclDuplexChannelFactory : null;
            }
            set { if (MainWindowParent != null) MainWindowParent.IclDuplexChannelFactory = value; }
        }

        // 12/6/2010
        /// <summary>
        /// Gets the current communication channel.
        /// </summary>
        public IclWebCommService CurrentChannel { get; set; }

        // 12/6/2010
        // NOTE: The 'Set' is required to keep PostSharp from crashing - bug.
        /// <summary>
        /// Gets the <see cref="ICommunicationObject"/> interface for the <see cref="IclDuplexChannelFactory"/>
        /// instance.  
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(ICommunicationObject), null)]
        public ICommunicationObject CommunicationObject
        {
            get { return IclDuplexChannelFactory; }
            private set { throw new InvalidOperationException(); }
        }

        #endregion

        /// <summary>
        /// Default constructor
        /// </summary>
        public RestoreCloudWindow()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="mainWindow">Instance of <see cref="MainWindow"/>.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public RestoreCloudWindow(MainWindow mainWindow) : this()
        {
            // 2/19/2011
            // Check for Valid License.
            /*var license = new LicenseHelper();
            if (!license.Check(false))
                Application.Current.Shutdown();*/

            // Save MainWindow ref
            MainWindowParent = mainWindow;

            // 3/19/2012 - Set UseCloudCache from property settings.
            _useCloudCache = (bool)mainWindow.GetPropertySetting(PropertySettingEnums.UseCloudCache);

            // 1/27/2012 - Set CloudService to Use in Wcf.
            mainWindow.SetCloudServiceToUse();
                
            // 8/26/2011
            _concurrentFooterListBox = new ConcurrentFooterListBox(this);

            // 3/2/2012
            _footerProgressBarDirectories = new FooterProgressBar(this, footerProgressBarDirectories);
            _footerProgressBarFiles = new FooterProgressBar(this, footerProgressBarFiles);

            // 12/7/2010 - Update the ItemTemplates style for TreeViewUserControl
            treeViewUserControl2.SetItemTemplatesType(false);

            // 9/3/2011 - Get Action Tasks
            _getRootDirectoriesTask = new DoRetrieveRootDirectoriesTask { MainWindowParent = MainWindowParent };
            _retrieveTask = new DoRetrieveFilesTask { MainWindowParent = MainWindowParent };
            _getDirectoryTask = new DoRetrieveDirectoriesTask { MainWindowParent = MainWindowParent };

            // Show WCF ProgressBar Window
            if (!_useCloudCache)
            {
                _progressBarService = new ProgressBarService();
                _progressBarService.Start(ProgressBarEventHandlerTypeEnum.RestoreEventHandler,
                                          ProgressBarType.StandardDualBar);
            }

            // Populate root directories
            GetCloudRootDirectories();

            // Set EventHandler for Cancel operation
            CancelCurrentOperation += DirectoryInfoViewModel.CancelAsyncOperation;

            // 8/15/2011 - Subscribe to Cloud Events
            DirectoryInfoViewModel.StartPopluateDirectories += DirectoryCloudItemViewModel_StartPopluateDirectories;
            DirectoryInfoViewModel.EndPopulateDirectories += DirectoryCloudItemViewModel_EndPopulateDirectories;
            DirectoryInfoViewModel.StartPopulateFiles += DirectoryCloudItemViewModel_StartPopulateFiles;
            DirectoryInfoViewModel.EndPopulateFiles += DirectoryCloudItemViewModel_EndPopulateFiles;
            DirectoryInfoViewModel.StartUpdatePropertyValues += DirectoryCloudItemViewModel_StartUpdatePropertyValues;
            DirectoryInfoViewModel.EndUpdatePropertyValues += DirectoryCloudItemViewModel_EndUpdatePropertyValues;

            // 3/19/2012 - Subscribe to the Treeview splitter moved event
            treeViewUserControl2.SplitterMoved += treeViewUserControl2_SplitterMoved;
            // 3/20/2012 - Set prior TreeView container width
            treeViewUserControl2.TreeViewContainerWidth = new GridLength((double)MainWindowParent.GetPropertySetting(PropertySettingEnums.TreeView2ContainerWidth));
            // 3/20/2012 - Set margin of checkboxUseCloudCache; 200 is subtracted, which is the MinWidth of the TreeViewContainerWidth
            checkBoxUseCloudCache.Margin = new Thickness(treeViewUserControl2.TreeViewContainerWidth.Value - 200,
                                                         checkBoxUseCloudCache.Margin.Top,
                                                         checkBoxUseCloudCache.Margin.Right,
                                                         checkBoxUseCloudCache.Margin.Bottom);
            // 3/20/2012 - Set MaxWidth for the TreeView1Container width
            treeViewUserControl2.TreeViewContainerMaxWidth = 500;
        }

        // 3/19/2012
        /// <summary>
        /// Occurs when the user moves the splitter bar.
        /// </summary>
        private void treeViewUserControl2_SplitterMoved(object sender, SplitterEventArgs e)
        {
            var currentMarginLeft = checkBoxUseCloudCache.Margin.Left + e.TreeViewSplitterHorizontalChange;
            currentMarginLeft = (currentMarginLeft < 0) ? 0 : currentMarginLeft;
            checkBoxUseCloudCache.Margin = new Thickness(currentMarginLeft, checkBoxUseCloudCache.Margin.Top,
                                                         checkBoxUseCloudCache.Margin.Right,
                                                         checkBoxUseCloudCache.Margin.Bottom);

            // 3/20/2012 - save width
            Properties.Settings.Default.TreeView2ContainerWidth = treeViewUserControl2.TreeViewContainerWidth.Value;
        }

        /// <summary>
        /// Sets the footer message and Unlocks the UI.
        /// </summary>
        void DirectoryCloudItemViewModel_StartUpdatePropertyValues(object sender, EventArgs e)
        {
            const string footerMessage = "Updating items.  This may take a while...";
            SetFooterMessage(footerMessage);

            _isPropertiesUpdating = true;

            LockUI(); 
        }

        /// <summary>
        /// Sets the footer message and Locks the UI.
        /// </summary>
        void DirectoryCloudItemViewModel_EndUpdatePropertyValues(object sender, EventArgs e)
        {
            _isPropertiesUpdating = false;

            EndCloudTransaction();
        }

        /// <summary>
        /// Sets the footer message and Locks the UI.
        /// </summary>
        private void DirectoryCloudItemViewModel_StartPopluateDirectories(object sender, EventArgs e)
        {
            if (_isPropertiesUpdating) return;

            const string footerMessage = "Retrieving directories from cloud service.  This may take a while...";
            SetFooterMessage(footerMessage);

            LockUI(); 
        }

        /// <summary>
        /// Sets the footer message and Unlocks the UI.
        /// </summary>
        private void DirectoryCloudItemViewModel_EndPopulateDirectories(object sender, EventArgs e)
        {
            if (_isPropertiesUpdating) return;

            const string footerMessage = "Retrieve directories completed.";
            SetFooterMessage(footerMessage);

            EndCloudTransaction();
        }

        /// <summary>
        /// Sets the footer message and Locks the UI.
        /// </summary>
        private void DirectoryCloudItemViewModel_StartPopulateFiles(object sender, EventArgs e)
        {
            if (_isPropertiesUpdating) return;

            const string footerMessage = "Retrieving files from cloud service.  This may take a while...";
            SetFooterMessage(footerMessage);
           
        }

        /// <summary>
        /// Sets the footer message and Unlocks the UI.
        /// </summary>
        private void DirectoryCloudItemViewModel_EndPopulateFiles(object sender, EventArgs e)
        {
            if (_isPropertiesUpdating) return;

            const string footerMessage = "Retrieve files completed.";
            SetFooterMessage(footerMessage);
        }

        // 8/11/2011
        private void btnStartRestore_Click(object sender, RoutedEventArgs e)
        {
            // 2/18/2011; 4/29/2011 - Returns if false
            if (!MainWindowParent.CheckAccountInfoExist()) return;

            // 2/19/2011
            // Check for Valid License.
            /*var license = new LicenseHelper();
            if (!license.Check(false))
                Application.Current.Shutdown();*/

            // 12/6/2010 - Refactor out.
            BeginRestoreFilesFromCloud();
            return;
        }

        #region Restore-Items method pair
       
        // 3/15/2012 - Updated to use new Action window.
        [CatchAllExceptionHandling(false, null, null)]
        private void BeginRestoreFilesFromCloud()
        {
            // Retrieve Files checked in treeView.
            List<IFileCheckedItem<IFileItem>> fileCheckedItems;
            _dirCloudTreeViewModelAdapter.GetListOfCheckedItems(out fileCheckedItems);

            // 1/11/2012
            if (fileCheckedItems == null)
            {
                MessageBox.Show("You MUST select an item first to restore.");
                return;
            }

            // 1/11/2012
            if (fileCheckedItems.Count == 0)
            {
                MessageBox.Show("You MUST select an item first to restore.");
                return;
            }

            string footerMessage = string.Format("Restore {0} files from cloud service.  This may take a while...", fileCheckedItems.Count);
            SetFooterMessage(footerMessage);

            // Create Empty restore action window
            var restoreWindow = new RestoreActionWindow(MainWindowParent, fileCheckedItems);
            restoreWindow.Show();

            // Close this window
            Close();
        }

        #endregion

        #region Retrieve Files Pair

        // 11/8/2010 - BeginGetFiles callback
        [ShowWaitCursor]
        public List<IFileItem> GetCloudFilesForDirectory(IDirectoryItemViewModel directoryItemViewModel, string directoryName)
        {
            if (directoryItemViewModel == null)
            {
                throw new ArgumentNullException("directoryItemViewModel");
            }

            if (directoryName == null)
            {
                throw new ArgumentNullException("directoryName");
            }

            // 3/16/2012 - First, check the 'cloudcache' for files.
            List<IFileItem> files;
            if (_useCloudCache && CloudCacheRetriever.GetCloudFilesForDirectory(directoryName, this, out files))
            {
                return files;
            }

            // Create task to get files
            var task = _retrieveTask.DoGetCloudFilesforDirectory(directoryName, this);

            // Wait until task completes
            if (!task.Wait(10))
            {
                Thread.Sleep(10);
            }

            // return result
            return task.Result;
        }

        #endregion

        #region Retrieve Directories Pair

        // 11/8/2010 - BeginGetDirectories callback
        public List<IDirectoryItem> GetCloudDirectoriesForDirectory(IDirectoryItemViewModel directoryItemViewModel, string directoryName)
        {
            if (directoryItemViewModel == null)
            {
                throw new ArgumentNullException("directoryItemViewModel");
            }

            if (directoryName == null)
            {
                throw new ArgumentNullException("directoryName");
            }

            // 3/16/2012 - First, check the 'cloudcache' for directories.
            List<IDirectoryItem> subDirectories;
            if (_useCloudCache && CloudCacheRetriever.GetCloudDirectoriesForDirectory(directoryName, this, directoryItemViewModel, out subDirectories))
            {
                return subDirectories;
            }

            // Create task to get directories
            var task = _getDirectoryTask.DoGetCloudDirectoriesForDirectory(directoryItemViewModel, directoryName, this);

            // Wait until task completes
            if (!task.Wait(10))
            {
                Thread.Sleep(10);
            }

            // return result
            return task.Result;
            //return new List<IDirectoryItem>();
        }

        #endregion

        #region Retrieve RootDirectories Pair

        [CatchAllExceptionHandling(false, null, null)]
        private void GetCloudRootDirectories()
        {
            // 3/16/2012 - First, check the 'cloudcache' for root directories.
            string[] rootDirectories;
            if (_useCloudCache && CloudCacheRetriever.GetCloudRootDirectories(out rootDirectories))
            {
                DirectoryTreeViewTasks.CreateDirectoryCloudTreeViewTask(new DirectoryTreeViewTasks.
                                                                        DirectoryCloudTreeViewTaskParams
                {
                    RestoreCloudWindow = this,
                    ListViewInstance = treeViewUserControl2.DirectoryListView,
                    TreeViewInstance = treeViewUserControl2.DetailsTreeView,
                    TreeViewUserControl = treeViewUserControl2,
                    Directories = rootDirectories
                });

                return;
            }

            // 1/7/2012 - Subscribe to completion event
            _getRootDirectoriesTask.TaskCompleted += GetRootDirectoriesTaskTaskCompleted;

            // Create task to get files
            _getRootDirectoriesTask.DoGetCloudRootDirectories(this);

        }

        /// <summary>
        /// Occurs when the task is completed.
        /// </summary>
        /// <param name="task">Instance of Task.</param>
        private void GetRootDirectoriesTaskTaskCompleted(Task<CloudRetrieveDirectories> task)
        {
            if (task.Result == null) return;
            
            if (task.Result.Directories.Length == 0) return;
            
            DirectoryTreeViewTasks.CreateDirectoryCloudTreeViewTask(new DirectoryTreeViewTasks.
                                                                        DirectoryCloudTreeViewTaskParams
            {
                RestoreCloudWindow = this,
                ListViewInstance = treeViewUserControl2.DirectoryListView,
                TreeViewInstance = treeViewUserControl2.DetailsTreeView,
                TreeViewUserControl = treeViewUserControl2,
                Directories = task.Result.Directories
            });
        }

        #endregion
      

        /// <summary>
        /// Resets the footer progress bar and unlocks the UI controls.
        /// </summary>
        private void EndCloudTransaction()
        {
            // 3/2/2012
            _footerProgressBarDirectories.EndFooterProgressBar();
            _footerProgressBarFiles.EndFooterProgressBar();

            UnLockUI(); // 8/15/2011
        }

        #region Footer ConcurrentFooter ListBox methods

        /// <summary>
        /// Method helper to set the Footer message.
        /// </summary>
        internal void SetFooterMessage(string footerMessage)
        {
            // 8/26/2011
            if (_concurrentFooterListBox == null) return;

            _concurrentFooterListBox.AddMessage(footerMessage);
        }
        
        /// <summary>
        /// Helper method which sets the footer's Header control.
        /// </summary>
        public void SetFooterHeaderTextCallback(object sender, ProgressEventArgs e)
        {
            if (!(sender is DirectoryInfoViewModel)) return;

            SetFooterHeaderText(e.Message);
        }

        /// <summary>
        /// Helper method which sets the footer's Header control.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        [OnGuiThread(DispatcherPriority.Send, Asynchronous = true)]
        private void SetFooterHeaderText(string message)
        {
            // 8/26/2011
            if (_concurrentFooterListBox == null) return;

            _concurrentFooterListBox.AddMessage(message);
        }

       

        #endregion

        /// <summary>
        /// OnGui WPF UI thread call.
        /// </summary>
        /// <typeparam name="T">Generic Type</typeparam>
        /// <param name="action">Action delegate</param>
        /// <param name="paramItem">Parameter to pass into Action delegate</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void OnGuiThreadMethodCall<T>(Action<T> action, T paramItem) where T : class
        {
            // The Work to perform on another thread
            ThreadStart start = delegate()
            {
                // ...

                // This will work as its using the dispatcher
                var op = Dispatcher.BeginInvoke(
                    DispatcherPriority.Send,
                    action,
                    paramItem);

                var status = op.Status;
                while (status != DispatcherOperationStatus.Completed)
                {
                    status = op.Wait(TimeSpan.FromMilliseconds(1000));
                    if (status == DispatcherOperationStatus.Aborted)
                    {
                        // Alert Someone
                    }
                }
            };

            // Create the thread and kick it started!
            new Thread(start).Start();

        }

        /// <summary>
        /// Occurs when the user clicks the Cancel button in the footer window section.
        /// </summary>
        // ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void btnCancel_Click(object sender, RoutedEventArgs e)
        // ReSharper restore InconsistentNaming
        {
            // trigger event
            if (CancelCurrentOperation != null)
                CancelCurrentOperation(this, EventArgs.Empty);
        }

        // 8/14/2011
        /// <summary>
        /// Locks the MainWindow UI during concurrent operation.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = true)]
        public void LockUI()
        {
            // Lock TreeView2
            treeViewUserControl2.LockUnLockDrivesUi(false);
            treeViewUserControl2.LockUnLockTreeViewUi(false);
        }

        // 8/14/2011
        /// <summary>
        /// UnLocks the MainWindow UI during concurrent operation.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = true)]
        public void UnLockUI()
        {
            // Lock TreeView2
            treeViewUserControl2.LockUnLockDrivesUi(true);
            treeViewUserControl2.LockUnLockTreeViewUi(true);
        }

        // 8/20/2011
        /// <summary>
        /// Occurs when the user closes this window.
        /// </summary>
        private void RestoreCloudTool_Closing(object sender, CancelEventArgs e)
        {
            if (treeViewUserControl2 != null) treeViewUserControl2.SaveUserSettings();

            // Turn Off the progressBar from being top window.
            if (_progressBarService != null) 
                _progressBarService.SetTopMostWindow(false);

            // 8/26/2011
            if (_concurrentFooterListBox != null)
                _concurrentFooterListBox.Dispose();
        }
        
        /// <summary>
        /// Occurs when a user clicks the 'Delete' button.
        /// </summary>
        // ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void btnDelete_Click(object sender, RoutedEventArgs e)
        // ReSharper restore InconsistentNaming
        {
            // Retrieve Files checked in treeView.
            List<IFileCheckedItem<IFileItem>> fileCheckedItems;
            _dirCloudTreeViewModelAdapter.GetListOfCheckedItems(out fileCheckedItems);

            // 1/11/2012
            if (fileCheckedItems == null)
            {
                MessageBox.Show("You MUST select an item first to delete.");
                return;
            }

            // 1/11/2012
            if (fileCheckedItems.Count == 0)
            {
                MessageBox.Show("You MUST select an item first to delete.");
                return;
            }

            string footerMessage = string.Format("Deleting {0} files from cloud service.  This may take a while...", fileCheckedItems.Count);
            SetFooterMessage(footerMessage);

            // Create Empty delete window
            var deleteWindow = new DeleteEmptyWindow(MainWindowParent, fileCheckedItems);
            deleteWindow.Show();

            // Close this window
            Close();
        }

        // 3/2/2012
        public class FooterProgressBar
        {
            private readonly RestoreCloudWindow _restoreCloudWindow;
            private readonly System.Windows.Controls.ProgressBar _footerProgressBar;

            private class DispatcherParams
            {
                public FooterProgressBar FooterProgressBar;
                public double Value;
                public double Count;
            }

            /// <summary>
            /// Constructor
            /// </summary>
            /// <param name="restoreCloudWindow"> </param>
            /// <param name="footerProgressBar">Instance of <see cref="ProgressBar"/> to use.</param>
            public FooterProgressBar(RestoreCloudWindow restoreCloudWindow, System.Windows.Controls.ProgressBar footerProgressBar)
            {
                _restoreCloudWindow = restoreCloudWindow;
                _footerProgressBar = footerProgressBar;
            }

            #region Footer Mini-ProgressBar methods

            /// <summary>
            /// Helper method which resets the Footer's mini-progressBar.
            /// </summary>
            public void ResetFooterProgressBarCallback(object sender, ProgressEventArgs e)
            {
                if (!(sender is DirectoryInfoViewModel)) return;

                ResetFooterProgressBar(e.Value);
            }

            [CatchAllExceptionHandling(false, null, null)]
            //[OnGuiThread(DispatcherPriority.Send, Asynchronous = true)]
            public void ResetFooterProgressBar(double count)
            {
                _restoreCloudWindow.OnGuiThreadMethodCall(SetFooterProgressBarInitValuesCallback, new DispatcherParams()
                {
                    FooterProgressBar = this,
                    Value = 0,
                    Count = count,
                });
            }

            /// <summary>
            /// Sets the footerProgressBar's value
            /// </summary>
            /// <param name="obj"></param>
            private void SetFooterProgressBarInitValuesCallback(DispatcherParams obj)
            {
                obj.FooterProgressBar._footerProgressBar.Value = obj.Value;
                obj.FooterProgressBar._footerProgressBar.Maximum = obj.Count;
                obj.FooterProgressBar._footerProgressBar.Visibility = Visibility.Visible;
            }

            /// <summary>
            /// Helper method which set the current value into the footer's mini-progressBar.
            /// </summary>
            public void SetFooterProgressCallback(object sender, ProgressEventArgs e)
            {
                if (!(sender is DirectoryInfoViewModel)) return;

                SetFooterProgress(e.Value);
            }

            /// <summary>
            /// Helper method which set the current value into the footer's mini-progressBar.
            /// </summary>
            [CatchAllExceptionHandling(false, null, null)]
            public void SetFooterProgress(double value)
            {
                _restoreCloudWindow.OnGuiThreadMethodCall(SetFooterProgressBarCallback, new DispatcherParams()
                {
                    FooterProgressBar = this,
                    Value = value
                });
            }

            /// <summary>
            /// Sets the footerProgressBar's value
            /// </summary>
            /// <param name="obj"></param>
            private void SetFooterProgressBarCallback(DispatcherParams obj)
            {
                obj.FooterProgressBar._footerProgressBar.Value = obj.Value;
            }

            /// <summary>
            /// Helper method which clears the footer text and hides the progress bar.
            /// </summary>
            public void EndFooterProgressBarCallback(object sender, EventArgs e)
            {
                if (!(sender is DirectoryInfoViewModel)) return;

                EndFooterProgressBar();
            }

            /// <summary>
            /// Helper method which clears the footer text and hides the progress bar.
            /// </summary>
            [CatchAllExceptionHandling(false, null, null)]
            public void EndFooterProgressBar()
            {
                _restoreCloudWindow.OnGuiThreadMethodCall(SetFooterProgressBarVisiblityCallback, new DispatcherParams()
                {
                    FooterProgressBar = this,
                    Value = 0
                });
            }

            /// <summary>
            /// Sets the footerProgressBar's visibility to hidden.
            /// </summary>
            /// <param name="obj"></param>
            private void SetFooterProgressBarVisiblityCallback(DispatcherParams obj)
            {
                //obj.FooterProgressBar._footerProgressBar.Visibility = Visibility.Hidden;
            }

            #endregion
        }

        /// <summary>
        /// Occurs when the user checks the 'Use CloudCache'.
        /// </summary>
        private void checkBoxUseCloudCache_Checked(object sender, RoutedEventArgs e)
        {
            _useCloudCache = true;
        }

        /// <summary>
        /// Occurs when the user unchecks the 'Use CloudCache'.
        /// </summary>
        private void checkBoxUseCloudCache_Unchecked(object sender, RoutedEventArgs e)
        {
            _useCloudCache = false;
        } 

        // 8/26/2011
        private class ConcurrentFooterListBox : IDisposable
        {
            private Thread _listBoxThread; // 8/26/2011
            private ConcurrentBag<string> _listBoxMessages;
            private bool _cancelListboxThread; // 8/26/2011
            private readonly RestoreCloudWindow _restoreCloudWindow;

            private class DispatcherParams
            {
                public RestoreCloudWindow RestoreCloudWindow;
                public string Message;
            }

            public ConcurrentFooterListBox(RestoreCloudWindow restoreCloudWindow)
            {
                _restoreCloudWindow = restoreCloudWindow;

                // Start listView Thread
                StartListViewThread();
            }

            /// <summary>
            /// Adds message to display in listbox.
            /// </summary>
            /// <param name="message"></param>
            public void AddMessage(string message)
            {
                _listBoxMessages.Add(message);
            }

            // 8/26/2011
            private void StartListViewThread()
            {
                _listBoxThread = new Thread(PopulateListViewCallback) { IsBackground = true };
                _listBoxMessages = new ConcurrentBag<string>();
                _listBoxThread.Start(this);
            }

            // 8/26/2011
            private void PopulateListViewCallback(object state)
            {
                var concurrentFooterListBox = (ConcurrentFooterListBox)state;


                while (!concurrentFooterListBox._cancelListboxThread)
                {
                    // check for messages
                    if (!concurrentFooterListBox._listBoxMessages.IsEmpty)
                    {
                        string message;
                        if (concurrentFooterListBox._listBoxMessages.TryTake(out message))
                        {
                            concurrentFooterListBox.AddListboxMessage(message);
                        }
                    }

                    Thread.Sleep(10);
                }
            }

           /// <summary>
           /// Adds a message to the ListBox using a Dispatcher callback.
           /// </summary>
           /// <param name="message"></param>
            private void AddListboxMessage(string message)
            {
                _restoreCloudWindow.OnGuiThreadMethodCall(AddListboxItemCallback, new DispatcherParams()
                                                                              {
                                                                                  RestoreCloudWindow =
                                                                                      _restoreCloudWindow,
                                                                                  Message = message
                                                                              });
            }

            /// <summary>
            /// Adds listBox message callback
            /// </summary>
            /// <param name="obj"></param>
            private void AddListboxItemCallback(DispatcherParams obj)
            {
                obj.RestoreCloudWindow.footerTextListBox.Items.Add(new TextBlock() { Text = obj.Message, FontSize = 8, Foreground = Brushes.Navy});
            }

            public void Dispose()
            {
                // cancel listbox thread
                _cancelListboxThread = true;
                _listBoxThread.Join(500);
                _listBoxThread = null;
            }
        }// End Concurrent class

    }
}
