﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Windows;
using System.Windows.Controls;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using Attributes;
using ImageNexus.BenScharbach.Common.AttsInjectorHelper;
using ImageNexus.BenScharbach.Common.MEFLoggerInterfaces.Interfaces;
using ImageNexus.BenScharbach.ICLTools.TaskManager;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;
using Microsoft.Win32.TaskScheduler;
using Task = System.Threading.Tasks.Task;

namespace ImageNexus.BenScharbach.ImageCloudLock
{
    /// <summary>
    /// Interaction logic for SchedulerWindow.xaml
    /// </summary>
     /*[MefLogger(LoggerTypeEnum.TraceLog, "ICLFrontEndConsole", LogPropertyCalls = true,
        LoggerVerbosity = LoggerVerbosityEnum.LogEntry | LoggerVerbosityEnum.LogExit | LoggerVerbosityEnum.LogExceptions,
        TraceLoggerOutputPath = @"ICL_FE_Logs\ScheduleWinCallsOutput.txt", WpfLogWindowInstanceId = 1)]*/
    public partial class SchedulerWindow : Window
    {
        private bool _isWeeklySchedule;
        private bool _isMonthlySchedule;

        #region Properties

        // 6/19/2011
        /// <summary>
        /// The MEF <see cref="MEFLoggerInterfaces.Interfaces.ILogger"/> required for using the <see cref="MefLoggerAttribute"/>.
        /// </summary>
        public ILogger Logger { get; set; }

        /// <summary>
        /// Owner of this window.
        /// </summary>
        public MainWindow MainWindowParent { get; private set; }

        /// <summary>
        /// Gets or sets if weekly schedule.
        /// </summary>
        public bool IsWeeklySchedule
        {
            get 
            { 
                if (rbWeekDay != null) 
                    return (rbWeekDay.IsChecked == null) ? false : rbWeekDay.IsChecked.Value;

                return false;
            }
            set
            {
                if (rbWeekDay != null) rbWeekDay.IsChecked = value;
                _isMonthlySchedule = value;
            }
        }

        /// <summary>
        /// Gets or sets if monthly schedule.
        /// </summary>
        public bool IsMonthlySchedule
        {
            get
            {
                return (rbMonth.IsChecked == null) ? false : rbMonth.IsChecked.Value; 
            }
            set
            {
                if (rbMonth != null) rbMonth.IsChecked = value;
                _isWeeklySchedule = value;
            }
        }

        // 12/13/2010
        /// <summary>
        /// Gets the current <see cref="DuplexChannelFactory{IclWebCommService}"/>.
        /// </summary>
        public DuplexChannelFactory<IclWebCommService> IclDuplexChannelFactory
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.IclDuplexChannelFactory : null;
            }
            set { if (MainWindowParent != null) MainWindowParent.IclDuplexChannelFactory = value; }
        }

        // 12/13/2010
        /// <summary>
        /// Gets the current communication channel.
        /// </summary>
        public IclWebCommService CurrentChannel { get; set; }

        // 12/13/2010
        // NOTE: The 'Set' is required to keep PostSharp from crashing - bug.
        /// <summary>
        /// Gets the <see cref="ICommunicationObject"/> interface for the <see cref="IclDuplexChannelFactory"/>
        /// instance.  
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public ICommunicationObject CommunicationObject
        {
            get { return IclDuplexChannelFactory; }
            private set { throw new InvalidOperationException(); }
        }

        #region Days Propreties

        /*/// <summary>
        /// Sunday
        /// </summary>
        public bool CbSunday
        {
            get { return (bool) GetValue(CbSundayProperty);}
            set { SetValue(CbSundayProperty, value); }
        }

        public static readonly DependencyProperty CbSundayProperty =
            DependencyProperty.Register("CbSunday", typeof (bool), typeof (SchedulerWindow), new PropertyMetadata(false));

       
        */
        #endregion

        #endregion


        /// <summary>
        /// Default constructor
        /// </summary>
        public SchedulerWindow()
        {
            InitializeComponent();
        }

        // 12/13/2010
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="mainWindow">Instance of <see cref="MainWindow"/></param>
        //[CatchAllExceptionHandling(false, null, null)]
        public SchedulerWindow(MainWindow mainWindow)
        {
            InitializeComponent();

            // 2/14/2011
            // Check for Valid License.
            /*var license = new LicenseHelper();
            if (!license.Check(false))
                Application.Current.Shutdown();*/

            MainWindowParent = mainWindow;
        }

        // 12/10/2010
        /// <summary>
        /// Occurs when user clicks the Close button.  
        /// </summary>
// ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void btnClose_Click(object sender, RoutedEventArgs e)
// ReSharper restore InconsistentNaming
        {
            Close();
        }

        // 12/10/2010
        /// <summary>
        /// Occurs when user clicks the Apply button, which applies
        /// the current schedule by creating the 'Task' in windows.
        /// </summary>
// ReSharper disable InconsistentNaming
        private void btnApplySchedule_Click(object sender, RoutedEventArgs e)
// ReSharper restore InconsistentNaming
        {
            // Apply schedule
            CreateSchedule();
        }

        // 12/12/2010
        [CatchAllExceptionHandling(false, null, null)]
        private void CreateSchedule()
        {
            var scheduledSuccess = false;

            // Retrieve Type of backup

            // Create Weekly or Update...
            if (IsWeeklySchedule)
            {
                scheduledSuccess = CreateWeeklySchedule();
            }
            // Create Monthly or update...
            else if (IsMonthlySchedule)
            {
                scheduledSuccess = CreateMonthlySchedule();
            }

            // Delete current schedule...
            if (rbNoBackup.IsChecked != null && rbNoBackup.IsChecked.Value)
            {
                DoTaskDelete(); // Now Delete current ScheduleId.
                return;
            }

            // Let user know result of apply.
            IsSuccessfulMessageBox(scheduledSuccess, "ScheduleId-{0} was saved succesfully.", "ScheduleId-{0} save failed.", "Apply Schedule");
        }

        // 12/12/2010
        /// <summary>
        /// Helper method, which displays a messageBox to the end user with result.
        /// </summary>
        /// <param name="scheduledSuccess">Success boolean value</param>
        /// <param name="messageSuccess">Message to display on success.</param>
        /// <param name="messageFailed">Message to display on failure.</param>
        /// <param name="messageCaption">Caption for MessageBox.</param>
        [CatchAllExceptionHandling(false, null, null)]
        private void IsSuccessfulMessageBox(bool scheduledSuccess, string messageSuccess, string messageFailed, string messageCaption)
        {
            if (scheduledSuccess)
                MessageBox.Show(string.Format(messageSuccess, GetCurrentScheduleId()), messageCaption, MessageBoxButton.OK,
                                MessageBoxImage.Information, MessageBoxResult.None);
            else
                MessageBox.Show(string.Format(messageFailed, GetCurrentScheduleId()), messageCaption, MessageBoxButton.OK,
                                MessageBoxImage.Error, MessageBoxResult.None);
        }

        // 12/13/2010
        /// <summary>
        /// Converts a standard month integer to the enum <see cref="MonthsOfTheYear"/>.
        /// </summary>
        /// <param name="month">Month integer</param>
        /// <returns>Enum of <see cref="MonthsOfTheYear"/>.</returns>
        [CatchAllExceptionHandling(true, typeof(MonthsOfTheYear), MonthsOfTheYear.January)]
        private static MonthsOfTheYear ConvertToMonthsOfTheYearEnum(int month)
        {
            // verify
            if (month < 1 || month > 12)
                throw new ArgumentOutOfRangeException("month", @"Value given MUST be in the range of 1-12.");

            switch (month)
            {
                case 1:
                    return MonthsOfTheYear.January;
                case 2:
                    return MonthsOfTheYear.February;
                case 3:
                    return MonthsOfTheYear.March;
                case 4:
                    return MonthsOfTheYear.April;
                case 5:
                    return MonthsOfTheYear.May;
                case 6:
                    return MonthsOfTheYear.June;
                case 7:
                    return MonthsOfTheYear.July;
                case 8:
                    return MonthsOfTheYear.August;
                case 9:
                    return MonthsOfTheYear.September;
                case 10:
                    return MonthsOfTheYear.October;
                case 11:
                    return MonthsOfTheYear.November;
                case 12:
                    return MonthsOfTheYear.December;
                default:
                    throw new ArgumentOutOfRangeException();
            }
            
        }
        

        // 12/13/2010
        /// <summary>
        /// Creates a monthly scheduled task for all
        /// choosen days.  If a task already exist, the task
        /// will automatically be deleted first.
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(bool), false)]
        private bool CreateMonthlySchedule()
        {
            // Get Month Day for schedule.
            var calendarSelectedDates = calendar.SelectedDates;
           

            // Get ScheduleId
            var scheduleId = GetCurrentScheduleId();

            // Check if task exist; if so, deleted it since creating updated version.
            if (ICLTaskScheduler.DoesTaskExist(scheduleId))
            {
                // Delete task
                if (!ICLTaskScheduler.DeleteTask(scheduleId))
                    throw new InvalidOperationException("The 'DeleteTask' operation failed.");
            }

            // Get Time
            var hour = (int)nudHour.Value;
            var minutes = (int)nudMinutes.Value;
            var amPm = GetCurrentAmPmType();

            // Get DateTime.Now, to use the Year, etc.
            var dateTimeNow = DateTime.Now;

            // Months
            var monthsPicked = calendarSelectedDates.Select(item => item.Month).Distinct();

            // iterate collection
            var monthlyTasks = new List<TaskMonthly>(monthsPicked.Count());
            foreach (var month in monthsPicked)
            {
                // Retrieve days for given month
                var daysInMonth = calendarSelectedDates.Where(item => item.Month == month).Select(day => day.Day).ToArray();

                // Create Monthly task.
                var taskMonthly = new TaskMonthly()
                                      {
                                          TaskId = string.Format("T-{0}", "M" + month),
                                          DaysOfMonth = daysInMonth,
                                          DateTime = new DateTime(dateTimeNow.Year, dateTimeNow.Month, dateTimeNow.Day,
                                                                  ConvertHourTo24Format(hour, amPm), minutes, 0,
                                                                  DateTimeKind.Local),
                                          Months = ConvertToMonthsOfTheYearEnum(month),
                                      };
                monthlyTasks.Add(taskMonthly);
            }


            // Now add new ScheduleId with tasks
            return ICLTaskScheduler.AddMonthlyTask(scheduleId, monthlyTasks.ToArray());

        }

       

        // 12/12/2010
        /// <summary>
        /// Creates a weekly scheduled task for all
        /// choosen days.  If a task already exist, the task
        /// will automatically be deleted first.
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(bool), false)]
        private bool CreateWeeklySchedule()
        {
            // Get ScheduleId
            var scheduleId = GetCurrentScheduleId();

            // Check if task exist; if so, deleted it since creating updated version.
            if (ICLTaskScheduler.DoesTaskExist(scheduleId))
            {
                // Delete task
                if (!ICLTaskScheduler.DeleteTask(scheduleId))
                    throw new InvalidOperationException("The 'DeleteTask' operation failed.");
            }

            // Get Time
            var hour = (int)nudHour.Value;
            var minutes = (int)nudMinutes.Value;
            var amPm = GetCurrentAmPmType();

            // Get Days
            var scheduledDays = new List<DaysOfTheWeek?>(7)
                                    {
                                        ((cbMonday.IsChecked == null)
                                             ? null
                                             : cbMonday.IsChecked.Value ? DaysOfTheWeek.Monday : (DaysOfTheWeek?) null),
                                        ((cbTuesday.IsChecked == null)
                                             ? null
                                             : cbTuesday.IsChecked.Value ? DaysOfTheWeek.Tuesday : (DaysOfTheWeek?) null),
                                        ((cbWednesday.IsChecked == null)
                                             ? null
                                             : cbWednesday.IsChecked.Value
                                                   ? DaysOfTheWeek.Wednesday
                                                   : (DaysOfTheWeek?) null),
                                        ((cbThursday.IsChecked == null)
                                             ? null
                                             : cbThursday.IsChecked.Value
                                                   ? DaysOfTheWeek.Thursday
                                                   : (DaysOfTheWeek?) null),
                                        ((cbFriday.IsChecked == null)
                                             ? null
                                             : cbFriday.IsChecked.Value ? DaysOfTheWeek.Friday : (DaysOfTheWeek?) null),
                                        ((cbSaturday.IsChecked == null)
                                             ? null
                                             : cbSaturday.IsChecked.Value
                                                   ? DaysOfTheWeek.Saturday
                                                   : (DaysOfTheWeek?) null),
                                        ((cbSunday.IsChecked == null)
                                             ? null
                                             : cbSunday.IsChecked.Value ? DaysOfTheWeek.Sunday : (DaysOfTheWeek?) null)
                                    };

            // Get DateTime.Now, to use the Year, etc.
            var dateTimeNow = DateTime.Now;
         
            var weeklyTasks = new List<TaskWeekly>(7);
            foreach (var scheduledDay in scheduledDays)
            {
                if (scheduledDay == null) continue;

                var taskWeekly = new TaskWeekly
                                     {
                                         TaskId = string.Format("T-{0}", scheduledDay.Value),
                                         TaskDay = scheduledDay.Value,
                                         DateTime = new DateTime(dateTimeNow.Year, dateTimeNow.Month, dateTimeNow.Day, 
                                             ConvertHourTo24Format(hour, amPm), minutes, 0, DateTimeKind.Local),
                                     };
                weeklyTasks.Add(taskWeekly);
            }

            // Now add new ScheduleId with tasks
            return ICLTaskScheduler.AddWeeklyTask(scheduleId, weeklyTasks.ToArray());
        }

        // 12/12/2010
        /// <summary>
        /// Helper method which converts a given 12-hours format to 
        /// a 24-hours format.
        /// </summary>
        /// <param name="hour">Current 12-hour value</param>
        /// <param name="amPmType">Either 'Am' or 'Pm' type.</param>
        /// <returns>24-hour format.</returns>
        [CatchAllExceptionHandling(true, typeof(int), 12)]
        private static int ConvertHourTo24Format(int hour, string amPmType)
        {
            if (amPmType.Equals("PM"))
            {
                return (hour + 12 == 24) ? 0 : hour + 12;
            }
            return hour;
        }

        // 12/12/2010
        /// <summary>
        /// Helper method which converts a given 24-hours format to 
        /// a 12-hours format.
        /// </summary>
        /// <param name="hour">Current 24-hour value</param>
        /// <param name="amPmType">(OUT) Either 'Am' or 'Pm' type.</param>
        /// <returns>12-hour format.</returns>
        [CatchAllExceptionHandling(true, typeof(int), 12)]
        private static int ConvertHourTo12Format(int hour, out string amPmType)
        {
            // Set default value
            amPmType = "AM";
            var newHour = hour;

            // When zero, its midnight.
            if (hour == 0)
                newHour = 12;

            // if greater than 12, MUST be PM.
            if (hour > 12)
            {
                newHour = hour - 12;
                amPmType = "PM";
            }

            return newHour;
        }

        // 12/12/2010
        /// <summary>
        /// Returns the current ScheduleId.
        /// </summary>
        /// <returns>ScheduleId as string.</returns>
        [CatchAllExceptionHandling(true, typeof(string), "All")]
        private string GetCurrentScheduleId()
        {
            // Extract scheduleId from combobox.
            return ((ComboBoxItem) cbScheduleId.SelectedItem).Tag.ToString();
        }

        // 12/12/2010
        /// <summary>
        /// Returns the current Am/Pm type.
        /// </summary>
        /// <returns>Am/Pm type as string.</returns>
        [CatchAllExceptionHandling(true, typeof(string), "AM")]
        private string GetCurrentAmPmType()
        {
            // Extract Am/Pm type from combobox.
            return ((ComboBoxItem)cbAmPm.SelectedItem).Tag.ToString();
        }

        // 12/12/2010
// ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void rbWeekDay_Click(object sender, RoutedEventArgs e)
// ReSharper restore InconsistentNaming
        {
            // get sender
            var rbWeekly = (RadioButton) sender;
            if (rbWeekly == null) return;

            // store IsChecked 
            IsWeeklySchedule = (rbWeekly.IsChecked == null) ? false : rbWeekly.IsChecked.Value;

        }

        // 12/12/2010
// ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void rbMonth_Click(object sender, RoutedEventArgs e)
// ReSharper restore InconsistentNaming
        {
            // get sender
            var rbMonthy = (RadioButton)sender;
            if (rbMonthy == null) return;

            // store IsChecked 
            IsMonthlySchedule = (rbMonthy.IsChecked == null) ? false : rbMonthy.IsChecked.Value;
            
        }

        // 12/12/2010
// ReSharper disable InconsistentNaming
       
        [ServiceExceptionHandling]
        [CatchAllExceptionHandling(false, null, null)]
        private void rbBackupNow_Click(object sender, RoutedEventArgs e)
// ReSharper restore InconsistentNaming
        {
            // Get ScheduleId
            var scheduleId = GetCurrentScheduleId();
            
            // Call WCF service to start task.
            // Create Parallel task
            var task = Task.Factory.StartNew(new Func<object, bool>(DoScheduledTaskCallback), scheduleId);

            // Wait until task completes
            task.Wait();
        }

        // 8/28/2011
        [CatchAllExceptionHandling(true, typeof(bool), false)]
        private bool DoScheduledTaskCallback(object scheduleId)
        {
            // Note: 1st param can be null, since it will be set by postsharp.
            DoWCFScheduledTask(null, (string) scheduleId);

            return true;
        }

        // 8/28/2011
        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private void DoWCFScheduledTask(IclWebCommService currentChannel, string scheduleId)
        {
            currentChannel.DoScheduledTask(scheduleId);
        }

        // 12/12/2010
// ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void rbNoBackup_Click(object sender, RoutedEventArgs e)
// ReSharper restore InconsistentNaming
        {
            IsWeeklySchedule = false;
            IsMonthlySchedule = false;

            // Now Delete current ScheduleId.
            DoTaskDelete();
        }

        // 12/12/2010
        /// <summary>
        /// Method helper to delete a task from windows task scheduler.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void DoTaskDelete()
        {
            var result = ICLTaskScheduler.DeleteTask(GetCurrentScheduleId());

            // Let user know result of apply.
            IsSuccessfulMessageBox(result, "ScheduleId-{0} was deleted succesfully.", "ScheduleId-{0} delete failed.", "Delete Schedule");
        }

        // 12/12/2010
        /// <summary>
        /// Occurs when the ScheduleId comboBox is updated by user.
        /// </summary>
// ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void cbScheduleId_SelectionChanged(object sender, SelectionChangedEventArgs e)
// ReSharper restore InconsistentNaming
        {
            // 2/20/2011 - Reset all checkboxes to false
            ResetAllDaysToFalse();

            // Retrieve prior settings.
            List<TaskBase> tasks;
            if (!ICLTaskScheduler.RetrieveTask(GetCurrentScheduleId(), out tasks)) return;

            // check if null or empty set.
            if (tasks == null || tasks.Count == 0) return;


            // Is Weekly?
            if (tasks[0] is TaskWeekly)
            {
                // Reset all days to false.
                ResetAllDaysToFalse();

                if (rbWeekDay != null) rbWeekDay.IsChecked = true;
                IsWeeklySchedule = true;
                // Set Weekly tab
                if (tabs != null) tabs.SelectedIndex = 0;

                RepopulateWeeklyTasks(tasks);
                
            }
            // Check task type.
            // Is Monthly?
            else if (tasks[0] is TaskMonthly)
            {
                // Clear out all days on calendar
                if (calendar != null) 
                    calendar.SelectedDates.Clear();

                if (rbMonth != null) rbMonth.IsChecked = true; // Why null at beg?
                IsMonthlySchedule = true;
                // Set Monthly tab
                if (tabs != null) tabs.SelectedIndex = 1;

                // Iterate collection and repopulate calendar UI.
                RepopulateMonthlyTasks(tasks);
            }
           
        }

        // 12/13/2010
        /// <summary>
        /// Methods helper to repopulate the weekly tasks.
        /// </summary>
        /// <param name="tasks">IEnumerable collection of <see cref="TaskBase"/>.</param>
        [CatchAllExceptionHandling(false, null, null)]
        private void RepopulateWeeklyTasks(IEnumerable<TaskBase> tasks)
        {
            foreach (var taskWeekly in tasks)
            {
                // Repopulate UI with proper settings.
                UpdateTaskSettingsOnUi((TaskWeekly) taskWeekly);
            }
        }

        // 12/13/2010
        /// <summary>
        /// Methods helper to repopulate the monthly tasks.
        /// </summary>
        /// <param name="tasks">IEnumerable collection of <see cref="TaskBase"/>.</param>
        [CatchAllExceptionHandling(false, null, null)]
        private void RepopulateMonthlyTasks(IEnumerable<TaskBase> tasks)
        {
            // Retrieve Constants of Enum to iterate
            foreach (var taskMonthly in tasks)
            {
                // Repopulate Calendar UI with proper settings.
                var monthly = (TaskMonthly)taskMonthly;

                // Retrieve the months enum.
                var months = monthly.Months;

                // Note - Its works!.
                var month = RetrieveNumericValueForEnum(months);

                UpdateCalendarDaysOnUi(monthly, month);
            }
            
        }

        // 12/13/2010
        /// <summary>
        /// Method helper which adds the days back to the calendar control the
        /// instance of <paramref name="taskMonthly"/>.
        /// </summary>
        /// <param name="taskMonthly">Instance of <see cref="taskMonthly"/>.</param>
        /// <param name="month">Month value</param>
        [CatchAllExceptionHandling(false, null, null)]
        private void UpdateCalendarDaysOnUi(TaskMonthly taskMonthly, int month)
        {
            // Retrieve the days collection.
            var daysSelected = taskMonthly.DaysOfMonth.ToList();

            // Set DateTime.
            UpdateTimeValues(taskMonthly);

            // Get DateTime.Now, to use the Year, etc.
            var dateTimeNow = DateTime.Now;
           
            // iterate the days collection for a specific month
            foreach (var day in daysSelected)
            {
                // Recreate days selected on calendar control.
                calendar.SelectedDates.Add(new DateTime(dateTimeNow.Year, month, day, taskMonthly.DateTime.Hour,
                                                        taskMonthly.DateTime.Minute, 0, DateTimeKind.Local));
            }
            
            
        }

        // 12/13/2010
        /// <summary>
        /// Method helper to extract the <see cref="MonthsOfTheYear"/> enum stored, and
        /// return the proper month value as integer.
        /// </summary>
        /// <param name="months">Instance of <see cref="MonthsOfTheYear"/>.</param>
        /// <returns>Numeric value as month.</returns>
        [CatchAllExceptionHandling(true, typeof(int), 1)]
        private static int RetrieveNumericValueForEnum(MonthsOfTheYear months)
        {
            if (((int)months & (int)MonthsOfTheYear.January) != 0)
            {
                return 1;
            }
            if (((int)months & (int)MonthsOfTheYear.February) != 0)
            {
                return 2;
            }
            if (((int)months & (int)MonthsOfTheYear.March) != 0)
            {
                return 3;
            }
            if (((int)months & (int)MonthsOfTheYear.April) != 0)
            {
                return 4;
            }
            if (((int)months & (int)MonthsOfTheYear.May) != 0)
            {
                return 5;
            }
            if (((int)months & (int)MonthsOfTheYear.June) != 0)
            {
                return 6;
            }
            if (((int)months & (int)MonthsOfTheYear.July) != 0)
            {
                return 7;
            }
            if (((int)months & (int)MonthsOfTheYear.August) != 0)
            {
                return 8;
            }

            if (((int)months & (int)MonthsOfTheYear.September) != 0)
            {
                return 9;
            }
            if (((int)months & (int)MonthsOfTheYear.October) != 0)
            {
                return 10;
            }
            if (((int)months & (int)MonthsOfTheYear.November) != 0)
            {
                return 11;
            }
            if (((int)months & (int)MonthsOfTheYear.December) != 0)
            {
                return 12;
            }

            throw new ArgumentOutOfRangeException();
        }

        // 12/12/2010
        /// <summary>
        /// Resets all CheckBox controls for days to false.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void ResetAllDaysToFalse()
        {
            if (cbSunday != null) cbSunday.IsChecked = false;
            if (cbMonday != null) cbMonday.IsChecked = false;
            if (cbTuesday != null) cbTuesday.IsChecked = false;
            if (cbWednesday != null) cbWednesday.IsChecked = false;
            if (cbThursday != null) cbThursday.IsChecked = false;
            if (cbFriday != null) cbFriday.IsChecked = false;
            if (cbSaturday != null) cbSaturday.IsChecked = false;
        }

        // 12/12/2010
        [CatchAllExceptionHandling(false, null, null)]
        private void UpdateTaskSettingsOnUi(TaskWeekly taskWeekly)
        {
            // Check if UI ready
            if (nudHour == null || nudMinutes == null)
                return;

            // Set DateTime.
            UpdateTimeValues(taskWeekly);

            // Switch on TaskDay enum.
            switch (taskWeekly.TaskDay)
            {
                case DaysOfTheWeek.Sunday:
                    cbSunday.IsChecked = true;
                    break;
                case DaysOfTheWeek.Monday:
                    cbMonday.IsChecked = true;
                    break;
                case DaysOfTheWeek.Tuesday:
                    cbTuesday.IsChecked = true;
                    break;
                case DaysOfTheWeek.Wednesday:
                    cbWednesday.IsChecked = true;
                    break;
                case DaysOfTheWeek.Thursday:
                    cbThursday.IsChecked = true;
                    break;
                case DaysOfTheWeek.Friday:
                    cbFriday.IsChecked = true;
                    break;
                case DaysOfTheWeek.Saturday:
                    cbSaturday.IsChecked = true;
                    break;
                case DaysOfTheWeek.AllDays:
                    // TODO: Need to add to combo
                    break;
                default:
                    throw new ArgumentOutOfRangeException();
            }

            
        }

        // 12/13/2010
        [CatchAllExceptionHandling(true, typeof(bool), false)]
        private bool UpdateTimeValues(TaskBase taskWeekly)
        {
            string amPmValue;
            nudHour.Value = ConvertHourTo12Format(taskWeekly.DateTime.Hour, out amPmValue); // Convert to 12-hour format.
            nudMinutes.Value = taskWeekly.DateTime.Minute;

            // Update ComboBox control for Am/Pm.
            cbAmPm.SelectedIndex = amPmValue.Equals("AM") ? 0 : 1;

            return true;
        }

        // 12/13/2010
        /// <summary>
        /// Occurs on the Unchecked state.
        /// </summary>
// ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void rbMonth_Unchecked(object sender, RoutedEventArgs e)
// ReSharper restore InconsistentNaming
        {
            // Change to Weekly-Tab
            tabs.SelectedIndex = 0;
        }

        // 12/13/2010
        /// <summary>
        /// Occurs on the Checked state.
        /// </summary>
// ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void rbMonth_Checked(object sender, RoutedEventArgs e)
// ReSharper restore InconsistentNaming
        {
            // Change to Monthly-Tab
            tabs.SelectedIndex = 1;
        }

       
    }
}
