﻿using System;
using System.ServiceModel;
using System.Threading.Tasks;
using System.Windows.Forms;
using Attributes;
using ImageCloudLock;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.Common.AttsInjectorHelper;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;

namespace ImageNexus.BenScharbach.ImageCloudLock.TaskActions
{
    /// <summary>
    ///  Does the 'Retrieve Root directories' in a Parallel task.
    /// </summary>
    public class DoRetrieveRootDirectoriesTask
    {
        #region Properties

        /// <summary>
        /// Owner of this window.
        /// </summary>
        public MainWindow MainWindowParent { get; set; }

        /// <summary>
        /// Gets the current <see cref="DuplexChannelFactory{TChannel}"/>.
        /// </summary>
        public DuplexChannelFactory<IclWebCommService> IclDuplexChannelFactory
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.IclDuplexChannelFactory : null;
            }
            set { if (MainWindowParent != null) MainWindowParent.IclDuplexChannelFactory = value; }
        }

        // NOTE: The 'Set' is required to keep PostSharp from crashing - bug.
        /// <summary>
        /// Gets the <see cref="ICommunicationObject"/> interface for the <see cref="IclDuplexChannelFactory"/>
        /// instance.  
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(ICommunicationObject), null)]
        public ICommunicationObject CommunicationObject
        {
            get { return IclDuplexChannelFactory; }
            private set { throw new InvalidOperationException(); }
        }

        #endregion

        #region Events

        /// <summary>
        /// Callback when task is completed.
        /// </summary>
        public event Action<Task<CloudRetrieveDirectories>> TaskCompleted;

        #endregion

        private class TaskObjectState
        {
            public RestoreCloudWindow RestoreCloudWindow;
            public object ObjectStateName2;
            public object ObjectStateName3;
        }

        public Task<CloudRetrieveDirectories> DoGetCloudRootDirectories(RestoreCloudWindow restoreCloudWindow, object paramName2 = null, object paramName3 = null)
        {
            var task = Task.Factory.StartNew(new Func<object, CloudRetrieveDirectories>(DoTaskActionCallback),
                                             new TaskObjectState
                                                 {
                                                     RestoreCloudWindow = restoreCloudWindow,
                                                     ObjectStateName2 = paramName2,
                                                     ObjectStateName3 = paramName3
                                                 }).ContinueWith(TaskIsCompletedCallback);

            return (Task<CloudRetrieveDirectories>) task;
        }

        /// <summary>
        /// Task Callback which triggers the <see cref="TaskCompleted"/> event.
        /// </summary>
        /// <param name="obj"></param>
        private void TaskIsCompletedCallback(Task<CloudRetrieveDirectories> obj)
        {
            if (TaskCompleted != null)
            {
                TaskCompleted(obj);
            }
        }

        private CloudRetrieveDirectories DoTaskActionCallback(object arg)
        {
            // Note: 1st param can be null, since it will be set by postsharp.
            return DoWCFCloudRetrieveRootDirectories(null);
        }
       
        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private static CloudRetrieveDirectories DoWCFCloudRetrieveRootDirectories(IclWebCommService currentChannel)
        {
            try
            {
                // Start the operation to retrieve root directories from cloud.
                CloudRetrieveDirectories retrieveDirectories = currentChannel.CloudRetrieveRootDirectories();

                return retrieveDirectories;
            }
            catch (Exception err)
            {
                MessageBox.Show("Test");
                throw;
            }
           
        }
    }
}