﻿using System;
using System.ServiceModel;
using System.Windows;
using System.Windows.Forms;
using System.Windows.Media;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsMisc;
using Attributes;
using ImageNexus.BenScharbach.Common.AttsInjectorHelper;
using ImageNexus.BenScharbach.Common.ProgressBarInterface;
using ImageNexus.BenScharbach.Common.ProgressBarWindow.Enums;
using ImageNexus.BenScharbach.Common.MEFLoggerInterfaces.Interfaces;
using ImageNexus.BenScharbach.ImageCloudLock.Enums;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;
using ImageNexus.BenScharbach.ImageCloudLock.ProgressBar;
using MessageBox = System.Windows.MessageBox;
using MessageBoxOptions = System.Windows.MessageBoxOptions;

namespace ImageNexus.BenScharbach.ImageCloudLock
{
    /// <summary>
    /// Interaction logic for UploadDirectoryWindow.xaml
    /// </summary>
    /*[MefLogger(LoggerTypeEnum.TraceLog, "ICLFrontEndConsole", LogPropertyCalls = true,
        LoggerVerbosity = LoggerVerbosityEnum.LogEntry | LoggerVerbosityEnum.LogExit | LoggerVerbosityEnum.LogExceptions,
        TraceLoggerOutputPath = @"ICL_FE_Logs\UploadDirectoryCallsOutput.txt", WpfLogWindowInstanceId = 1)]*/
    public partial class UploadDirectoryWindow : Window
    {
        // 12/6/2010 - Tracks if directory upload is currently in process.
        private bool _doingDirectoryUploadOp;

        // 9/4/2011
        private readonly ProgressBarService _progressBarService;

        #region Properties

        // 6/29/2011
        /// <summary>
        /// The MEF <see cref="MEFLoggerInterfaces.Interfaces.ILogger"/> required for using the <see cref="MefLoggerAttribute"/>.
        /// </summary>
        public ILogger Logger { get; set; }

        /// <summary>
        /// Owner of this window.
        /// </summary>
        public MainWindow MainWindowParent { get; private set; }

        // 12/6/2010
        /// <summary>
        /// Gets the current <see cref="DuplexChannelFactory{IclWebCommService}"/>.
        /// </summary>
        public DuplexChannelFactory<IclWebCommService> IclDuplexChannelFactory
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.IclDuplexChannelFactory : null;
            }
            set { if (MainWindowParent != null) MainWindowParent.IclDuplexChannelFactory = value; }
        }

        // 12/6/2010
        /// <summary>
        /// Gets the current communication channel.
        /// </summary>
        public IclWebCommService CurrentChannel { get; set; }

        // 12/6/2010
        // NOTE: The 'Set' is required to keep PostSharp from crashing - bug.
        /// <summary>
        /// Gets the <see cref="ICommunicationObject"/> interface for the <see cref="IclDuplexChannelFactory"/>
        /// instance.  
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public ICommunicationObject CommunicationObject
        {
            get { return IclDuplexChannelFactory; }
            private set { throw new InvalidOperationException(); }
        }

        #endregion

        /// <summary>
        /// Default constructor
        /// </summary>
        public UploadDirectoryWindow()
        {
            InitializeComponent();
        }

         /// <summary>
        /// Constructor, which sets the <see cref="MainWindow"/> parent.
        /// </summary>
        /// <param name="mainWindow">Instance of <see cref="MainWindow"/> as parent.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public UploadDirectoryWindow(MainWindow mainWindow)
        {
            InitializeComponent();

            MainWindowParent = mainWindow;

            // 2/19/2011
            // Check for Valid License.
            /*var license = new LicenseHelper();
            if (!license.Check(false))
                Application.Current.Shutdown();*/

            // 1/27/2012 - Set CloudService to Use in Wcf.
            mainWindow.SetCloudServiceToUse();

            // 3/1/2012
            _progressBarService = new ProgressBarService(); 
           
            // Subscribe to the 'TransferCompleted' event.
            MainWindowParent.IclWebDuplexCallbacks.OperationCompleted += IclWebDuplexCallbacks_TransferCompleted;
        }

        /// <summary>
        /// Occurs when the transfer is complete.
        /// </summary>
// ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void IclWebDuplexCallbacks_TransferCompleted(object sender, IOperationCompletedEventArgs e)
// ReSharper restore InconsistentNaming
        {
            // 12/6/2010 - Set flag
            _doingDirectoryUploadOp = false;
            // 12/6/2010 - Disable Upload button and clear text.
            btnUploadAllFiles.IsEnabled = false;
            txtDirectoryPath.Text = string.Empty;

            // 11/10/2010 - First check if cancelled.
            if (e.OperationCancelled)
            {
                txtResults2.Foreground = new SolidColorBrush(Colors.OrangeRed);
                txtResults2.Text = "Cancelled";
                return;
            }
           
            // Check result of transfer
            if (!e.OperationResult)
            {
                txtResults2.Foreground = new SolidColorBrush((e.FailedCount == e.TotalCount) ? Colors.Red : Colors.Orange);
                txtResults2.Text = (e.FailedCount == e.TotalCount) ? "Failed" : "Success with failures";
            }
            else
            {
                txtResults2.Foreground = new SolidColorBrush(Colors.Green);
                txtResults2.Text = "Success"; 
            }

            // 3/23/2012 - Open up the FilesOperationResults window
            var filesOpResults = new FilesOperationResultsWindow(MainWindowParent);
            filesOpResults.Show();
        }

        // 12/6/2010 - Updated to use the FolderDialog window rather than FilesDialog.
        /// <summary>
        /// Browses for a directory of files to upload.
        /// </summary>
// ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void btnBrowseDirectories_Click(object sender, RoutedEventArgs e)
// ReSharper restore InconsistentNaming
        {
            // Create OpenFileDialog
            // Set filter for file extension and default file extension
            //var dlg = new Microsoft.Win32.OpenFileDialog { DefaultExt = ".*", Filter = "All File Types (.*)|*.*" };
            // 12/6/2010 - Use 'Folder' dialog.
            var dlg = new FolderBrowserDialog
                          {Description = @"Select Directory to upload.", ShowNewFolderButton = false};

            // Display OpenFileDialog by calling ShowDialog method
            var result = dlg.ShowDialog();

            // Get the selected file name and display in a TextBox
            if (result != System.Windows.Forms.DialogResult.OK) return;

            // Set name into textbox.
            txtDirectoryPath.Text = dlg.SelectedPath;

            // 12/6/2010 - Enable the Upload button.
            btnUploadAllFiles.IsEnabled = true;

        }

        // 12/6/2010: Updated with new Attributes.
        /// <summary>
        /// Uploads ALL files contain in the 'txtDirectoryPath' textbox, to Amazon S3 service.
        /// </summary>
// ReSharper disable InconsistentNaming
        [ShowWaitCursor]
        [CatchAllExceptionHandling(false, null, null)]
        private void btnUploadAllFiles_Click(object sender, RoutedEventArgs e)
// ReSharper restore InconsistentNaming
        {
            // 12/6/2010 - Set flag
            _doingDirectoryUploadOp = true;

            // 2/11/2011 - Show WCF ProgressBar Window
            _progressBarService.Start(ProgressBarEventHandlerTypeEnum.BackupEventHandler, ProgressBarType.StandardDualBar);

            txtResults2.Foreground = new SolidColorBrush(Colors.Orange);
            txtResults2.Text = "Processing";

            // Note: 1st param can be null, since it will be set by postsharp.
            DoWCFTransferAllFilesToCloud(null); 
        }

        // 8/28/2011
        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.BeginCall)]
        private void DoWCFTransferAllFilesToCloud(IclWebCommService currentChannel)
        {
            currentChannel.TransferAllFilesToCloud(txtDirectoryPath.Text);
        }

        // 12/6/2010
        /// <summary>
        /// Occurs when user is attempting to close window.
        /// </summary>
// ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void UploadSingleDirectoryWindow_Closing(object sender, System.ComponentModel.CancelEventArgs e)
// ReSharper restore InconsistentNaming
        {
            if (!_doingDirectoryUploadOp) return;

            // Turn Off the progressBar from being top window.
            _progressBarService.SetTopMostWindow(false);

            var dialogResult = MessageBox.Show("This will cancel the current operation!  Do you want to continue?",
                                               "Cancel Operation",
                                               MessageBoxButton.OKCancel, MessageBoxImage.Exclamation,
                                               MessageBoxResult.Cancel,
                                               MessageBoxOptions.None);

            // Should operation cancel.
            switch (dialogResult)
            {
                case MessageBoxResult.OK:
                    e.Cancel = false;
                    // Cancel Operation.
                    _progressBarService.DoCancelOperation();
                    break;
                case MessageBoxResult.Cancel:
                    e.Cancel = true;
                    break;
            }
        }
    }
}
