﻿using System;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.Common.ProgressBarInterface;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;

namespace ImageNexus.BenScharbach.ImageCloudLock.WcfCallbacks
{
   
    ///<summary>
    /// ProgressBar Operation Completed EventHandler
    ///</summary>
    public delegate void OperationCompletedEventHandler(object sender, OperationCompletedEventArgs e);

    // 9/24/2011 - Updated to add 2 additional params; failedCount/totalCount
    ///<summary>
    /// ProgressBar Operation Completed EventArgs
    ///</summary>
    public class OperationCompletedEventArgs : EventArgs, IOperationCompletedEventArgs
    {
        /// <summary>
        /// Was operation succesful?
        /// </summary>
        public bool OperationResult { get; set; }

        /// <summary>
        /// Was operation cancelled?
        /// </summary>
        public bool OperationCancelled { get; set; }

        // 9/24/2011
        /// <summary>
        ///  The number of files which failed the current operation.
        /// </summary>
        public int FailedCount { get; set; }

        // 9/24/2011
        /// <summary>
        /// Total number of files processed for the current operation.
        /// </summary>
        public int TotalCount { get; set; }
    }

    /// <summary>
    /// The <see cref="IclWebDuplexCallbacks"/> is used to handle WCF type
    /// callbacks; essentially, the equivilent to events.
    /// </summary>
    public class IclWebDuplexCallbacks : IclWebCommServiceCallback
    {
        /// <summary>
        /// Occurs when operation to cloud service is complete.
        /// </summary>
        public event OperationCompletedEventHandler OperationCompleted;

        // 11/14/2010
        /// <summary>
        /// Occurs when the retrieve files to cloud service is complete.
        /// </summary>
        public event EventHandler RetrieveFilesCompleted;

        // 10/29/2010
        /// <summary>
        /// Occurs during operations to cloud service, which provides constant feedback
        /// via the given <see cref="OperationProgress"/> instance.
        /// </summary>
        public event EventHandler OperationFileProgres;

        /// <summary>
        /// Was operation succesful?
        /// </summary>
        public bool OperationResult { get; private set; }

        /// <summary>
        /// Was operation cancelled?
        /// </summary>
        public bool OperationCancelled { get; private set; }
       

        // 10/29/2010
        /// <summary>
        /// Gets the current file <see cref="OperationProgress"/> instance.
        /// </summary>
        public OperationProgress CurrentFileTransferProgress { get; private set; }

        // 9/24/2011 - Updated to add 2 additional params; failedCount/totalCount
        /// <summary>
        /// Captures the WCF Callback.
        /// </summary>
        /// <param name="result">Result of operation</param>
        /// <param name="cancelled">Set when asynchronous operation was cancelled by user.</param>
        /// <param name="failedCount">Set number of failed items in current operation</param>
        /// <param name="totalCount">Set total count of items processed in current operation</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void OperationCompletedEventHandler(bool result, bool cancelled, int failedCount, int totalCount)
        {
            // Store result locally.
            OperationResult = result;

            // 11/10/2010
            // Store Cancelled result
            OperationCancelled = cancelled;

            // 2/11/2011 - Updated to use the custom OperationCompleted eventArgs.
            if (OperationCompleted != null)
            {
                OperationCompleted(this, new OperationCompletedEventArgs
                                             {
                                                 OperationCancelled = cancelled,
                                                 OperationResult = result,
                                                 FailedCount = failedCount, // 9/24/2011
                                                 TotalCount = totalCount // 9/24/2011
                                             });
            }
        }

        [CatchAllExceptionHandling(false, null, null)]
        public IAsyncResult BeginOperationCompletedEventHandler(bool result, bool cancelled, int failedCount, int totalCount, AsyncCallback callback, object asyncState)
        {
            throw new NotImplementedException();
        }

        [CatchAllExceptionHandling(false, null, null)]
        public IAsyncResult BeginOperationCompletedEventHandler(bool result, bool cancelled, AsyncCallback callback, object asyncState)
        {
            throw new NotImplementedException();
        }

        [CatchAllExceptionHandling(false, null, null)]
        public void EndOperationCompletedEventHandler(IAsyncResult result)
        {
            throw new NotImplementedException();
        }

        // 10/29/2010
        /// <summary>
        /// Communicates the current progress bar's percent data.
        /// </summary>
        /// <param name="operationProgress"><see cref="OperationProgress"/> instance with ProgressBar data.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void OperationFileProgress(OperationProgress operationProgress)
        {
            // Store result locally.
            CurrentFileTransferProgress = operationProgress;

            if (OperationFileProgres != null)
                OperationFileProgres(this, EventArgs.Empty);
        }

        [CatchAllExceptionHandling(false, null, null)]
        public IAsyncResult BeginOperationFileProgress(OperationProgress operationProgress, AsyncCallback callback, object asyncState)
        {
            throw new NotImplementedException();
        }

        [CatchAllExceptionHandling(false, null, null)]
        public void EndOperationFileProgress(IAsyncResult result)
        {
            throw new NotImplementedException();
        }

        // 11/14/2010
        [CatchAllExceptionHandling(false, null, null)]
        public void OnEndRetrieveFiles()
        {
            if (RetrieveFilesCompleted != null)
                RetrieveFilesCompleted(this, EventArgs.Empty);
        }

        [CatchAllExceptionHandling(false, null, null)]
        public IAsyncResult BeginOnEndRetrieveFiles(AsyncCallback callback, object asyncState)
        {
            throw new NotImplementedException();
        }

        [CatchAllExceptionHandling(false, null, null)]
        public void EndOnEndRetrieveFiles(IAsyncResult result)
        {
            throw new NotImplementedException();
        }
    }
}