using System.Collections.Generic;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;

namespace ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.CheckedDataModel
{
    /// <summary>
    /// Interface to access the <see cref="CheckedCollection{T,TU}"/>.
    /// </summary>
    public interface ICheckedItems
    {
        /// <summary>
        /// Gets if checked collection has been updated by user.
        /// </summary>
        bool IsDirty { get; }

        /// <summary>
        /// CheckedCollection's unique key.
        /// </summary>
        int CheckedCollectionUniqueKey { get; }

        /// <summary>
        /// Clears all internal collections within the <see cref="CheckedCollection{T,TU}"/>.
        /// </summary>
        void ClearAllCheckedItems();

        /// <summary>
        /// Retrieves a collection of FileCheckedItem instances which have the
        /// 'IsChecked' flag set to true.  
        /// </summary>
        /// <param name="checkedItems">(OUT) Collection of files set with 'IsChecked' flag to true.</param>
        void GetListOfCheckedItems(out List<IFileCheckedItem<IFileItem>> checkedItems);

        /// <summary>
        /// Retrieves a collection of DirectoryCheckedItem instances which have the
        /// 'IsChecked' flag set to true.  
        /// </summary>
        /// <param name="checkedItems">(OUT) Collection of directories set with 'IsChecked' flag to true.</param>
        void GetListOfCheckedItems(out List<IDirectoryCheckedItem<IDirectoryItem>> checkedItems);

        /// <summary>
        /// Clears the internal collection and updates with given <paramref name="checkedItems"/>
        /// collection.
        /// </summary>
        /// <param name="checkedItems">Collection of files which are marked as 'IsChecked'.</param>
        void SetCheckedItems(List<IFileCheckedItem<IFileItem>> checkedItems);

        /// <summary>
        /// Clears the internal collection and updates with given <paramref name="checkedItems"/>
        /// collection.
        /// </summary>
        /// <param name="checkedItems">Collection of directories which are marked as 'IsChecked'.</param>
        void SetCheckedItems(List<IDirectoryCheckedItem<IDirectoryItem>> checkedItems);
    }
}