﻿using System;
using System.Globalization;
using System.Windows.Data;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;

namespace ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.Converters
{
    // 11/3/2010
    /// <summary>
    /// The <see cref="FileSizeConverter"/> converts the given fileSize to either KB, MB, GB or leaves as bytes if
    /// less than 1 KB.
    /// </summary>
    [ValueConversion(typeof(long), typeof(string))]
    public class FileSizeConverter : IValueConverter
    {
        /// <summary>
        /// Converts a value. 
        /// </summary>
        /// <returns>
        /// A converted value. If the method returns null, the valid null value is used.
        /// </returns>
        /// <param name="value">The value produced by the binding source.</param><param name="targetType">The type of the binding target property.</param><param name="parameter">The converter parameter to use.</param><param name="culture">The culture to use in the converter.</param>
        [CatchAllExceptionHandling(true, typeof(object), null)]
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            // retrieve given value
            var fileSize = (long) value;

            // constants
            const long kiloByte = 1024;
            const long megaByte = kiloByte * kiloByte; // 1024^2
            const long gigaByte = kiloByte * kiloByte * kiloByte; // 1024^3
            const long teraByte = megaByte*megaByte; // 1024^4

            // conversion data
            long convertedSize;
            var fileSizeOutput = string.Empty;

            // Note: Using KiloByte as 1024 bytes.
            // Convert to proper string with suffix 'Bytes, 'KB', 'MB', or 'GB'.
            if (fileSize < kiloByte)
            {
                fileSizeOutput = fileSize + " bytes";
            }
            else if (fileSize >= kiloByte && fileSize < megaByte)
            {
                convertedSize = fileSize/kiloByte;
                fileSizeOutput = convertedSize + " kB";
            }
            else if (fileSize >= megaByte && fileSize < gigaByte)
            {
                convertedSize = fileSize/megaByte;
                fileSizeOutput = convertedSize + " MB";
            }
            else if (fileSize >= gigaByte && fileSize < teraByte)
            {
                convertedSize = fileSize/gigaByte;
                fileSizeOutput = convertedSize + " GB";
            }
            else if (fileSize >= teraByte)
            {
                convertedSize = fileSize/teraByte;
                fileSizeOutput = convertedSize + " TB";
            }

            return fileSizeOutput.ToString(culture);

        }

        /// <summary>
        /// Converts a value. 
        /// </summary>
        /// <returns>
        /// A converted value. If the method returns null, the valid null value is used.
        /// </returns>
        /// <param name="value">The value that is produced by the binding target.</param><param name="targetType">The type to convert to.</param><param name="parameter">The converter parameter to use.</param><param name="culture">The culture to use in the converter.</param>
        [CatchAllExceptionHandling(false, typeof(object), null)]
        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
}
