﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.EventArgs;

// 7/9/2011
/*[assembly: MefLogger(LoggerTypeEnum.TraceLog, "ICLFrontEndConsole", LogPropertyCalls = true,
        LoggerVerbosity = LoggerVerbosityEnum.LogEntry | LoggerVerbosityEnum.LogExceptions,
        TraceLoggerOutputPath = @"ICL_Trace_Logs\LoggerTreeViewControl.txt", WpfLogWindowInstanceId = 1,
        AttributeTargetTypes = "ImageNexusWpfTreeViewControl.*",
        AttributeTargetTypeAttributes = MulticastAttributes.Public)]*/

namespace ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl
{

    /// <summary>
    /// Interaction logic for TreeViewUserControl.xaml
    /// </summary>
    public partial class TreeViewUserControl : UserControl
    {

        #region Events

        // 3/19/2012
        /// <summary>
        /// Occurs when the splitter is moved.
        /// </summary>
        public event EventHandler<SplitterEventArgs> SplitterMoved; 

        // 10/26/2010
        /// <summary>
        /// Occurs when some directory checkBox is checked.
        /// </summary>
        public event RoutedEventHandler DirectoryCheckboxChecked;

        // 10/26/2010
        /// <summary>
        /// Occurs when some directory checkBox is unchecked.
        /// </summary>
        public event RoutedEventHandler DirectoryCheckboxUnChecked;

        // 12/3/2010
        /// <summary>
        /// Occurs when some directory checkBox is clicked by user.
        /// </summary>
        public event RoutedEventHandler DirectoryCheckboxClicked;

        // 10/9/2011
        /// <summary>
        /// Occurs when some file checkBox is checked.
        /// </summary>
        public event RoutedEventHandler FileCheckboxChecked;

        // 10/9/2011
        /// <summary>
        /// Occurs when some file checkBox is unchecked.
        /// </summary>
        public event RoutedEventHandler FileCheckboxUnChecked;

        // 10/9/2011
        /// <summary>
        /// Occurs when some file checkBox is clicked by user.
        /// </summary>
        public event RoutedEventHandler FileCheckboxClicked;

        // 12/8/2010
        /// <summary>
        /// Occurs when some directory ScheduleNumber control is updated by user.
        /// </summary>
        public event EventHandler DirectoryScheduleNumberUpdated;

        // 12/8/2010
        /// <summary>
        /// Occurs when some listView ScheduleNumber control is updated by user.
        /// </summary>
        public event EventHandler ListViewScheduleNumberUpdated;

        // 5/14/2011
        /// <summary>
        /// Occurs when the user selects the 'Refresh' context menu item.
        /// </summary>
        public event EventHandler DirectoryRefreshClicked;

        #endregion

        #region Properties

        // 11/7/2010  - Expose the Directory ListView
        /// <summary>
        /// Gets the current directory <see cref="ListView"/> WPF control.
        /// </summary>
        public ListView DirectoryListView
        {
            get { return directoryListView; }
        }

        // 11/7/2010 - Expost the Details TreeView
        /// <summary>
        /// Gets the current details <see cref="TreeView"/> WPF control.
        /// </summary>
        public TreeView DetailsTreeView
        {
            get { return detailsTreeView; }
        }

        // 10/24/2010; 11/8/2010 - Updated to 'string' drive name.
        /// <summary>
        /// Gets the current drive's name selected in the listview.
        /// </summary>
        public string CurrentDriveSelected { get; set; }

        // 10/25/2010; 11/8/2010 - Updated to 'string' drive name.
        /// <summary>
        /// Gets the prior drive's name selected in the listview.
        /// </summary>
        public string PriorDriveSelected { get; set; }

        // 3/20/2012
        /// <summary>
        /// Gets or sets the current TreeView's container Width.
        /// </summary>
        public GridLength TreeViewContainerWidth
        {
            get { return treeGrid.ColumnDefinitions[0].Width; }
            set { treeGrid.ColumnDefinitions[0].Width = value; }
        }

        // 3/20/2012
        /// <summary>
        /// Gets or sets the current TreeView's container MaxWidth.
        /// </summary>
        /// <remarks>>
        /// If a percent is set in <see cref="TreeViewContainerMaxWidthPercent"/>, it will
        /// then override any use of this property.
        /// </remarks>
        public double TreeViewContainerMaxWidth
        {
            get { return treeGrid.ColumnDefinitions[0].MaxWidth; }
            set { treeGrid.ColumnDefinitions[0].MaxWidth = value; }
        }

        // 3/21/2012
        /// <summary>
        /// Gets or sets the current MaxWidth percent of control size to use.
        /// </summary>
        /// <remarks>
        /// Settings any value other than zero implies an override 
        /// of setting the <see cref="TreeViewContainerMaxWidth"/> value.
        /// </remarks>
        public int TreeViewContainerMaxWidthPercent { get; set; }

        #endregion

        /// <summary>
        /// Constructor
        /// </summary>
        public TreeViewUserControl()
        {
            InitializeComponent();
        }

        // 11/16/2010
        /// <summary>
        /// Used to save user settings for the TreeView control instance.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public void SaveUserSettings()
        {
            Properties.Settings.Default.Save();
        }
       
        // 10/26/2010
        /// <summary>
        /// Captures the checkBox 'Checked' event.
        /// </summary>
// ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void CheckBox_Checked(object sender, RoutedEventArgs e)
// ReSharper restore InconsistentNaming
        {
            if (DirectoryCheckboxChecked != null)
                DirectoryCheckboxChecked(sender, e);
        }
        // 10/26/2010
        /// <summary>
        /// Captures the checkBox 'UnChecked' event.
        /// </summary>
// ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void CheckBox_Unchecked(object sender, RoutedEventArgs e)
// ReSharper restore InconsistentNaming
        {
            if (DirectoryCheckboxUnChecked != null)
                DirectoryCheckboxUnChecked(sender, e);
        }

        // 12/3/2010
        /// <summary>
        /// Captures the checkbox 'Click' event.
        /// </summary>
// ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void CheckBox_Click(object sender, RoutedEventArgs e)
// ReSharper restore InconsistentNaming
        {
            if (DirectoryCheckboxClicked != null)
                DirectoryCheckboxClicked(sender, e);
        }

        // 10/9/2011
        /// <summary>
        /// Captures the checkBox 'Checked' event.
        /// </summary>
        private void FilesCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            if (FileCheckboxChecked != null)
                FileCheckboxChecked(sender, e);
        }

        // 10/9/2011
        /// <summary>
        /// Captures the checkBox 'UnChecked' event.
        /// </summary>
        private void FilesCheckBox_Unchecked(object sender, RoutedEventArgs e)
        {
            if (FileCheckboxUnChecked != null)
                FileCheckboxUnChecked(sender, e);
        }

        // 10/9/2011
        /// <summary>
        /// Captures the checkbox 'Click' event.
        /// </summary>
        private void FilesCheckBox_Click(object sender, RoutedEventArgs e)
        {
            if (FileCheckboxClicked != null)
                FileCheckboxClicked(sender, e);
        }

        // 3/19/2012
        /// <summary>
        /// Occurs when the treeView splitter is moved.
        /// </summary>
        private void treeViewSplitter_DragDelta(object sender, System.Windows.Controls.Primitives.DragDeltaEventArgs e)
        {
            // check if at max width for treeview
            if (TreeViewContainerWidth.Value >= TreeViewContainerMaxWidth)
            {
                treeViewSplitter.CancelDrag();
                return;
            }

            if (SplitterMoved != null)
                SplitterMoved(sender, new SplitterEventArgs() { TreeViewSplitterHorizontalChange = e.HorizontalChange});
           
        }

        // 12/7/2010
        /// <summary>
        /// Used to show/hide the comboBoxes within the ItemTemplates.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public void SetItemTemplatesType(bool showComboBoxes)
        {
            var controlDictStyle = showComboBoxes
                                       ? "/ImageNexusWpfTreeViewControl;component/ShowComboBoxesStyle.xaml"
                                       : "/ImageNexusWpfTreeViewControl;component/HideComboBoxesStyle.xaml";

            // Update the 'TreeView' control.
            SetResourceToUse(controlDictStyle, tvUserControl.Resources);
        }

        // 12/7/2010
        /// <summary>
        /// Helper method used to set the proper 'Blend' dictionary to use as the resource.
        /// </summary>
        /// <param name="controlDictStyle">Name of resource to load; for example, "/ProgressBarUserControl;component/MiniSingleProgressBarDictionary.xaml"</param>
        /// <param name="resourceDictionary">Resource to load dictionary into.</param>
        [CatchAllExceptionHandling(false, null, null)]
        private static void SetResourceToUse(string controlDictStyle, ResourceDictionary resourceDictionary)
        {
            var controlDictUri = new Uri(controlDictStyle, UriKind.Relative);
            var controlDict = Application.LoadComponent(controlDictUri) as ResourceDictionary;

            // Update the WPFProgressBar control's Resources with the proper dictionary.
            resourceDictionary.MergedDictionaries.Clear();
            resourceDictionary.MergedDictionaries.Add(controlDict);

            //Application.Current.Resources.MergedDictionaries.Clear();
            //Application.Current.Resources.MergedDictionaries.Add(langDict);
        }


        // 12/7/2010 - Helper method to retrieve the ContextMenu.
        [CatchAllExceptionHandling(false, null, null)]
        private static ContextMenu GetContextMenuControl(object sender)
        {
            //var menuClickPoint = sender.TranslatePoint(new Point(0, 0), listView);

            // get the first potential treeView object that was hit   
            //var obj = listView.InputHitTest(menuClickPoint) as DependencyObject;

            // cycle up the tree until you hit the parent NumUpDown control. 
            var obj = sender as DependencyObject;
            while (obj != null && !(obj is ContextMenu))
            {
                obj = VisualTreeHelper.GetParent(obj);
            }

            return obj as ContextMenu;
        }

        // 12/8/2010
        /// <summary>
        /// Occurs when user selects some comboBox number in the drop-down context menu.
        /// </summary>
// ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void cbtvScheduleNumber_SelectionChanged(object sender, SelectionChangedEventArgs e)
// ReSharper restore InconsistentNaming
        {
            if (DirectoryScheduleNumberUpdated != null)
                DirectoryScheduleNumberUpdated(sender, e);

            // 12/14/2010 - Close MenuContext.
            var contextMenu = GetContextMenuControl(sender);
            if (contextMenu != null)
            {
                contextMenu.IsOpen = false;
            }
        }

        // 5/15/2011
        private void menuItemRefresh_Click(object sender, RoutedEventArgs e)
        {
            if (DirectoryRefreshClicked != null)
                DirectoryRefreshClicked(sender, e);

            //Console.WriteLine(string.Format("Refresh Event: OriginalSource = {0}, RoutedEvent = {1}, Source = {2}",
            // e.OriginalSource, e.RoutedEvent.Name, e.Source));

            // 12/14/2010 - Close MenuContext.
            var contextMenu = GetContextMenuControl(sender);
            if (contextMenu != null)
            {
                contextMenu.IsOpen = false;
            }
        }

        // 12/14/2010
        // ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void cblvScheduleNumber_SelectionChanged(object sender, SelectionChangedEventArgs e)
        // ReSharper restore InconsistentNaming
        {
            // 12/14/2010 - Close MenuContext.
            var contextMenu = GetContextMenuControl(sender);
            if (contextMenu != null)
            {
                contextMenu.IsOpen = false;
            }
        }

        // 12/14/2010
        /// <summary>
        /// Call to lock or unlock the Drives Ui.
        /// </summary>
        /// <param name="isEnabled">Lock or Unlock Ui.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void LockUnLockDrivesUi(bool isEnabled)
        {
            if (directoryListView != null) 
                directoryListView.IsEnabled = isEnabled;
        }

        // 12/14/2010
        /// <summary>
        /// Call to lock or unlock the TreeView Ui.
        /// </summary>
        /// <param name="isEnabled">Lock or Unlock Ui.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void LockUnLockTreeViewUi(bool isEnabled)
        {
            if (detailsTreeView != null)
                detailsTreeView.IsEnabled = isEnabled;
        }

        // 12/14/2010
        /// <summary>
        /// Call to lock or unlock the TreeView-Details Ui.
        /// </summary>
        /// <param name="isEnabled">Lock or Unlock Ui.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void LockUnLockTreeViewDetailsUi(bool isEnabled)
        {
            if (detailsListView != null)
                detailsListView.IsEnabled = isEnabled;
        }

        // 3/21/2012
        /// <summary>
        /// Occurs when the control size changes.
        /// </summary>
        private void tvUserControl_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            // If 0 percent, then return an do nothing.
            if (TreeViewContainerMaxWidthPercent == 0)
            {
                return;
            }

            // Calculate new MaxWidth for TreeView column 0
            var percentToUse = (float) (TreeViewContainerMaxWidthPercent/100.0f);
            var maxWidthToUse = (ActualWidth*percentToUse);
            treeGrid.ColumnDefinitions[0].MaxWidth = maxWidthToUse;
        }
    }
}
