﻿using System.Windows.Controls;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;

namespace ProgressBarUserControl
{
    /// <summary>
    /// Interaction logic for WpfProgressBarUserControl.xaml
    /// </summary>
    public partial class WpfProgressBarUserControl : UserControl
    {
        // Progress bar 1
        private readonly UpdateProgressBar _monitorProgressBar1;
        // Progress bar 2
        private readonly UpdateProgressBar _monitorProgressBar2;

        // Start threads?
        private bool _startProgressBar1;
        private bool _startProgressBar2;

        #region Properties

        /// <summary>
        /// Start or stop <see cref="_monitorProgressBar1"/> monitoring thread?
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(bool), false)]
        public bool StartProgressBar1
        {
            get { return _startProgressBar1; }
            set
            {
                _startProgressBar1 = value;

                if (value)
                {
                    if (_monitorProgressBar1 != null) _monitorProgressBar1.StartProgressBarThread();
                    return;
                }

                if (_monitorProgressBar1 != null) _monitorProgressBar1.StopProgressBarThread();
            }
        }

        /// <summary>
        /// Start or stop <see cref="_monitorProgressBar2"/> monitoring thread?
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(bool), false)]
        public bool StartProgressBar2
        {
            get { return _startProgressBar2; }
            set
            {
                _startProgressBar2 = value;

                if (value)
                {
                    if (_monitorProgressBar2 != null) _monitorProgressBar2.StartProgressBarThread();
                    return;
                }

                if (_monitorProgressBar2 != null) _monitorProgressBar2.StopProgressBarThread();
            }
        }

        /// <summary>
        /// Updates the Main-Description label for progress bar 1.
        /// </summary>
// ReSharper disable InconsistentNaming
        public string ProgressBar1_MainDescription
// ReSharper restore InconsistentNaming
        {
            get
            {
                return lblProgressBar1_MainDescription != null
                           ? (lblProgressBar1_MainDescription.Content != null)
                                 ? lblProgressBar1_MainDescription.Content.ToString()
                                 : string.Empty
                           : string.Empty;
            }
            set 
            { 
                if (lblProgressBar1_MainDescription != null) lblProgressBar1_MainDescription.Content = value; 
            }
        }

        /// <summary>
        /// Updates the Main-Description label for progress bar 2.
        /// </summary>
// ReSharper disable InconsistentNaming
        public string ProgressBar2_MainDescription
// ReSharper restore InconsistentNaming
        {
            get
            {
                return lblProgressBar2_MainDescription != null
                         ? (lblProgressBar2_MainDescription.Content != null)
                               ? lblProgressBar2_MainDescription.Content.ToString()
                               : string.Empty
                         : string.Empty;
            }
            set { if (lblProgressBar2_MainDescription != null) lblProgressBar2_MainDescription.Content = value; }
        }

        /// <summary>
        /// Updates the Detail-Data label for progress bar 1.
        /// </summary>
// ReSharper disable InconsistentNaming
        public string ProgressBar1_DetailData
// ReSharper restore InconsistentNaming
        {
            get
            {
                return lblProgressBar1_DetailData != null
                       ? (lblProgressBar1_DetailData.Content != null)
                             ? lblProgressBar1_DetailData.Content.ToString()
                             : string.Empty
                       : string.Empty;
            }
            set { if (lblProgressBar1_DetailData != null) lblProgressBar1_DetailData.Content = value; }
        }

        /// <summary>
        /// Updates the Detail-Data label for progress bar 1.
        /// </summary>
// ReSharper disable InconsistentNaming
        public string ProgressBar2_DetailData
// ReSharper restore InconsistentNaming
        {
            get
            {
                return lblProgressBar2_DetailData != null
                     ? (lblProgressBar2_DetailData.Content != null)
                           ? lblProgressBar2_DetailData.Content.ToString()
                           : string.Empty
                     : string.Empty;
            }
            set { if (lblProgressBar2_DetailData != null) lblProgressBar2_DetailData.Content = value; }
        }

        #endregion

        /// <summary>
        /// Default Constructor.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public WpfProgressBarUserControl()
        {
            InitializeComponent();

            // Create instance of overall progressBar.
            _monitorProgressBar2 = new UpdateProgressBar(progressBar1);

            // Create instance of upload progressBar.
            _monitorProgressBar1 = new UpdateProgressBar(progressBar2);
        }

        /// <summary>
        /// Updates <see cref="_monitorProgressBar1"/> percentage completed.
        /// </summary>
        /// <param name="percentDone">Enter value 0-100 as percent completed.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void UpdatePercentCompleteForProgressBar1(int percentDone)
        {
            // Update PercentCompleted property
            if (_monitorProgressBar1 != null) _monitorProgressBar1.PercentCompleted = percentDone;
        }

        /// <summary>
        /// Updates <see cref="_monitorProgressBar2"/> percentage completed.
        /// </summary>
        /// <param name="percentDone">Enter value 0-100 as percent completed.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void UpdatePercentCompleteForProgressBar2(int percentDone)
        {
            // Update PercentCompleted property
            if (_monitorProgressBar2 != null) _monitorProgressBar2.PercentCompleted = percentDone;
        }
    }
}
