﻿using System;
using System.Linq;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using AttsInjectorHelper;
using AttsInjectorHelper.Enums;
using MEFLoggerInterfaces.Enums;
using PostSharp.Extensibility;

// 7/9/2011
[assembly: MefLogger(LoggerTypeEnum.TraceLog, "ICLFrontEndConsole", LogPropertyCalls = true,
        LoggerVerbosity = LoggerVerbosityEnum.LogEntry | LoggerVerbosityEnum.LogExceptions,
        TraceLoggerOutputPath = @"ICL_Trace_Logs\LoggerReflectionHelper.txt", WpfLogWindowInstanceId = 1,
        AttributeTargetTypes = "ReflectionHelper.*",
        AttributeTargetTypeAttributes = MulticastAttributes.Public)]

namespace ReflectionHelper
{
    // 12/8/2010
    /// <summary>
    /// The <see cref="ReflectionHelperTool"/> is used to update properties via reflection for any target object.
    /// </summary>
    public static class ReflectionHelperTool
    {
        // 10/23/2010
        /// <summary>
        /// Sets the given <paramref name="propertyValue"/> into the given <paramref name="target"/> object, which
        /// contains the given <paramref name="propertyName"/> type, using reflection.
        /// </summary>
        /// <param name="target">object target which has property to update</param>
        /// <param name="propertyName">Property name to update</param>
        /// <param name="propertyValue">Value to set</param>
        [CatchAllExceptionHandling(false, null, null)]
        public static void SetPropertyValue(Object target, string propertyName, object propertyValue)
        {
            var type = target.GetType();
            var prop = type.GetProperty(propertyName);
            prop.SetValue(target, propertyValue, null);
        }

        // 12/8/2010
        /// <summary>
        /// Gets a given value from the given <paramref name="target"/> object, which
        /// contains the given <paramref name="propertyName"/> type, using reflection.
        /// </summary>
        /// <param name="target">object target which has property value to return</param>
        /// <param name="propertyName">Property name to retrieve value from</param>
        /// <returns>Value to return</returns>
        [CatchAllExceptionHandling(true, typeof(object), null)]
        public static object GetPropertyValue(Object target, string propertyName)
        {
            var type = target.GetType();
            var prop = type.GetProperty(propertyName);

            return prop.GetValue(target, null);
        }

        // 10/23/2010
        /// <summary>
        /// Checks if given <paramref name="propertyName"/> exist for given <paramref name="target"/> type, using reflection.
        /// </summary>
        /// <param name="target">object target which has property to update</param>
        /// <param name="propertyName">Property name</param>
        /// <returns>True/false if name exist.</returns>
        [CatchAllExceptionHandling(true, typeof(bool), false)]
        public static bool DoesPropertyNameExist(Object target, string propertyName)
        {
            // get all public properties of 'target' type
            var type = target.GetType();
            var propertyInfos = type.GetProperties();

            // check if name exist
            return propertyInfos.Any(propertyInfo => propertyInfo.Name == propertyName);
        }
    }
}
